# Aufbau eines PlatformIO-Projektes

Wie auch schon bei den vorherigen Praktika bekommen Sie wieder vorkonfigurierte Projekte.

Das Projekt lässt sich auch mit dem PlatformIO-Assistenten erstellen, so ist es aber einfacher.

Im Gegensatz zu unseren anderen Praktika gibt es hier aber einige Änderungen. Sie werden einige Dateien vermissen.

PlatformIO setzt auf ein anderes Buildsystem. Sie kennen bereits die ``CMakeLists.txt``. Dieses wird vom Buildsystem ``cmake`` verwendet.

PlatformIO verwendet _SCons_ und jede Menge Feenstaub (ﾉ◕ヮ◕)ﾉ*:･ﾟ✧.

## Projektstruktur

Wenn Sie in das Quellcode-Archiv schauen, werden Sie die folgenden Dateien sehen:

- ``platformio.ini``: Die Konfigurationsdatei für ein PlatformIO-Projekt (siehe nächster Abschnitt)
- ``.editorconfig``: Ist bekannt, konfiguriert wie der Code formatiert wird
- ``compiledb.py``: Wird benötigt, damit ``clangd`` richtig funktioniert
- ``src/*``: Quelltexte und Header

_Laden Sie nun das Projekt in Visual Studio Code_. PlatformIO wird es automatisch erkennen und konfigurieren. Dies dauert einen Moment.

### ``platformio.ini``

Diese Datei entspricht mehr oder weniger der ``CMakeLists.txt``.

Normalerweise müssen Sie hier aber nichts ändern.

Neue ``cpp``-Dateien werden automatisch erkannt, ohne das diese explizit aufgelistet werden müssen.

```ini
[env:megaatmega2560]
platform = atmelavr
board = megaatmega2560
framework = arduino
build_flags =
    -I${platformio.packages_dir}/toolchain-atmelavr/avr/include
lib_deps = mike-matera/ArduinoSTL@^1.3.3
extra_scripts = post:compiledb.py
```

Die Meisten Optionen können grafisch bei "PlatformIO (Haus in der Toolbar) -> Projects -> Configure" eingestellt werden.

Bedeutung der Zeilen:

- ``[env:megaatmega2560]``: Name der Sektion. Muss mit ``env:`` anfangen, der Rest ist egal
- ``platform``: Zielplatform
- ``board``: Muss dem zu programmierenden Board entsprechen, sonst funktioniert es nicht.
- ``framework``: Stellt die Arduino-Bibliothek zur Verfügung
- ``build_flags``: Eigentlich nicht nötig, aber sonst findet ``clangd`` einige Header nicht
- ``lib_deps``: Zusätzliche Abhängigkeiten. In unserem Fall die C++ Standardbibilothek (``ArduinoSTL``)
- ``extra_scripts``: ``post:compiledb.py`` stellt sicher, dass ``clangd`` richtig funktioniert

### Automatisch generierte Dateien

Sobald Sie das Projekt laden, werden von PlatformIO die folgenden Dateien und Ordner automatisch generiert:

- ``.pio``: Entspricht ``build`` aus CMake-Projekten
- ``.vscode``: Dateien, damit kompilieren usw. funktioniert. Auch aus CMake-Projekten bekannt.
- ``.gitignore``: Irrelevant. Dateien, die nicht versioniert werden soll, wenn man eine Versionsverwaltung (Git) verwendet.

An diesen sollten Sie also nichts ändern.
