# Den Arduino programmieren

Wenn Sie ein PlatformIO-Projekt laden, werden unten in der Statusleiste neue Symbole angezeigt.

![](Images/Toolbar.png)

F5 funktioniert mit PlatformIO leider nicht zuverlässig, da der Debugger nicht richtig konfiguriert ist. Klicken Sie daher stattdessen immer auf _Upload_ in der Statusleiste.

- Home: Öffnet die PlatformIO-Startseite
- Kompilieren: Baut das Projekt
- Upload: Baut das Projekt und flasht es anschließend auf den Arduino
- Aufräumen: Entfernt alle Zwischendateien
- Serielle Konsole: Wenn der Arduino richtig konfiguriert ist, können hier Ausgaben von ``std::cout`` gesehen werden
- Terminal: Öffnet das integrierte Terminal

## Syntaxhervorhebung

Damit die Syntaxhervorhebung funktioniert, muss die ``compile_commands.json`` manuell generiert werden.

Im Gegensatz zu ``cmake`` funktioniert dies nicht automatisch.

Führen sie dazu im integrierten Terminal ``pio run -t compiledb`` aus.

Wenn Sie nun einmalig kompilieren und die ``main.cpp`` bearbeiten, sollte alles funktionieren, inklusive Syntaxhervorhebung, usw.

Wenn Sie eine neue Datei hinzufügen, müssen Sie unter Umständen den Befehl erneut ausführen.

## Änderungen gegenüber _normalem_ C++

Prinzipiell lassen sich große Teile der Standardbibliothek wie gewohnt verwenden. ``std::cout`` ist auf die serielle Konsole umgebogen.

Da der Arduino ein eingebettetes System ist, müssen aber einige Dinge beachtet werden.

Schauen wir uns dieses Beispielprogramm an, dass die On-Board LED blinken lässt:

```cpp
#include <Arduino.h>
#include <ArduinoSTL.h>

void setup() {
    // Den Pin der On-Board LED auf Ausgabe stellen
    pinMode(LED_BUILTIN, OUTPUT);
}

void loop() {
    // Pin der On-Board LED auf 5V setzen (High)
    digitalWrite(LED_BUILTIN, HIGH);

    // 1s schlafen
    delay(1000);

    // Pin der On-Board LED auf 0V setzen (Low)
    digitalWrite(LED_BUILTIN, LOW);

    // 1s schlafen
    delay(1000);
}
```

Die erste Auffälligkeit ist: Es gibt kein ``int main()`` mehr, stattdessen wird am Anfang ``setup()`` aufgerufen und anschließend in einer Endlosschleife ``loop()``.

Die vom Arduino ausgeführte ``main`` sieht also ungefähr so aus:

```cpp
int main() {
    // Hardware initialisieren

    setup();

    for (;;) {
        loop();
    }

    return 0;
}
```

Der Header ``#include <Arduino.h>`` stellt Arduino-spezifische Funktionen zur Verfügung.

Der Header ``#include <ArduinoSTL.h>`` ist eine Portierung der uClibc++. Dies ist eine C++ Standardbibliothek für eingebettete Systeme. Beim Compiler ``avr-g++`` ist normalerweise keine Standardbibliothek dabei, d.h. es können nur Funktionalitäten verwendet werden, die keine Header benötigen (beispielsweise Klassen). Für Dinge wie ``std::vector`` und ``std::string`` benötigen wir aber eine Standardbibliothek.

Die restlichen Programmfunktionalitäten sind im Quelltext kommentiert.

### Speicherverwaltung

Die Faustregel ist: Fordern Sie Speicher nur in ``setup()`` an und verwenden Sie dann den angeforderten Speicher in ``loop()``. Nichts ist ärgerlicher, als wenn Ihr Programm irgendwann abstürzt, weil der Speicher alle ist.

Der Arduino hat im Gegensatz zu Ihrem Computer sehr wenig Arbeitsspeicher (8 KB).

Daher müssen Sie sparsam sein und sollten möglichst auf dynamischen Speicher verzichten. Wenn Sie ständig Speicher anfordern und freigeben [fragmentiert das den Heap](https://cpp4arduino.com/2018/11/06/what-is-heap-fragmentation.html) und irgendwann ist es nicht mehr möglich Speicher anzufordern, obwohl noch etwas frei ist. Zudem kann das Anfordern von dynamischen Speicher unterschiedlich lange dauern kann und dies ist natürlich dramatisch ist bei einem System, wo man eine Nanosekunden-Genauigkeit erreicht werden kann.

Dynamischer Speicher ist alles, was auf dem _Heap_ erzeugt wird.

Auch ``std::string`` und ``std::vector`` verwenden dynamischen Speicher.

Solange wir aber den Heapspeicher in ``setup()`` anfordern und dann in ``loop()`` nur den angeforderten Speicher beschreiben (und nicht ständig ``push_back`` machen), ist alles in Ordnung.
