# Praktikum 10: Objektorientierte Programmierung auf dem Arduino

Sofern man es nicht übertreibt, kann selbst auf eingebetteten Systemen wie dem Arduino die Objektorientierte Programmierung die Lesbarkeit erhöhen.

## Aufgabe 1: LEDs

### Aufgabe 1.1: Aufbau

Verkabeln Sie den Arduino wie abgebildet (die Ports müssen natürlich nicht übereinstimmen, also welche PWM-Ports verwendet werden, ist egal und es gibt auch mehrere 5V- und GND-Anschlüsse)

![](Images/Arduino.svg)

### Aufgabe 1.2: Klassendiagramm umsetzen

![](Images/Led.png)

Für die Abstraktion einer LED werden wir die Klasse ``Led`` erstellen. Programmieren Sie dazu das abgebildete UML-Diagramm nach. Verwenden Sie für den Quelltext der LED die gleichnamigen Dateien ``led.cpp/.h``.

Eine LED wird durch den Ausgabe-Port auf dem Arduino (``port``) und ihre Helligkeit (``brightness``) beschrieben.

Zum Erzeugen von ``Led``-Objekten sollen zwei Konstruktoren bereitgestellt werden:

1. Der erste Konstruktor ist parameterlos. Dieser macht gar nichts und sollte auch nicht verwendet werden. Er ist nur aus technischen Gründen da, damit die ``Led`` in ``std::vector`` verwendbar ist.
2. Der zweite Konstruktor akzeptiert die Port-Nummer und merkt sich diese auch. Dieser Port soll für die Ausgabe konfiguriert werden (``pinMode``) und die Helligkeit auf ``255`` (das Maximum) setzen.

Der Getter (``get_brightness``) gibt die aktuelle Helligkeit zurück. Der Setter speichert die Helligkeit in der Variable ``brightness`` und ändert dann die Helligkeit auf dem Ausgabe-Port.

Die Methoden ``on`` bzw. ``off`` sollen die LED an-/ausschalten. ``off`` sendet dazu eine ``0`` an den Ausgabe-Port. ``on`` sendet die alte Helligkeit an den Ausgabe-Port.

### Aufgabe 1.3: Mir geht ein Licht auf

_Hinweis:_ Da PlatformIO die Dateien alle automatisch kompiliert, können Sie schlecht die Lösungen für mehrere Aufgaben gleichzeitig aufbewahren. So etwas wie Startprojekte bei CMake existiert nicht. Kopieren Sie daher Ihre Lösungen immer an einen zusätzlichen Ort.

Erzeugen Sie drei Instanzen von der ``Led`` Klasse und schalten Sie in ``setup()`` alle LEDs an.

## Aufgabe 1.4: Lichterkette

Schauen Sie sich den ``loop()`` an. Wir erinnern uns: Diese Funktion wird in einer Endlosschleife vom Arduino aufgerufen.

Im ``loop()`` befindet sich bereits eine Variable ``counter``, die bei jedem Aufruf um eins erhöht wird. Nutzen Sie diesen Zähler, um die LEDs nach folgenden Regeln zu schalten:

- Wenn der Zähler kleiner als ``11``, soll die rote LED leuchten.
- Im Bereich ``[11, 20]`` soll die grüne LED leuchten.
- Wenn größer als ``20`` soll die blaue LED leuchten.
- Bei ``30`` wird der Zähler zurückgesetzt.

## Aufgabe 1.5: Impuls

LEDs sollen nun auch pulsieren können, also kontinuierlich die Helligkeit von ``0`` - ``255`` erhöhen.

Dazu fügen Sie eine neue Methode ``pulse`` zur Klasse ``Led`` hinzu.

![Diagramm wie oben, aber mit neuer Methode ``pulse``](Images/Led2.png)

Diese Methode erhöht bei jedem Aufruf die Helligkeit um ``1`` und übergibt die Helligkeit dann an den Ausgabe-Port.

Wenn das ``pulse`` nun innerhalb von ``loop()`` aufgerufen wird, ist der Puls ersichtlich.

Entfernen Sie nun die blaue LED aus der _Lichterkette_ und lassen Sie diese stattdessen pulsieren.

_Nachdenkaufgabe_: Irgendwann geht die LED aus und der Puls beginnt von vorne. Warum ist das so?

## Aufgabe 2: Taster

### Aufgabe 2.1: Aufbau

Nun wollen wir etwas Interaktivität hinzufügen.

Dazu fügen wir zuerst zwei Taster hinzu (siehe Abbildung).

![](Images/Arduino2.svg)

### Aufgabe 2.2: Klassendiagramm umsetzen

Für die Taster entwerfen wir eine weitere Klasse ``Button``.

Mit dieser Klasse soll es möglich sein zu erkennen, ob ein Taster gedrückt wird.

![](Images/Button.png)

Auch der ``Button`` verfügt über zwei Konstruktoren:

1. Der erste Konstruktor ist parameterlos und genauso nutzlos wie bei der ``Led`` :).
2. Der zweite Konstruktor akzeptiert die Port-Nummer. Dieser Port soll für die Eingabe konfiguriert werden (``pinMode``). Standardmäßig ist der Button _nicht_ gedrückt.

Der komplizierte Teil ist die ``update``-Methode: Taster haben das Problem, dass diese prellen. Das bedeutet, dass diese beim Herunterdrücken mehrmals gedrückt/nicht gedrückt über den Port senden. Das ist natürlich nicht erwünscht, daher muss der Taster _entprellt_ werden.

Das funktioniert folgendermaßen:

1. ``update`` wird für jeden ``loop``-Durchlauf aufgerufen
2. In ``update`` wird geprüft, ob der Taster momentan gedrückt ist (also lesen ``HIGH`` meldet).
3. Wenn er gedrückt ist, merkt man sich das in ``last_state``
4. Beim nächsten ``update``-Aufruf, wird nun wieder auf ``HIGH`` geprüft. Wenn der Taster beim letzten Mal gedrückt war, dann wird ``pressed`` auf ``true`` gesetzt. Das richtige Drücken ist also um einen Update-Zyklus verzögert (dies ist das entprellen).
5. Falls der Taster zwischendurch mal nicht gedrückt wird, werden beide Variablen zurückgesetzt

Die Methode ``is_pressed`` gibt den in ``update`` ermittelten Status zurück.

### Aufgabe 2.3: Impulspause

Die momentan pulsierende LED soll nur noch pulsieren, wenn der erste Taster gedrückt gehalten wird.

### Aufgabe 2.4: Lichtschalter

Entfernen Sie nun auch die grüne LED aus der Lichterkette.

Der zweite Taster soll nun wie ein Lichtschalter funktionieren:

Er schaltet bei Druck die grüne LED an bzw. aus.

Dafür ist ``is_pressed`` ungeeignet, da wir damit keinen Kippschalter simulieren können, sondern nur einen geschlossenen Schalter.

Die Lösung ist eine neue Methode namens ``is_triggered``. Diese gibt ``true`` zurück, wenn der Taster beim letzten ``update`` gedrückt wurde, ansonsten ``false``.

![Diagramm von oben mit notwendigen Ergänzungen für die Lichtschalter-Funktionalität](Images/Button2.png)
