# Praktikum 12: Animiertes DVD-Logo

**Frage:** Stimmt es, dass wir in diesem Praktikum einen animierten, farbigen DVD-Bildschirmschoner mit SDL2 programmieren werden?

**Antwort:**

Im Prinzip ja.

Aber es wird kein DVD-Logo sein, sondern ein Kreis.

Und der Kreis wird auch nicht farbig sein, sondern weiß.

Und wir programmieren auch nicht mit SDL2, sondern auf dem Arduino.

[Alles andere stimmt](https://de.wikipedia.org/wiki/Radio_Eriwan#Aufbau_der_Witze).

## Erläuterung

Hoffentlich kennen Sie [noch alle diese Animation](https://bouncingdvdlogo.com/). Das wird heute grob nachgebaut.

Schließen Sie das Display und eine LED auf Pin 2 an.

![](Images/OLED_Steckplatine.svg)

Laden Sie das Projekt ``p12_dvd_arduino`` und spielen Sie es auf den Arduino auf.

Wenn das Display richtig angeschlossen ist, wird ein Kreis dargestellt.

Dieser heißt passenderweise ``kreis``.

Nun soll dieser Kreis animiert werden. Machen Sie sich dazu zuerst damit vertraut, wie der Kreis auf dem Bildschirm dargestellt wird:

Der Ablauf ist wie in der Vorlesung erläutert. Alles, was auf dem Bildschirm gezeichnet werden kann, erbt von ``Sprite``. Dadurch wird das Programmier-Interface einfacher, weil alle 2D-Objekte auf die gleiche Art gezeichnet werden können. Alle Kind-Klassen, also hier der ``Circle``, müssen ``draw`` überladen. In dieser Methode findet das eigentliche Zeichnen statt. Da Adafruit bereits ziemlich fortschrittliche Zeichenroutinen anbietet, ist die Funktion sehr kurz.

## 1. Aufgabe

Jeder ``Sprite`` (und damit ist der ``kreis`` keine Ausnahme) erbt die Variablen ``position`` und ``velocity``. Beide Variablen sind vom Typ ``Point``.

- ``position`` steht für die Position des Objektes (obere linke Ecke)
- ``velocity`` ist der Geschwindigkeitsvektor. Dieser wird angepasst, wenn ``update()`` aufgerufen wird

Bewegen Sie nun den Kreis nach rechts. Was passiert, wenn der Kreis am rechten Rand ankommt?

## 2. Aufgabe

Sobald der Kreis am rechten Rand ankommt, soll er nach links zurückprallen. Kehren Sie dazu den Richtungsvektor um.

Dies funktioniert dann also wie beim Reflexionsgesetz.

## 3. Aufgabe

Implementieren Sie dies nun auch für die Y-Komponente des Vektors.

Nun ist der DVD-Bildschirmschoner auch schon fertig.

![](Images/DVD.mp4)

# Praktikum 12: Conveys Game of Life

Nun programmieren wir noch [Conways Game of Life](https://de.wikipedia.org/wiki/Conways_Spiel_des_Lebens). Dies ist ein zellulärer Automat in dem die Zellen nach spezifischen Regeln lebendig oder tot sein können.

Das Game of Life verwendet ein zweidimensionales Spielfeld. Dadurch lernen wir den Umgang mit zweidimensionalen Arrays kennen.

Schauen Sie sich diese Animation an: schwarze Zellen sind lebendig, weiße tot. Auf dem Arduino werden wir uns auf angeschaltete Rechtecke lebendig und ausgeschaltete Rechtecke tot festlegen.

Am Ende wird es so aussehen:

![](Images/Fertig.mp4)

Als Basis verwenden wir das abgebildete UML-Diagramm. Alle Klassen mit hellgrünem Hintergrund sind bereits vorhanden. Von Ihnen zu erstellende Klassen sind _weiß_ dargestellt, wobei die ``Board``-Klasse ebenfalls bereits teilweise vorhanden ist.

![](Images/UML.png)

## Das Spielfeld (Klasse ``Board``)

``Board`` erbt, genau wie ``Circle`` und ``Rectangle``, von der Klasse ``Sprite``.

Das Spielfeld (``field``) ist zweidimensional. Solch ein Layout sieht aber C++ nicht vor. Daher verwenden wir stattdessen einen ``std::vector``, der einen ``std::vector<uint8_t>`` enthält (Datentyp ``std::vector<std::vector<uint8_t>>``). Die Indizierung erfolgt dann über einen zweifachen Array-Zugriff: Das Feld in der 2. Zeile und 5. Spalte ist ``field[1][4]``.

![Adressierung der Felder in einem 5x4 Gitter (``row``, ``col``)](Images/Koordinaten.png)

Eine lebende Zelle hat in diesem Array den Wert ``1``, tote Zellen den Wert ``0``.

Die Variablen ``field_width``, ``field_height`` und ``rect_dim`` sind Konstanten. ``field_width`` bzw. ``field_height`` enthält die Anzahl an Zellen je Zeile bzw. Spalte (hier: ``32`` bzw. ``16``). ``rect_dim`` enthält für die Darstellung die Breite bzw. Höhe eines Feldes in Pixeln (hier: ``4``).

Dies bedeutet, dass beim Zeichnen einer Zelle nicht einzelne Pixel hervorgehoben werden müssen, sondern 4x4-Rechtecke.

### Konstruktor ``Board()``

Der Konstruktor enthält nachfolgenden Code:

```cpp
field.resize(field_height);
for (auto& f: field) {
    f.resize(field_width);
}
```

Der Konstruktor muss zuerst das Spielfeld (``field``) auf die richtige Größe initialisieren. Da das Feld zweidimensional ist, muss dies in zwei Schritten erfolgen.

In der ersten Anweisung wird der äußere Vektor mit ``resize`` auf die Anzahl an Feldern pro Zeile vergrößert. Die anschließende Schleife durchläuft nun diesen Vektor und vergrößert nun ebenfalls die gerade erzeugten, leeren Vektoren auf die Felder pro Spalte.

Nun muss noch direkt dahinter das Feld mit lebenden Zellen befüllt werden. Dies kann entweder zufällig erfolgen oder mit einer vorgegebenen Initialisierung.

Als vorgegebene Initialisierung verwenden wir vorerst einen Gleiter:

```cpp
field[0][0] = 1;
field[0][2] = 1;
field[1][1] = 1;
field[1][2] = 1;
field[2][1] = 1;
```

![So sieht ein Gleiter aus](Images/Gleiter.png)

### Methode ``draw``

_Jetzt sind Sie dran_

``draw`` ist eine Methode von ``Sprite`` und muss überschrieben werden. Deshalb hat diese auch einen ``override``-Modifizierer.

In dieser Methode sollen weiße Rechtecke an den Stellen erzeugt werden, wo auf dem Spielfeld lebende Zellen sind.

Dazu müssen Sie zuerst das zweidimensionale Feld durchlaufen.

Ein zweidimensionales Feld kann folgendermaßen durchlaufen werden (Code bereits vorgegeben):

```cpp
for (int row = 0; row < field_height; ++row) {
    for (int col = 0; col < field_width; ++col) {
        // Adressierung des Feldes über [row][col]
        // Einzelne Zelle zeichnen
    }
}
```

Für ein 5x4-Feld sieht der Durchlauf dann so aus:

![](Images/Gitter.png)

Verwenden Sie nun ``draw`` von der Klasse ``Rectangle`` um überall dort schwarze Rechtecke zu malen, wo lebendige Zellen sind. Die Rechtecke sollen eine Breite/Höhe von ``rect_dim`` Pixel (also ``4`` haben.

Überlegen Sie sich dazu zuerst, wie eine einzelne Zelle im Raum verortet werden kann. Führen Sie diese Überlegung für die markierte Zelle (7) durch und dann implementieren Sie in der Schleife den allgemeinen Code.

![](Images/Gitter_Dim.png)

_Hinweis_: Das mit ``field_height``/``field_width`` funktioniert nur für unser konkretes Spielfeld. Damit die Schleife für beliebige 2D-Arrays funktioniert, muss der Code so lauten:

```cpp
for (size_t row = 0; row < field.size(); ++row) {
    for (size_t col = 0; col < field[row].size(); ++col) {
        // Adressierung des Feldes über [row][col]
    }
}
```

### Der erste Test

Nun können wir einen ersten Test durchführen.

Flashen Sie das Programm.

Wenn alles funktioniert, werden Sie einen statischen Gleiter sehen:

![](Images/Gleiter.png)

### Ein lebendes System

Nun müssen wir dem System noch Leben einhauchen. Dabei gelten die folgenden Regeln:

Bei allen Zellen wir die 8er-Nachbarschaft angeschaut (siehe Abbildung) und dann gezählt, wie viele lebende Nachbarn die Zelle hat.

![Eine lebende Zelle mit ihren acht Nachbarn](Images/8.png)

Eine lebende Zelle mit weniger als zwei Nachbarn stirbt an Einsamkeit und eine Zelle mit mehr als drei Nachbarn stirbt an Überbevölkerung.

Eine tote Zelle mit drei Nachbarn erwacht zum Leben.

Alle anderen Zellen bleiben unverändert.

#### Zählen der Nachbarn

Die Methode ``count_neighbours`` soll mithilfe von ``item_at`` die Nachbarn ermitteln.

Das Hauptproblem sind die Ränder. Wenn wir eine Zelle am linken Rand betrachten, sind die Nachbarn teilweise auf der rechten Seite, da an Rändern ein Übergang zur gegenüberliegenden Seite erfolgt. Die Topologie des Feldes ist also ein Torus.

![Wie unser Spielfeld wirklich aussieht (LucasVB, gemeinfrei)](Images/Torus_from_rectangle.gif)

Die Indizes beim Feldzugriff müssen in den Randfällen also neu berechnet werden. Dafür soll ``item_at`` zuständig sein: Diese Methode gibt im üblichen Fall einfach den Wert des Spielfeldes bei der übergebenen Zeile/Spalte (``row/col``) zurück.

Es gibt aber folgende Sonderfälle:

- Wenn die Zeile/Spalte ``-1`` ist, wird der Index vorher auf das letzte Feld in der Zeile/Spalte geändert.
- Wenn die Zeile/Spalte ein Feld größer als die Feldgröße ist (also rechts bzw. unten das Feld verlässt), wird die Zeile/Spalte auf ``0`` gesetzt.

Die Methode ``count_neighbours`` ermittelt nun, ausgehend von den übergebenen Feld-Koordinaten und mithilfe von ``item_at``, wie viele Nachbarn in der 8er-Nachbarschaft lebendig sind.

#### Aktualisieren des Spielfeldes

Nun müssen noch die oben beschriebenen Regeln umgesetzt werden.

Dies machen wir in ``update_field``.

Da die Änderungen sich immer auf die nächste Generation auswirken, muss zuerst das Spielfeld kopiert werden. Die Kopie nennen wir ``next_field``.

Nun durchlaufen wir das Feld der aktuellen Generation (also ``field``) und setzen in ``field_next`` die Zellen anhand der Regeln von oben auf lebendig bzw. tot.

Nach der Schleife kopieren wir jetzt ``field_next`` nach ``field``, damit die Änderungen übernommen werden.

Diese Methode muss nun noch in direkt vor ``board.draw()`` aufgerufen werden.

Führen Sie das Programm nun aus. Wenn alles stimmt, sehen Sie nun einen animierten Gleiter:

![](Images/Gleiter.mp4)

# Praktikum 12: Zusätzliche Aufgaben

![](Images/OLED2_Steckplatine.svg)

Da diese recht viel Aufbau erfordern empfehle ich, diese erst zu Hause zu machen.

## Für die DVD-Animation

1. Der 1. Taster kehrt bei Tastendruck die X-Komponente des Beschleunigungsvektors um
2. Der 2. Taster kehrt bei Tastendruck die Y-Komponente des Beschleunigungsvektors um
3. Das Potentiometer soll die Geschwindigkeit regulieren. Bei 0V soll der Kreis stillstehen und bei 5V mit 10px pro Frame wandern. Die Zwischenstufen sollen linear interpoliert werden.

## Für Game of Life

### Zufällige Initialisierung

Verwenden Sie anstatt des Gleiters nun im Konstruktor eine zufällige Initialisierung.

Eine Zelle soll mit einer Wahrscheinlichkeit von ``25%`` lebendig sein.

Belesen Sie sich dazu, wie Zufall auf dem Arduino funktioniert.

Führen Sie das Programm aus. Das Ergebnis ist ein anfängliches chaotisches System, dass nach vielen Generationen einen statischen Zustand mit einigen oszillierenden Objekten erreicht.

![](Images/Zufall_Gitter.png)

### Geschwindigkeitsregelung

Führen Sie wieder ein Potentiometer ein, damit die Geschwindigkeit reguliert werden kann.

### Pause

Beim Drücken des 1. Tasters soll die Simulation pausiert werden. Ein erneuter Tastendruck führt diese fort.

### Neustart

Beim Drücken des 2. Tasters soll das Feld zufällig neu initialisiert werden.
