#include "board.h"
#include "rectangle.h"

Board::Board() {
    // Feld anlegen
    field.resize(field_height);
    for (auto& f: field) {
        f.resize(field_width);
    }

    for (int row = 0; row < field_height; ++row) {
        for (int col = 0; col < field_width; ++col) {
            field[row][col] = random(0, 4) == 1;
        }
    }

    // Erzeugt einen Gleiter in der oberen linken Ecke
    field[0][0] = 1;
    field[0][2] = 1;
    field[1][1] = 1;
    field[1][2] = 1;
    field[2][1] = 1;
}

int Board::item_at(int row, int col) const {
    if (row == -1) {
        row = field_height - 1;
    } else if (row == field_height) {
        row = 0;
    }

    if (col == -1) {
        col = field_width - 1;
    } else if (col == field_width) {
        col = 0;
    }

    return field[row][col];
}

int Board::count_neighbours(int row, int col) const {
    int tl = item_at(row - 1, col - 1);
    int tm = item_at(row - 1, col);
    int tr = item_at(row - 1, col + 1);
    int ml = item_at(row, col - 1);
    int mr = item_at(row, col + 1);
    int bl = item_at(row + 1, col - 1);
    int bm = item_at(row + 1, col);
    int br = item_at(row + 1, col + 1);

    return tl + tm + tr + ml + mr + bl + bm + br;
}

void Board::update_field() {
    auto next_field = field;

    for (int row = 0; row < field_height; ++row) {
        for (int col = 0; col < field_width; ++col) {
            int field = item_at(row, col);
            int neighbours = count_neighbours(row, col);

            if (field == 1) {
                if (neighbours < 2 || neighbours > 3) {
                    next_field[row][col] = 0;
                }
            } else {
                if (neighbours == 3) {
                    next_field[row][col] = 1;
                }
            }
        }
    }

    field = next_field;
}

void Board::draw(Adafruit_SSD1306& renderer) {
    // Zellen
    for (int row = 0; row < field_height; ++row) {
        for (int col = 0; col < field_width; ++col) {
            // Adressierung des Feldes über [row][col]
            // Einzelne Zelle zeichnen
            if (item_at(row, col) == 1) {
                Rectangle rect(Point(col * rect_dim, row * rect_dim), rect_dim, rect_dim);
                rect.draw(renderer);
            }
        }
    }

}
