#include "point.h"

#include <cmath>
#include <stdexcept>

Point::Point() {
    // Dieser Konstruktor macht nichts,
    // x und y sind schon auf 0.0 :)
}

Point::Point(double x, double y) {
    check_coordinates(x, y);

    this->x = x;
    this->y = y;
}

double Point::get_x() const {
    return x;
}

double Point::get_y() const {
    return y;
}

void Point::move_by(double dx, double dy) {
    check_coordinates(x + dx, y + dy);

    x += dx; // x = x + dx
    y += dy;
}

void Point::move_to(double xn, double yn) {
    check_coordinates(xn, yn);

    x = xn;
    y = yn;
}

double Point::distance_to(const Point& other_p) const {
    return sqrt(
        pow(x - other_p.x, 2) +
        pow(y - other_p.y, 2)
    );
}

void Point::check_coordinates(double new_x, double new_y) const {
    // Da wir keinen einzelnen Setter für x und y haben wird stattdessen auf
    // eine Hilfsmethode gesetzt, die von allen Methoden die X/Y modifizieren
    // aufgerufen wird

    // Alternativ könnte man natürlich set_x und set_y implementieren und dann
    // an allen Stellen aufrufen

    if (new_x < -1000 || new_x > 1000) {
        throw std::runtime_error("x ist nicht in (-1000, 1000)");
    }

    if (new_y < -1000 || new_y > 1000) {
        throw std::runtime_error("y ist nicht in (-1000, 1000)");
    }
}
