# Konfiguration

Bei den SDL2-Projekten müssen Sie neben dem wohlbekannten Kit nun auch die Build-Variante einstellen.

Klicken Sie dazu auf den CMake-Text neben dem eingerahmten I:

![](Images/Variant.png)

Es erscheint der folgende Dialog:

![](Images/Variant2.png)

Wählen Sie dort, abhängig von Ihrem System, die richtige Option aus.

Die Optionen werden leider nicht direkt angewendet. Damit diese funktionieren, rechtsklicken Sie auf die CMakeLists.txt und wählen _Alle Projekte sauber neu konfigurieren / Reconfigure and clean all projects_ aus. Die erste Installation von SDL2 kann einige Minuten dauern, da viele Dinge einmalig heruntergeladen werden. Einfach geduldig sein.

# Praktikum 11: Game of Life

Im heutigen Praktikum programmieren wir [Conways Game of Life](https://de.wikipedia.org/wiki/Conways_Spiel_des_Lebens). Dies ist ein zellulärer Automat in dem die Zellen nach spezifischen Regeln lebendig oder tot sein können.

TODO: Fertig.mp4

Als Basis verwenden wir die _Engine_ aus der Vorlesung (siehe Abbildung). Alle Klassen mit hellgrünem Hintergrund sind bereits vorhanden. Von Ihnen zu erstellende Klassen sind _weiß_ dargestellt und von Ihnen durchzuführende Änderungen in den existierenden Klassen _blau_.

``SDL_Helper`` und die ``main()`` sind bereits korrekt auf das Problem vorkonfiguriert.

![](Images/UML.png)

## Das Spielfeld (Klasse ``Board``)

Erstellen Sie zuerst das Grundgerüst der Klasse in ``board.cpp/.h``. ``Board`` erbt, genau wie ``Rectangle``, von der Klasse ``Sprite``.

Das Spielfeld (``field``) ist zweidimensional. Solch ein Layout sieht aber C++ nicht vor. Daher verwenden wir stattdessen einen ``std::vector``, der einen ``std::vector<int>`` enthält (Datentyp ``std::vector<std::vector<int>>``). Die Indizierung erfolgt dann über einen zweifachen Array-Zugriff: Das Feld in der 2. Zeile und 5. Spalte ist ``field[1][4]``.

Eine lebende Zelle hat in diesem Array den Wert ``1``, tote Zellen den Wert ``0``.

Die Variablen ``field_dim`` und ``rect_dim`` sind Konstanten. ``field_dim`` enthält die Anzahl an Feldern je Zeile bzw. Spalte (hier: ``30``). ``rect_dim`` enthält für die Darstellung die Breite bzw. Höhe eines Feldes in Pixeln (hier: ``24``, dies ergibt sich aus ``720 / 30``).

### Konstruktor ``Board()``

Verwenden Sie folgenden Code für den Konstruktor:

```cpp
field.resize(field_dim);
for (auto& f: field) {
    f.resize(field_dim);
}
```

Der Konstruktor muss zuerst das Spielfeld (``field``) auf die richtige Größe initialisieren. Da das Feld aus zweidimensional ist, muss dies in zwei Schritten erfolgen.

In der ersten Anweisung wird der äußere Vektor mit ``resize`` auf die Anzahl an Feldern pro Zeile vergrößert. Die anschließende Schleife durchläuft nun diesen Vektor und vergrößert nun ebenfalls die gerade erzeugten, leeren Vektoren auf die Felder pro Spalte.

Nun muss noch direkt dahinter das Feld mit lebenden Zellen befüllt werden. Dies kann entweder zufällig erfolgen oder mit einer vorgegebenen Initialisierung.

Initialisieren Sie das Spielfeld vorerst folgendermaßen. Damit erzeugen Sie einen Gleiter:

```cpp
field[0][0] = 1;
field[0][1] = 0;
field[0][2] = 1;
field[1][0] = 0;
field[1][1] = 1;
field[1][2] = 1;
field[2][0] = 0;
field[2][1] = 1;
field[2][2] = 0;
```

### Methode ``draw``

``draw`` ist eine Methode von ``Sprite`` und muss überschrieben werden. Deshalb hat diese auch einen ``override``-Modifizierer.

In dieser Methode sollen schwarze Rechtecke an den Stellen erzeugt werden, wo auf dem Spielfeld lebende Zellen sind.

Dazu müssen Sie zuerst das zweidimensionale Feld durchlaufen.

Ein zweidimensionales Feld kann folgendermaßen durchlaufen werden:

```cpp
for (int row = 0; row < field_dim; ++row) {
    for (int col = 0; col < field_dim; ++col) {
        // Adressierung des Feldes über [row][col]
    }
}
```

Für ein 4x3-Feld sieht der Durchlauf dann so aus:

![](Images/Gitter.png)

Verwenden Sie nun ``draw`` von der Klasse ``Rectangle`` um überall dort schwarze Rechtecke zu malen, wo lebendige Zellen sind. Die Rechtecke sollen eine Breite/Höhe von ``rect_dim`` Pixel haben.

Überlegen Sie sich dazu zuerst, wie eine einzelne Zelle im Raum verortet werden kann. Führen Sie diese Überlegung für die markierte Zelle 7 durch und dann implementieren Sie in der Schleife den allgemeinen Code.

![](Images/Gitter_Dim.png)

_Hinweis_: Das mit ``field_dim`` funktioniert nur für unser konkretes Spielfeld, weil Zeilen und Spalten die gleiche Anzahl an Elementen haben. Damit die Schleife für beliebige 2D-Arrays funktioniert, muss der Code so lauten:

```cpp
for (size_t row = 0; row < field.size(); ++row) {
    for (size_t col = 0; col < field[row].size(); ++col) {
        // Adressierung des Feldes über [row][col]
    }
}
```

### Der erste Test

Nun können wir einen ersten Test durchführen.

Ändern Sie die Klasse ``Engine`` so ab wie im UML-Diagramm dargestellt und rufen Sie dann an geeigneter Stelle in dieser Klasse die ``draw``-Methode vom ``Board`` auf.

Starten Sie das Programm. Wenn alles funktioniert, werden Sie einen statischen Gleiter sehen:

![](Images/Gleiter.png)

### Ein lebendes System

Nun müssen wir dem System noch Leben einhauchen. Dabei gelten die folgenden Regeln:

Bei allen Zellen wir die 8er-Nachbarschaft angeschaut (siehe Abbildung) und dann gezählt, wie viele lebende Nachbarn die Zelle hat.

![Eine lebende Zelle mit ihren acht Nachbarn](Images/8.png)

Eine lebende Zelle mit weniger als zwei Nachbarn stirbt an Einsamkeit und eine Zelle mit mehr als drei Nachbarn stirbt an Überbevölkerung.

Eine tote Zelle mit drei Nachbarn erwacht zum Leben.

Alle anderen Zellen bleiben unverändert.

#### Zählen der Nachbarn

Die Methode ``count_neighbours`` soll mithilfe von ``item_at`` die Nachbarn ermitteln.

Das Hauptproblem sind die Ränder. Wenn wir eine Zelle am linken Rand betrachten, sind die Nachbarn teilweise auf der rechten Seite, da an Rändern ein Übergang zur gegenüberliegenden Seite erfolgt. Die Topologie des Feldes ist also ein Torus.

![Wie unser Spielfeld wirklich aussieht (LucasVB, gemeinfrei)](Images/Torus_from_rectangle.gif)

Die Indizes beim Feldzugriff müssen in den Randfällen also neu berechnet werden. Dafür soll ``item_at`` zuständig sein: Diese Methode gibt im üblichen Fall einfach den Wert des Spielfeldes bei der übergebenen Zeile/Spalte (``row/col``) zurück.

- Wenn die Zeile/Spalte ``-1`` ist, wird der Index aber vorher auf das letzte Feld in der Zeile/Spalte geändert.

- Wenn die Zeile/Spalte ein Feld größer als die Feldgröße ist (also rechts bzw. unten das Feld verlässt), wird die Zeile/Spalte auf ``0`` gesetzt.

Die Methode ``count_neighbours`` ermittelt nun, ausgehend von den übergebenen Feld-Koordinaten und mithilfe von ``item_at``, wie viele Nachbarn in der 8er-Nachbarschaft lebendig sind.

#### Aktualisieren des Spielfeldes

Nun müssen noch die oben beschriebenen Regeln umgesetzt werden.

Dies machen wir in ``update_field``.

Da die Änderungen sich immer auf die nächste Generation auswirken, muss zuerst das Spielfeld kopiert werden. Die Kopie nennen wir ``next_field``.

Nun durchlaufen wir das Feld der aktuellen Generation (also ``field``) und setzen in ``field_next`` die Zellen anhand der Regeln von oben auf lebendig bzw. tot.

Nach der Schleife kopieren wir jetzt ``field_next`` nach ``field``, damit die Änderungen übernommen werden.

Diese Methode muss nun noch in ``Engine`` direkt vor ``board.draw`` aufgerufen werden.

Führen Sie das Programm nun aus. Wenn alles stimmt, sehen Sie nun einen animierten Gleiter:

TODO: Gleiter.mp4

## Zufällige Initialisierung

Verwenden Sie anstatt des Gleiters nun im Konstruktor eine zufällige Initialisierung.

Eine Zelle soll mit einer Wahrscheinlichkeit von ``25%`` lebendig sein.

Belesen Sie sich noch einmal, wie Zufall funktioniert.

Führen Sie das Programm aus. Das Ergebnis ist ein anfängliches chaotisches System, dass nach vielen Generationen einen statischen Zustand mit einigen oszillierenden Objekten erreicht.

![](Images/Zufall_Gitter.png)

## Ein Gitternetz (*)

Ergänzen Sie nun in ``draw`` für die bessere Übersichtlichkeit ein Gitternetz:

![](Images/Gleiter_Gitter.png)

Dafür können Sie Rechtecke verwenden, die in eine Dimension die Fensterbreite/Fensterhöhe haben und in die andere Dimension ``1``.

Verwenden Sie als Farbe einen mittleren Grauton.
