#include "board.h"
#include "rectangle.h"
#include <random>

Board::Board() {
    field.resize(field_dim);
    for (auto& f: field) {
        f.resize(field_dim);
    }

    std::random_device rd;
    std::mt19937 rand(rd());
    std::uniform_int_distribution<int> dist{0, 3};

    for (int row = 0; row < field_dim; ++row) {
        for (int col = 0; col < field_dim; ++col) {
            field[row][col] = dist(rand) == 1;
        }
    }

    /*field[0][0] = 1;
    field[0][1] = 0;
    field[0][2] = 1;
    field[1][0] = 0;
    field[1][1] = 1;
    field[1][2] = 1;
    field[2][0] = 0;
    field[2][1] = 1;
    field[2][2] = 0;*/
}

int Board::item_at(int row, int col) const {
    if (row == -1) {
        row = field_dim - 1;
    } else if (row == field_dim) {
        row = 0;
    }

    if (col == -1) {
        col = field_dim - 1;
    } else if (col == field_dim) {
        col = 0;
    }

    return field[row][col];
}

int Board::count_neighbours(int row, int col) const {
    int tl = item_at(row - 1, col - 1);
    int tm = item_at(row - 1, col);
    int tr = item_at(row - 1, col + 1);
    int ml = item_at(row, col - 1);
    int mr = item_at(row, col + 1);
    int bl = item_at(row + 1, col - 1);
    int bm = item_at(row + 1, col);
    int br = item_at(row + 1, col + 1);

    return tl + tm + tr + ml + mr + bl + bm + br;
}

void Board::draw(SDL_Renderer* renderer) {
    // Zellen
    for (int row = 0; row < field_dim; ++row) {
        for (int col = 0; col < field_dim; ++col) {
            if (item_at(row, col) == 1) {
                Rectangle rect(Point(col * rect_dim, row * rect_dim), rect_dim, rect_dim);
                rect.set_color(0, 0, 0);
                rect.draw(renderer);
            }
        }
    }

    // Gitter
    for (int i = 1; i < field_dim; ++i) {
        Rectangle rect(Point(i * rect_dim, 0), 1, 720);
        Rectangle rect2(Point(0, i * rect_dim), 720, 1);
        rect.set_color(128, 128, 128);
        rect2.set_color(128, 128, 128);
        rect.draw(renderer);
        rect2.draw(renderer);
    }
}

void Board::update_field() {
    auto next_field = field;

    for (int row = 0; row < field_dim; ++row) {
        for (int col = 0; col < field_dim; ++col) {
            int field = item_at(row, col);
            int neighbours = count_neighbours(row, col);

            if (field == 1) {
                if (neighbours < 2 || neighbours > 3) {
                    next_field[row][col] = 0;
                }
            } else {
                if (neighbours == 3) {
                    next_field[row][col] = 1;
                }
            }
        }
    }

    field = next_field;
}
