#include <SDL.h>
#include <iostream>
#include <chrono>
#include <thread>

#include "sdl_helper.h"
#include "engine.h"

// WICHTIG:
// SDL2 unter Windows benoetigt unbedingt main(int, char**), sonst gibt es einen Fehler
// Die Variablennamen für die Parameter wurden weggelassen, sonst gibt es Compilerwarnungen
// (ungenutzter Parameter)
int main(int, char**) {
    // Hilfsklasse aufrufen, die für uns SDL initialisiert
    SDL_Helper sdl_helper;

    SDL_Renderer* renderer = sdl_helper.get_renderer();

    // Engine-Objekt erzeugen
    Engine engine(renderer, sdl_helper.get_width(), sdl_helper.get_height());

    /* Game Loop */
    // ca. 16,6 ms
    const auto time_per_frame = std::chrono::microseconds(1000000 / 10);

    for (;;) {
        // Startzeit der Schleife ermitteln
        auto time_begin = std::chrono::steady_clock::now();

        if (!engine.process_events()) {
            return 0;
        }

        engine.update_and_render();

        // Ermitteln, wie lange die Schleife ausgeführt wurde
        auto time_end = std::chrono::steady_clock::now();
        auto time_delta = time_end - time_begin;

        if (time_delta < time_per_frame) {
            // In diesem Frame ist noch Zeit übrig: Warten
            std::this_thread::sleep_for(time_per_frame - time_delta);
        }
    }

    return 0;
}
