#include "board.h"
#include "rectangle.h"
#include <random>

Board::Board() {
    snake.push_back(Point(1, 1));
    snake.push_back(Point(2, 1));
    snake.push_back(Point(3, 1));
    snake.push_back(Point(4, 1));

    place_food();
}

void Board::draw(SDL_Renderer* renderer) {
    if (game_over) {
        SDL_SetRenderDrawColor(renderer, 0xFF, 0xD9, 0xEA, 0xFF);
        SDL_RenderClear(renderer);
    }

    for (size_t i = 0; i < snake.size(); ++i) {
        Point sp = snake[i];
        sp.x *= rect_dim;
        sp.y *= rect_dim;
        Rectangle rect(sp, rect_dim, rect_dim);
        if (i == snake.size() - 1) {
            // Kopf
            rect.set_color(255, 0, 0);
        } else {
            rect.set_color(0, 0, 0);
        }
        rect.draw(renderer);
    }

    Point fp = Point(food.x * rect_dim, food.y * rect_dim);
    Rectangle food_rect(fp, rect_dim, rect_dim);
    food_rect.set_color(0, 0, 255);
    food_rect.draw(renderer);

    // Gitter
    for (int i = 1; i < field_dim; ++i) {
        Rectangle rect(Point(i * rect_dim, 0), 1, 720);
        Rectangle rect2(Point(0, i * rect_dim), 720, 1);
        rect.set_color(128, 128, 128);
        rect2.set_color(128, 128, 128);
        rect.draw(renderer);
        rect2.draw(renderer);
    }
}

void Board::update_field() {
    if (game_over) {
        return;
    }

    ++tick;
    if (tick >= ticks_per_move) {
        tick = 0;

        if (!did_eat) {
            snake.erase(snake.begin());
        }
        did_eat = false;

        Point kopf = snake.back();

        if (direction == up) {
            kopf.move_by(0, -1);
            snake.push_back(kopf);
        } else if (direction == down) {
            kopf.move_by(0, 1);
            snake.push_back(kopf);
        } else if (direction == left) {
            kopf.move_by(-1, 0);
            snake.push_back(kopf);
        } else if (direction == right) {
            kopf.move_by(1, 0);
            snake.push_back(kopf);
        }

        // Pruefen, ob der Kopf mit etwas kollidiert
        // Wenn der neue Kopf außerhalb ist: Game Over
        if (kopf.x < 0 || kopf.x >= field_dim || kopf.y < 0 || kopf.y >= field_dim) {
            game_over = true;
        }

        // Wenn der neue Kopf ein Körperteil berührt: Game Over
        for (size_t i = 0; i < snake.size() - 1; ++i) {
            Point sp = snake[i];
            if (kopf.x == sp.x && kopf.y == sp.y) {
                game_over = true;
            }
        }

        // Nahrung
        if (food.x == snake.back().x && food.y == snake.back().y) {
            place_food();
            did_eat = true;
            ticks_per_move -= 0.5;
            if (ticks_per_move < 1) {
                ticks_per_move = 1;
            }
        }
    }
}

void Board::set_direction(int new_direction) {
    direction = new_direction;
}

void Board::place_food() {
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<int> dist{0, field_dim - 1};

    food = Point(dist(gen), dist(gen));
}
