# 0

Zahlen können in C++ verschiedene Wertebereiche haben, da die Datentypen auf die Speicherbreite des Prozessors abgebildet werden.

Wir schauen uns hier die üblichen Datentypen für Zahlen an.

Alle Datentypen für Ganzzahlen verwenden eine Darstellung im Zweierkomplement.

# 1

Dies ist ein ``char``. Er hat die Größe von _8 Bit_ und wird für kleine Zahlen und für ASCII-Zeichen verwendet.

Die Größe von 8 Bit ist eine der wenigen Garantien, die uns C++ gibt.

# 2

Zahlen vom Typ ``short`` sind auf allen relevanten Systemen _16 Bit_ groß.

Laut Standard muss er aber nur mindestens so groß sein wie ein ``char`` sein.

# 3

Zahlen vom Typ ``int`` sind üblicherweise 32 Bit groß. Die Ausnahme sind veraltete 16 Bit-Betriebssystem, z.B. MS-DOS. Dort ist der ``int`` nur 16 Bit.

Dieser Datentyp wird bei Ganzzahlen am häufigsten verwendet, da er einen ziemlich großen Wertebereich hat und viele Prozessoren effizient mit 32 Bit rechnen können.

Laut Standard muss ``int`` mindestens so groß wie ``short`` sein.

# 4

``long`` ist stark vom Betriebssystem abhängig.

Bei Windows-Systemen ist ``long`` immer 32 Bit groß.

Bei Linux-Systemen kommt es auf das System an: Bei 32 Bit-Systemen ist er 32 Bit und bei 64 Bit-Systemen ist er 64 Bit.

# 5

Für Gleitkommazahlen gibt es zwei Datentypen.

``float`` hat die halbe Genauigkeit von ``double``. Welchen Typ man verwendet ist ein Kompromis aus Geschwindigkeit vs. Genauigkeit: Mit ``float`` rechnet es sich schneller, aber ``double`` ist genauer.

Diese beiden Typen Verhalten sich auf allen Systemen gleich, da diese dem [Standard IEEE754](https://de.wikipedia.org/wiki/IEEE_754) entsprechen.

# 6

Vor alle Ganzzahl-Typen kann noch ``unsigned`` geschrieben werden. Dies sorgt dafür, dass das oberste Bit (MSB) nicht mehr als Vorzeichenbit interpretiert wird. Die Zahl ist jetzt also vorzeichenlos.

# 7

Bei Gleitkommazahlen ist ``unsigned`` unzählig. Der Standard sieht es einfach nicht vor. Zudem gibt es auch keine spezielle Hardware in Prozessoren, um diese Berechnung effizient durchzuführen. Die Berechnung wäre also langsam und unpraktikabel.

# 8

Im Header ``<cstdint>`` sind hardwareneutrale Datentypen definiert.

Ein Beispiel für solche Datentypen ist ``int32_t`` und ``uint32_t``. Diese Ganzzahl-Typen sind auf allen Platformen 32 Bit und mit oder ohne Vorzeichen (``u``/unsigned).

Diese Datentypen sind nützlich, wenn man hardwarenah programmiert und es auf die exakte Menge von Bits ankommt.

Das ``c`` am Anfang des Header bedeutet übrigens, dass dieser Header von C stammt und nicht von C++.

# 9

Mit ``int64_t`` können 64 Bit-Zahlen verwendet werden.

Diesen Typ verwendet man, wenn 32 Bit nicht ausreichend sind.

# 10

Mit ``sizeof`` kann man zur Laufzeit die Größe eines Typs in Byte ermitteln.

## Wie groß ist dieser Datentyp in Byte?

- 0,5
* 1
- 2

# 11

## Wie groß ist dieser Datentyp in Byte?

- 2
- 4
- 6
* 8
- 10

# 12

``float`` hat eine Größe von 4 Byte.

``double`` ist das doppelte, also 8.

# 13

Das hier muss man sich nicht unbedingt merken. In ``<limits>`` sind Funktionen, um z.B. die Zahlenbereiche eines Datentyps zu ermitteln.

# 14

Hiermit werden die kleinste bzw. größte Zahl für den Typ ``char`` ermittelt.

Also ``-128`` und ``127``

# 15

Hier nochmal das gleiche für den vorzeichenlosen ``char``.

## Welche Werte werden hier ausgegeben?

- ``-128`` und ``127``
- ``-255`` und ``255``
* ``0`` und ``255``
- ``0`` und ``65535``
