# 0

Zum Abschluss schauen wir uns mal noch ein besonderes Thema bei C++ an: Templates.

Damit werden wir uns nicht weiter beschäftigen, da diese ein fortgeschrittenes Thema sind, aber manchmal sind diese bereits in sehr einfachen Fällen unglaublich praktisch.

Wir fangen mit der ``anzahl``-Funktion aus dem ersten Storyteller an.

Konkret möchte ich hier ein Problem aufzeigen: Diese Funktion ist eigentlich universell für alle Arten von Vektoren verwendbar, akzeptiert aber nur Vektoren vom Typ ``int``.

Wie könnte man das lösen?

# 1

Also das hier funktioniert nicht, weil der Vektor vom Typ ``std::vector<std::string>`` ist, die Funktion aber ``std::vector<int>`` möchte.

# 2

Die erste Option wäre, die gesamte Funktion einfach zu kopieren und dann ``int`` mit ``std::string`` zu ersetzen.

Jetzt funktioniert es.

# 3

So ganz nebenbei ergänzen wir jetzt noch beim Typ des Schleifenelements ``const`` und ``&``, sonst werden unnötige Kopien des Strings erzeugt.

Bei der Zahl oben war das egal, da die Zahl in ein Prozessorregister passt.

# 4

Oh nein! Diese Lösung funktioniert nun schon wieder nicht, da der Vektor vom Typ ``std::vector<double>`` ist.

Aber wir können doch nicht schon wieder die gesamte Funktion kopieren …

# 5

Wir lassen stattdessen einfach den Compiler die Funktion intern kopieren.

Dafür verwendet man sogenannte ``templates``, also auf Deutsch _Vorlagen_.

Dazu muss über der Funktion ``template<typename T>`` stehen. Statt ``T`` kann hier auch etwas anderes stehen, aber nach Konvention verwendet man das ``T``.

Das ``T`` steht für den _Typ_.

# 6

Nun ersetzen wir an allen Stellen, den konkreten Typ (also ``int``) mit ``T``.

Der zu übergebene Typ ist nun also ein ``std::vector`` beliebigen Typs.

# 7

Das ``int`` in der Schleife ersetzen wir nun auch mit ``T``.

Jetzt kompiliert das Programm bereits und das Programm funktioniert für jeden Vektor.

# 8

Hier wird jetzt also ein ``std::vector<int>`` an die template-Funktion übergeben, d. h. ``T`` entspricht ``int``.

# 9

Hier wird jetzt also ein ``std::vector<std::string>`` an die template-Funktion übergeben, d. h. ``T`` entspricht ``std::string``.

# 10

Hier wird jetzt also ein ``std::vector<double>`` an die template-Funktion übergeben.

## ``T`` entspricht hier

- ``int``
- ``float``
* ``double``
- ``std::string``

# 11

Eine kleine Unschönheit müssen wir noch beheben:

Da in der Schleife nur ``T`` stand, wurde bei ``std::string`` der String kopiert.

Daher ersetzen wir das noch mit ``const T&``.

Das führt nun beim Int-Vektor zu ``const int&``. Das ist unnötig, aber auch nicht weiter schlimm.

# 12

Anmerkung: Da es hier um den Datentyp geht, kann natürlich auch ``auto`` anstatt ``T`` verwendet werden.
