#include <fstream>
#include <iostream>
#include <vector>
#include <cstdint>

/*
Aufbau: Bitmap-Header

Offset | Größe | Beschreibung
00h    | 2     | Magic Bytes "BM"
02h    | 4     | Größe der Datei in Byte
06h    | 4     | (reserviert)
0Ah    | 4     | Offset, wo Pixeldaten anfangen
0Eh    | 4     | Größe der nachfolgenden Struktur in Byte (immer 40)
12h    | 4     | Höhe in Pixel
16h    | 4     | Breite in Pixel
*/

int main() {
    std::ifstream in("assets/Image.bmp", std::ios_base::in | std::ios_base::binary);

    std::vector<char> header;

    header.resize(2);

    in.read(header.data(), header.size());

    if (header[0] != 'B' || header[1] != 'M') {
        // Fehler!
        return 1;
    }

    int32_t size;
    in.read(reinterpret_cast<char*>(&size), sizeof(int32_t));

    in.seekg(8, std::ios_base::cur);

    int32_t bm_info;
    in.read(reinterpret_cast<char*>(&bm_info), sizeof(int32_t));

    if (bm_info != 40) {
        // Fehler!
        return 1;
    }

    int32_t width;
    int32_t height;

    in.read(reinterpret_cast<char*>(&width), sizeof(int32_t));
    in.read(reinterpret_cast<char*>(&height), sizeof(int32_t));

    std::cout << "Größe: " << size << "\n";
    std::cout << "Breite: " << width << "\n";
    std::cout << "Höhe: " << height << "\n";


    return 0;
}
