# 0

Was objektorientierte Programmierung grundlegend bedeutet, haben Sie in Vorlesung und Praktikum bereits erfahren.

Nun behandeln wir noch die Umsetzung in C++.

# 1

Mit dieser Anweisung wird eine neue Klasse namens ``Point`` definiert.

# 2

Wie auch die Meisten anderen Konstrukte muss für den Inhalt der Klasse eine geschweifte Klammer verwendet werden.

# 3

Dieses Semikolon darf auf keinen Fall vergessen werden (!!!).

Wenn das Semikolon fehlt, ist dies ein Syntaxfehler.

# 4

Dieses ``public`` hier entspricht dem ``+`` bei UML: Alles nach dem ``public`` ist öffentlich zugänglich.

``public`` definiert hier ein Scope, d. h. alles nachfolgende ist öffentlich, bis eine neue Beschränkung (z. B. ``private``) auftaucht.

Das ``public`` ist nicht eingerückt, weil das alle so machen. Methoden und Felder sind wiederum relativ zum ``public`` eingerückt und würde man das ``public`` einrücken, dann hätte man einen Leerraum von 8 vor allen Zeilen. Das sieht halt hässlich aus.

Aber wie Sie ja wissen, ist Modellierung und Programmierung ein kreativer Prozess. Wenn Sie Ihr Kunstwerk mit 8 Leerzeichen schöner finden, dann nehmen Sie halt 8 Leerzeichen. Aber nur, wenn Sie ein eigenes Projekt haben. Wenn Sie irgendwo mitwirken, nehmen Sie die Einrückungstiefe, die schon existiert :).

# 5

Das ist jetzt das Grundgerüst einer Klasse.

In ihrer minimalen Form sieht jede Klasse exakt so aus.

# 6

Mit dieser Syntax erzeugen Sie nun eine Instanz ``p`` von der Klasse ``Point``.

Syntaktisch ist dies identisch zum Erzeugen eines ``std::string`` oder ``std::vector<>``.

# 7

Das hier ist ein Feld.

Jede Instanz der Klasse ``Point`` verfügt also über ein Feld namens ``x``.

Die Zuweisung bedeutet, dass der Wert standardmäßig ``0.0`` ist.

# 8

Jetzt noch das Gleiche für ``y``. Damit kann jetzt unser Punkt im Koordinatensystem platziert werden.

# 9

Wenn der Dokumentationskommentar sehr kurz ist, kann dieser auch auf einer Zeile geschrieben werden.

# 10

Da die Felder ``x`` und ``y`` ``public`` sind, daher können diese direkt gelesen und modifiziert werden.

# 11

Punkte sollen auch schon beim Erzeugen direkt positionierbar sein.

Daher erzeugen wir hier einen _Konstruktor_, der zwei Parameter akzeptiert.

Konstruktoren werden verwendet, um Objekte zu erzeugen. Sie haben den Gleichen Namen, wenn die Klasse.

Zu beachten ist hier, dass Konstruktoren keinen Rückgabewert haben.

# 12

Damit der Konstruktor verwendbar ist, muss er natürlich noch implementiert werden.

Dies sieht syntaktisch fast wie eine Funktion aus.

# 13

Neu ist dieses ``Point::``.

Dies gibt an, dass es ein Teil von der Klasse ``Point`` ist.

# 14

Jetzt kommt noch der Konstruktor (gleicher Namen wie Klassenname) und dessen Parameterliste.

# 15

Jetzt weisen wir die übergebenen Parameter der neuen Instanz der Klasse zu.

``this->`` bezieht sich dabei auf die aktuelle Klasse.

# 16

Auf diese Art kann nun unser neuer Konstruktor verwendet werden, wenn ein neuer Punkt erzeugt werden soll.

# 17

Leider kompiliert nun diese Anweisung nicht mehr.

Dies liegt daran, dass C++ standardmäßig einen parameterlosen Konstruktor generiert, aber nur, wenn es gar keinen anderen Konstruktor gibt.

Da wir nun einen eigenen Konstruktor programmiert haben, müssen wir nun manuell einen parameterlosen Konstruktor implementieren.

# 18

So definiert man einen parameterlosen Konstruktor.

# 19

Und so implementiert man den parameterlosen Konstruktor.

Da ``x`` und ``y`` bereits standardmäßig auf ``0.0`` gesetzt sind, ist hier nichts mehr zu machen.

Das ``// no-op`` ist kurz für _no operation_. Dies schreibe ich immer rein um zu zeigen, dass die Funktion absichtlich keinen Inhalt hat.

Jedenfalls ist unser parameterloser Konstruktor implementiert und die ``main`` kompiliert wieder.

# 20

Jetzt wollen wir noch zwei Methoden programmieren.

Methoden sind Funktionen einer Klasse.

Syntaktisch ist hier gar nichts Neues. Es sieht exakt so aus, wie eine normale Funktion.

Diese Methode akzeptiert eine konstante Referenz auf einen anderen ``Point`` und wird nun unseren Punkt relativ zu diesem Punkt verschieben.

# 21

Dies ist nun die Implementierung: Die beiden Felder werden von ``other_p`` abgefragt und dann auf die Felder des aktuellen ``Point`` aufaddiert.

Auffällig ist hier, dass diesmal vor ``x`` und ``y`` kein ``this->`` steht. Wenn die Namen eindeutig sind (also keine Parameter existieren, die wie Felder der Klasse heißen), kann auf das ``this->`` verzichtet werden.

Hier kann man nun schön sehen, dass bei Konstruktoren, im Gegensatz zu Funktionen/Methoden der Rückgabewert fehlt.

# 22

Diese Methode soll exakt wie die andere ``move_by`` funktionieren.

Der Unterschied ist die Parameterliste: Anstatt eines ``Point`` werden die Koordinaten direkt übergeben.

Mehrere Methoden erhöhen die Flexibilität, wie das Objekt verwendet werden kann.

# 23

Anstatt die Addition von ``x`` und ``y`` nochmal zu implementieren, erzeugen wir einfach einen neuen ``Point`` und übergeben diesen dann an unsere andere ``move_by``-Methode.

Durch diesen Ansatz müssen wir die Implementierung nur an einer Stelle ändern, falls beispielsweise ein Fehler gefunden wurde. Fehler sind hier natürlich quasi ausgeschlossen, aber wenn die Funktion 50 Zeilen hat, ist der Vorteil besser sichtbar.

# 24

Für die Debugausgabe auf der Konsole definieren wir nun noch eine Funktion ``print``.

Mit dieser Funktion wird es übrigens nicht möglich sein, den Punkt in folgendem Kontext zu verwenden: ``std::cout << p << "\n"``

Damit dies funktioniert, benötigt man Operator-Überladung. Dieses Thema werden wir aber nicht behandeln, da es nie benötigt wird und daher für einen Einsteigerkurs ungeeignet ist.

# 25

Stattdessen machen wir es einfach so: Bei Aufruf der Methode wird ``Point(x, y)`` auf der Konsole ausgegeben.

# 26

Jetzt können wir das alles testen. Dazu hier einige Beispiele.

## Wo ist ``r`` nach dieser Zeile?

- Bei ``(1, -3)``
- Bei ``(2, 4)``
* Bei ``(3, 1)``
- Bei ``(2, -3)``

# 27

Die direkte Zuweisung funktioniert übrigens auch. Dafür wird ein _copy constructor_ (Kopie-Konstruktor) verwendet.

Dieser wird von C++ automatisch generiert, wenn man ihn nicht selbst schreibt.

Mit dem Thema _copy constructor_ wollen wir uns aber nicht weiter befassen. Die generierte Variante funktioniert in den meisten Fällen problemlos.
