# 0

Ein Problem bei Klassen ist die Fehlerbehandlung.

In Konstruktoren können Fehler durch nicht durch die Rückgabe eines Fehlercodes behandelt werden.

Eine Fehlerquelle wäre beispielsweise ein Kreis, der keinen negativen Radius haben darf.

Wie wir mit solchen Fehlern umgehen, schauen wir uns zuerst außerhalb von Klassen an.

Dazu werfen wir einen Blick auf ``std::stoi``. Diese Funktion ist bereits bekannt: Sie wandelt einen String in eine Zeichenkette um.

Wenn der String nicht konvertierbar ist, lässt ``std::stoi`` das Programm abstürzen. In Wirklichkeit wirft diese Funktion aber eine _Exception_ (Ausnahme). Exceptions können auch in Konstruktoren verwendet werden.

Exceptions können zudem behandelt werden, damit das Programm nicht abstürzt.

# 1

Zuerst erstellen wir eine Endlosschleife.

Am Ende soll die Schleife so lange laufen, bis eine gültige Zahl eingegeben wurde.

# 2

Auf diese Weise wird eine Exception behandelt.

Gehen wir dies einmal Schritt für Schritt durch.

# 3

Im ``try``-Block muss die Anweisung stehen, die eine Exception erzeugen kann. In unserem Fall also die ``std::stoi``.

Wenn keine Exception auftritt, werden alle ``catch``-Blöcke ignoriert und das Programm wird weiter ausgeführt.

Wenn ``std::stoi`` die Exception auslöst, wird die Ausführung von ``try`` direkt abgebrochen und ein passender ``catch``-Block ausgeführt. Weitere ``catch``-Blöcke werden ignoriert und das Programm wird weiter ausgeführt.

Wenn es eine Exception gibt, aber keinen passenden ``catch``-Block, stürzt das Programm kontrolliert ab.

# 4

Im ``catch``-Block werden Exceptions behandelt. Die Klammer hinter ``catch`` enthält die zu fangende Exception und der Block enthält dann den auszuführenden Code, wenn die Exception auftritt.

``std::stoi`` kann zwei Arten von Exceptions erzeugen: ``std::invalid_argument`` und ``std_out_of_range``.

Welche Exceptions geworfen werden können, muss in der Dokumentation nachgelesen werden.

``std::invalid_argument`` tritt auf, wenn die Zeichenkette keine Zahl ist.

``e.what()`` gibt die Fehlermeldung aus.

# 5

``std::out_of_range`` tritt auf, wenn die Zahl zu groß oder zu klein ist für den Wertebereich.

Also der String ist eine Zahl, aber diese passt nicht in einen 32-Bit Integer.

# 6

Wenn keine Exception erzeugt wurde wissen wir, dass es sich um eine passende Zahl handeln muss. In diesem Fall können wir also die Schleife unterbrechen.

# 7

Beide Exceptions erben übrigens von ``std::logic_error``.

Wenn einen die konkrete Exception also nicht interessiert, kann hier auch ``logic_error`` verwendet werden.
