# 0

Das vorherige Problem, wo nicht klar ist, wer löschen soll, werden wir nun mit einem Smart Pointer lösen: dem ``std::unique_ptr``.

_Smart Pointer_ und existieren seit dem C++11 (2011) Sprachstandard.

_Smart Pointer_ existieren auf dem _Stack_, verwalten aber Speicher auf dem _Heap_. Der ``std::unique_ptr`` löscht das Objekt automatisch, wenn es aus dem Scope fällt. Ein Aufruf von ``delete`` ist nicht notwendig.

Zudem wird durch den ``std::unique_ptr`` dem Entwickler signalisiert, wem das Objekt gehört. Wenn Sie etwas mehr mit C++ programmieren, werden Sie merken, dass ``std::unique_ptr`` unglaublich nützlich ist.

# 1

Für _Smart Pointer_ benötigen wir den neuen Header ``<memory>``.

# 2

Dies ist die Typdefinition von einem ``std::unique_ptr`` der einen ``Point*`` verwaltet. Der ``*`` wird beim Typ aber nicht mit angegeben.

# 3

``std::unique_ptr`` werden üblicherweise mit der Funktion ``std::make_unique<DATENTYP>`` erzeugt. Wobei ``DATENTYP`` der Typ des Objektes ist, dass erzeugt werden soll.

Dies ist eine sogenannte Fabrikfunktion (_Factory function_). Solchen Funktionen geben beim Aufruf Objekte zurück.

# 4

Die an ``std::make_unique`` übergebenen Parameter werden an den Konstruktor der Klasse ``Point`` übergeben.

Intern passiert hier also ``new Point(where, where)``.

Die beiden Parameter müssen natürlich nicht identisch sein, sie sind es nur in diesem Beispiel, weil die Funktion ``make_point`` so aufgebaut ist.

# 5

Durch ``std::make_unique`` ist der Datentyp bereits klar, daher kann hier auch einfach ``auto`` verwendet werden.

# 6

Die Funktion muss nun auch diesen Datentyp zurückgeben.

# 7

Hier wurde jetzt durch die Rückgabe der Besitz von ``p`` in ``make_point`` auf ``q`` in der ``main`` übertragen.

Es gibt also weiterhin nur einen Besitzer.

# 8

``std::unique_ptr`` sind übrigens nicht kopierbar. Diese Anweisung kompiliert also nicht.

# 9

Es kann aber dennoch ein normaler Pointer auf das Objekt erzeugt werden.

Sofern der ``std::unique_ptr`` zerstört wird, zeigt der Pointer aber auch wieder ins Leere.

Wenn es mehrere Besitzer gibt, sind ``std::shared_ptr`` und ``std::weak_ptr`` eventuell besser. Siehe dazu die nächsten Storyteller.

# 10

Das hier ist Verboten! Der Pointer gehört dem ``std::unique_ptr`` und nicht ``r``. Das ist so, als würde man bei einem Freund die Wände streichen, weil einem die Farbe nicht gefällt.

Zudem verursacht dies einen _double free_, wenn der ``std::unique_ptr`` am Ende versucht das Objekt zu löschen.

# 11

Mit ``reset()`` wird der ``std::unique_ptr`` freigegeben und das verwaltete Objekt gelöscht.

Hier kann auch direkt mit ``new`` ein neues Objekt dem ``std::unique_ptr`` zugeordnet werden.

Hier steht ein ``new``, gelöscht werden muss aber nicht, da es der ``std::unique_ptr`` verwaltet.

# 12

Dies hier erzeugt einen neuen ``std::unique_ptr`` aus einem Pointer, der bereits in einem anderem ``std::unique_ptr`` ist.

Dies ist ebenfalls verboten, da nun das gleiche Objekt von zwei ``std::unique_ptr`` verwaltet wird.

Dies hat ebenfalls einen _double free_ zur Folge.

Wenn es zwei Besitzer gibt, muss ein ``std::shared_ptr`` verwendet werden. Siehe dazu die nächsten Storyteller.

# 13

Der Besitz kann aber explizit mit ``std::move`` übertragen werden.

``q`` gibt hier den Besitz auf und überträgt den Besitz an ``s``. Somit gibt es wieder nur einen Besitzer und alle sind glücklich.

# 14

Hier fällt der ``std::unique_ptr`` wieder aus dem Scope und der von ihm verwaltete Speicher wird freigegeben.

