# 0

Im vorherigen Beispiele haben wir weitere Tücken der C++-Speicherverwaltung kennengelernt. Dies lässt sich aber auch mit _Smart Pointer_ ziemlich gut lösen.

Wenn es mehrere Referenzen auf einen Pointer gibt, also der Besitz unklar ist, und man den Speicher erst freigeben möchte, wenn es keine Referenzen mehr gibt, kann ein ``std::shared_ptr`` verwendet werden.

Dies ist neben ``std::unique_ptr`` ein weiterer _Smart Pointer_.

# 1

``std::shared_ptr`` ist im gleichen Header wie ``std::unique_ptr``.

# 2

Anstatt ``std::make_unique`` verwenden wir hier ``std::make_shared``. Der Datentyp zur linken ist dann ``std::shared_ptr<Point>``.

Zur Erinnerung: Die an ``std::make_shared`` übergebenen Parameter werden an den Konstruktor der Klasse ``Point`` übergeben.

Intern passiert hier also ``new Point(1, 2)``.

# 3

``q`` ist nun ein zweiter ``std::shared_ptr``, der auf den ``Point`` zeigt.

Der ``std::shared_ptr`` merkt sich, wie viele Referenzen es auf diesen gibt.

Zur Erinnerung: Bei ``std::unique_ptr`` war dies nicht möglich. Dort konnte nur der Besitz mit ``std::move`` übertragen werden. Hier gibt es jetzt aber zwei Besitzer.

# 4

Dereferenzieren und eine Referenz erhalten funktioniert weiterhin, aber Vorsicht: Der ``std::shared_ptr`` weiß nicht, dass diese Referenz existiert.

# 5

Dies funktioniert fast wie beim ``std::unique_ptr``: Anstatt das Objekt direkt zu löschen, wird in ``reset()`` zuerst der Referenzzähler um eins verringert. Wenn dieser ``0`` erreicht, wird das Objekt gelöscht.

Das Objekt wird hier nicht gelöscht, da noch ``std::shared_ptr q`` auf dieses zeigt.

# 6

Diese Zeilen stürzen diesmal nicht ab, da das Objekt noch nicht gelöscht wurde.

# 7

Nach diesem Aufruf gibt es keine ``std::shared_ptr`` mehr, die auf dieses Objekt zeigen. Das Objekt wird nun gelöscht.

# 8

Diese Anweisung stürzt aber weiterhin ab, da die Referenz auf ein gelöschtes Objekt zeigt.

# 9

Eine Art von verwalteter Referenz, die aber den Referenzzähler nicht erhöht, ist ein ``std::weak_ptr``.

# 10

``lock()`` erzeugt aus diesem ``std::weak_ptr`` einen ``std::shared_ptr``, aber nur, wenn das Objekt noch nicht gelöscht wurde.

Wenn es schon gelöscht wurde, gibt ``lock()`` einen ``nullptr`` zurück. Dies ist aber besser als eine reine Referenz, diese kann nicht ermitteln, ob das Objekt gelöscht wurde. Der ``std::weak_ptr`` kann es.

# 11

Es existiert zwar noch ein ``std::weak_ptr`` auf den ``Point``, aber da dieser nicht den Referenzzähler erhöht, wird das Objekt trotzdem zerstört.

# 12

Diese Bedingung ist nur wahr, wenn das Objekt noch nicht zerstört wurde.

Das Programm stürzt nun also nicht mehr ab, da vorher überprüft werden kann, ob das Objekt noch aktiv ist.

# 13

Genau wie bei ``std::unique_ptr`` kann auf diese Art ein ``std::shared_ptr`` mit einem neuen Objekt befüllt werden.

Zur Erinnerung: Hier steht ein ``new``, gelöscht werden muss aber nicht.

# 14

Am Funktionsende werden die ``std::shared_ptr`` zerstört und geben den Speicher frei. Es gibt also keinen _Memory Leak_ mehr und ``delete`` kann man auch nicht vergessen. Super!
