# 0

Nun fangen wir mit SDL2.

SDL2 ist eine Bibliothek, welche Komponenten des Betriebssystems abstrahiert.

Dazu gehören unter anderem Grafik, Ton und Eingabe.

In diesem Storyteller beschäftigen wir uns mit der Initialisierung von SDL2.

Diese sieht fast immer gleich aus.

# 1

Um SDL2 verfügbar zu machen, muss der Header ``SDL.h`` eingebunden werden.

# 2

Wichtig ist die Signatur von ``main``.

Wir erinnern uns, dass es hier zwei Möglichkeiten gibt:

- ``int main()``: Dies ist die parameterlose ``main``Funktion
- ``int main(int argc, char** argv)``: Bei dieser ``main`` können Kommandozeilenparameter übergeben werden.

SDL2 benötigt unbedingt die letztere, sonst funktioniert das Programm nicht unter Windows (technische Gründe, die ich nicht weiter ausführen möchte). Die Parameternamen ``argc`` und ``argv`` werden weggelassen, damit es keine Compiler-Fehler gibt.

# 3

Die komplette Initialisierung von SDL2 findet aus Gründen der Lesbarkeit in ``SDL_Helper`` statt.

Dies ist eine von mir bereitgestellte Klasse. Schauen wir uns diese doch einfach mal an.

# 4

Die gesamte Initialisierung findet im Konstruktor statt. SDL2 ist also sofort verfügbar, sobald eine Instanz von ``SDL_Helper`` erzeugt wird.

# 5

Diese Variablen enthalten die zukünftige Fensterbreite und Fensterhöhe (hier: 320×240).

# 6

Diese Option behebt Probleme auf einigen Linux-Systemen. Möchte ich nicht erläutern und benötigt man eigentlich auch nicht.

# 7

Mit ``SDL_Init`` wird SDL2 initialisiert. Da SDL2 eine C-Bibliothek ist, gibt es keine Objektorientierung. Stattdessen existieren globale Funktionen, um mit SDL2 zu arbeiten.

``SDL_INIT_EVERYTHING`` initialisiert alles was SDL2 zu bieten hat. Mit ``SDL_INIT_VIDEO`` wird nur das Videosystem initialisiert.

Mehrere Optionen können bitweise Oder-verknüpft werden: ``SDL_INIT_VIDEO | SDL_INIT_AUDIO`` initialisiert Video und Ton.

# 8

Sofern irgendwas fehlschlägt wirft der Konstruktor eine Ausnahme.

Mit ``SDL_GetError()`` kann der letzte Fehler ermittelt werden.

# 9

Mit ``SDL_CreateWindow`` wird ein Fenster erzeugt.

SDL2 kann mehrere Fenster verwalten, aber im Regelfall existiert nur eins.

Die Parameter sind wie im Kommentar erläutert:

1. Der Titel des Fenster (hier: ``SDL Demo``)
2. X-Koordinate (obere linke Ecke) des Fensters, wo es erzeugt werden soll. ```SDL_WINDOWPOS_CENTERED`` zentriert das Fenster.
3. Y-Koordinate (obere linke Ecke) des Fensters, wo es erzeugt werden soll. ```SDL_WINDOWPOS_CENTERED`` zentriert das Fenster.
4. Fensterbreite (``320``)
5. Fensterhöhe (``240``)
6. Flags an den Fenstermanager. Kann wieder bitweise Oder-verknüpft werden. ``SDL_WINDOW_RESIZABLE`` bedeutet, dass die Fenstergröße mit der Maus geändert werden kann.

# 10

Das Fenster wird in einem ``std::unique_ptr`` namens ``window`` abgelegt. Dies geht eigentlich nur bei C++-Objekten.

Hier erläutere ich kurz den Trick.

# 11

Bei ``std::unique_ptr`` kann in ``<>`` ein zweiter Parameter übergeben werden. Dieser Parameter ist der Klassenname für einen _Deleter_. Also eine Klasse die weiß, wie ``SDL_Window`` zu löschen ist.

# 12

``SdlDeleter`` wird von mir mitgeliefert und ist ein ``struct``. Ein ``struct`` stammt aus C und funktioniert bei C++ fast wie eine Klasse, aber standardmäßig ist alles ``public`` und nicht ``private``.

# 13

Mit dieser Syntax wird signalisiert, dass Objekte vom Typ ``SdlDeleter`` wie eine Funktion aufgerufen werden können.

Folgende Syntax ist dann möglich:

```
SDL_Window* window = // Fenster erstellen
SdlDeleter deleter;
deleter(window); // Sieht aus wie ein Funktionsaufruf
```

Beim Löschen ruft der ``std::unique_ptr`` die Klasse wie dargestellt auf und ``SDL_DestroyWindow`` gibt dann den Speicher frei.

# 14

Das ``SDL_Window`` ist nur das Fenster. Damit etwas dargestellt werden kann, muss noch ein ``SDL_Renderer`` erzeugt werden.

Dieser stellt die Funktionalität für das Zeichnen im Fenster zur Verfügung.

# 15

Mit ``SDL_RenderClear`` wird der Inhalt des Fensters komplett übermalt.

Die Farbe kann mit ``SDL_SetRenderDrawColor`` konfiguriert werden. Standardmäßig ist diese schwarz.

Diese Änderung erscheint aber nicht auf dem Bildschirm.

# 16

Mit ``SDL_RenderPresent`` werden die Änderungen sichtbar gemacht.

Intern werden zwei Grafikpuffer verwendet: ein Frontbuffer und ein Backbuffer.

Der Frontbuffer ist der gerade sichtbare Bildschirminhalt.

In den Backbuffer wird gemalt und beim Aufruf von ``SDL_RenderPresent`` werden die zwei Puffer einfach ausgetauscht.

Dadurch flimmert es nicht.

# 17

``SDL_RendererSetLogicalSize`` gibt an, wie groß der Renderer unabhängig von der Fenstergröße sein soll.

In diesem Fall wird er auf 320×240 gesetzt. Wenn das Fenster vergrößert wird, skaliert nun SDL2 den Inhalt automatisch, wodurch der Inhalt unscharf wird.

# 18

``~SDL_Helper`` ist ein sogenannter _Destruktor_. Den haben wir nicht behandelt.

Diese Methode wird aufgerufen, wenn das Objekt zerstört wird.

# 19

Mit ``SDL_Quit()`` wird SDL2 beendet.

# 20

Mit ``get_renderer`` wird der aktuelle ``SDL_Renderer`` zurückgegeben, damit das Programm zeichnen kann.

Wie das geht sehen wir im nächsten Storyteller.
