---
title: "Prozedurale Programmierung -- Einführung"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: März 2022
---

## Über mich

- Gabriel Kind, M.Sc.
- 2008-2013: Studium an der HS Mittweida (Informatik/Bioinformatik)
- 2014-2017: Forschungsprojekt Cyanofactory
- 2018-2020: MINT-Kompass
- seit 09/2019: Lehrkraft für besondere Aufgaben
	- WS19: Rechnerarchitektur
	- seit WS20: Grundlagen der Informationstechnologie
	- seit SS21: Prozedurale Programmierung (C++) & Introduction to Programming (Python)

## Kontakt

- Sprechzeiten: Mi 14:00 - 18:00 Uhr (bitte vorher anmelden) und jederzeit per Mail
- Haus 8, Raum 8-207
- Digitale Sprechstunde möglich (Webcam—Pflicht!)
- Mail: <kind@hs-mittweida.de>
- Im Betreff "[ProzProg]" oder "\[C++\]" verwenden
- **Wichtig**: Nennen Sie Ihre Seminargruppe
- Tel: +49 3727 58-1013 (nur in dringenden Fällen)

## Kontakt

![Position von Haus 8 © OpenStreetMaps](Images/Haus8.png)

## Kommunikation während der Vorlesung

- Für Fragen in der Vorlesung gibt es Textchat und Mikrofon
- Bei Mikrofon vorher "Hand heben" und warten, dass ich Sie Aufrufe

# Über das Modul

## Teilnehmer

- ~~Betriebswirtschaft~~
- Elektrotechnik - Automation
- Lasertechnik / Physikalische Technik
- ~~Maschinenbau~~
- ~~Media and Acoustical Engineering~~
- Wirtschaftsingenieurwesen

## Modulbeschreibung (laut Modulhandbuch)

- Das Handbuch für Wirtschaftsingenieurwesen konnte ich nicht finden
- Nur E-Technik und Lasertechnik
- Der typische Adressat dieser Veranstaltung ist der Techniker oder Ingenieur, der an der Schnittstelle zur Hardwareentwicklung unter anderem eingebettete Systeme und Mikrocontroller programmiert. Im Gegensatz zum Folgemodul "Grundlagen Mikroprozessortechnik" bezieht sich der Inhalt aber auf keine spezielle Plattform.
- Bei Lasertechnik existiert das Modul "Mikroprozessortechnik" gar nicht ;)

## Modulbeschreibung (laut Modulhandbuch, 2)

[...] die Teilnehmer [lernen] die Programmiersprache C/C++ zu beherrschen. Über die reine Vermittlung der Syntax hinaus, werden die Grundlagen von Betriebssystemen im Kontext der Programmierung, die Verwendung der Kommandozeile und von freien Entwicklungswerkzeugen behandelt. Unabhängig von der Syntax einer Sprache lernen die Teilnehmer klassische Algorithmen und Datenstrukturen kennen, die später als Repertoire zur Lösung der unterschiedlichsten Probleme eingesetzt werden können.

## Lehrinhalte (laut Modulhandbuch)

- ~~Einführung zum Aufbau und zur Funktionsweise von Computern und Betriebssystemen, insbesondere in Hinblick auf eingebettete Systeme~~
- Syntax der Programmiersprache C/C++
- ~~prakt. Nutzung der Kommandozeile~~, Kompiler Toolchain
- Programmierwerkzeuge und Entwicklungsumgebungen zur Softwareentwicklung
- Algorithmen und Datenstrukturen (interne Informationsdarstellung, einfache und komplexe Datentypen)

## Weitere Lehrinhalte (die nicht im Modulhandbuch stehen)

- Primitive Datentypen, Bedingungen und Schleifen
- Funktionen
- Objektorientiere Programmierung (Klassen)
- UML-Klassendiagramme & Struktogramme
- 2D-Grafik mit SDL2

## Literatur

- <https://cplusplus.com>: DIE C++ Referenz
- <https://stackoverflow.com>: Antworten auf fast alle Programmierprobleme
- Falls Sie Bücher mögen: Gottschling, P. (2019): "FORSCHUNG mit
modernem C++: C++17-INTENSIVKURS für Wissenschaftler,
Ingenieure und Programmierer". Hanser Verlag

## Vorlesung

- 14 Veranstaltungen, je Do 15:45 hybrid (Präsenz & Digital)
- Flipped Classroom
- Wird teilweise aufgezeichnet

## Flipped Classroom

- Didaktisches Konzept
- Lehrinhalte werden im Voraus bereitgestellt (geplant: Do oder Fr)
- Studierende arbeiten diese bis zur kommenden Vorlesung durch
- Vorlesung selbst:
    - Fragen klären
    - Anwendungsbeispiele
    - Ausblick auf kommende Woche

## Flipped Classroom (2)

- Lehrinhalte können in eigenem Tempo gelernt werden
- Beim Programmieren funktioniert reiner Frontalunterricht schlecht

## Reihenfolge

1. Sie bekommen im Anschluss an die Vorlesung Lehrinhalte - durchzuarbeiten bis V02
2. V02: Klären von Fragen & Anwendungsbeispiele, Bereitstellung Inhalte für V03
3. P02: Selbstständiges Bearbeiten von Aufgaben basierend auf Inhalten von V02
4. usw.

## Planung

![](Images/Ablauf.png)

## Seminar/Praktikum

- 1x die Woche, Zeit siehe Stundenplan bzw. Planung auf Moodle
- Erstmal digital über Zoom (Link auf Moodle)
- Ab 25.4. Präsenz und ein Online-Ausweichtermin

## Prüfung

- "Unter Stress programmieren" (90 Min.) funktioniert nicht
- Take Home Exam / "Beleg Lite"
- Bekommen Aufgaben für zu Hause
- Einige Stunden bis Tage Zeit (noch nicht festgelegt) zum Bearbeiten, dann Abgabe
- Ermöglicht einfachere Bewertung als ein Beleg (und ist weniger aufwendig für beide Seiten)

## Vorherige Veranstaltung

Die Inhalte vom letzten Jahr finden Sie auf <https://moodle.hs-mittweida.de/course/view.php?id=700>

# C++

## Geschichte & Eigenschaften

:::::::::::::: {.columns}
::: {.column}

- 1979 von Bjarne Stroustrup bei AT&T entwickelt
- Erweiterung der Programmiersprache C
- Hardwarenah (Kompilierung nach Maschinencode)

:::
::: {.column}

![Ein Foto von Bjarne Stroustrup (GFDL)](Images/BjarneStroustrup.jpg)

:::
::::::::::::::

## Verwendete Technologien

![C++ Ökosystem, laut Jetbrains-Umfrage 2020 ([Quelle](https://www.jetbrains.com/lp/devecosystem-2020/cpp/))](Images/Technologien.png)

## Kompilieren

- Kompilieren ist das umwandeln des Quellcodes in Maschinencode
- Dies macht ein Compiler
- Für C++ gibt es mehrere Compiler:
	- Windows: cl (von Visual Studio)
	- Linux: g++ oder clang++
	- macOS: clang++

## Vergleich C++ mit Python

- Python-Kenntnisse auf C++ übertragbar
- Syntaktische Abweichungen
- Python wird interpretiert, C++ kompiliert

## Anweisungen

:::::::::::::: {.columns}
::: {.column}

Anweisungsende mit Semikolon

```C++
a = 1 + 2;

b = 1 +
    2;
```

:::
::: {.column}

Anweisungsende am Zeilenende

```Python
a = 1 + 2

b = 1 + \
    2
```

:::
::::::::::::::

## Module

:::::::::::::: {.columns}
::: {.column}

Import mit ``#include``

```C++
// Text Ein-/Ausgabe
#include <iostream>

// Mathematische Funktionen
#include <cmath>
```

:::
::: {.column}

Import mit ``import``

```Python
# Ein-/Ausgabe
import io

# Mathematische Funktionen
import math
```

:::
::::::::::::::

## Blöcke

:::::::::::::: {.columns}
::: {.column}

Geschweifte Klammern

```C++
if (a > 3) {
    std::cout << "a ist größer als 3\n";
}
```

:::
::: {.column}

Einrückung mit Leerzeichen

```Python
if a > 3:
    print("a ist größer als 3")
```

:::
::::::::::::::

## Typisierung

- Statisch typisiert: Der Datentyp muss angegeben werden
- Typfehler werden bei der Kompilierung erkennt
- Bei Python fallen diese erst zur Laufzeit auf (dafür aber flexibler)

C++:

```C++
int zahl = 1;          // zahl ist Ganzzahl
string text = "Hallo"; // text ist String
zahl = "2";            // FEHLER: Keine Zahl
```

Python:

```Python
zahl = 1               # zahl ist Ganzzahl
text = "Hallo"         # text ist String
zahl = "2"             # OK, zahl ist nun String
```

# Ein erstes C++-Programm

## Ein erstes C++-Programm

```C++
#include <iostream>

using namespace std;

int main() {
    cout << "Hallo Welt!\n";

    return 0;
}
```

## Parameter

```C++
#include <iostream>

using namespace std;

int main(int argc, char** argv) {
    cout << "Hallo Welt!\n";
    cout << "Ich heiße " << argv[0] << " und habe "
        << argc << " Parameter\n";

    return 0;
}
```

## Texteingabe

```C++
#include <iostream>
#include <string>

using namespace std;

int main(int argc, char** argv) {
    cout << "Hallo Welt!\n";
    cout << "Ich heiße " << argv[0] << " und habe "
         << argc << " Parameter\n";

    cout << "Wie heißt du?\n";
    string name;
    cin >> name;
    cout << "Hallo " << name << "\n";

    return 0;
}
```

# Kompilieren mit CMake

## CMake

- CMake ist ein Programm um plattformunabhängig C++-Code zu kompilieren
- Läuft unter Windows, Linux, macOS
- Der quasistandard für C++-Entwicklung
- Von fast allen IDEs (außer XCode) unterstützt

## Eine einfache CMakeLists.txt

```CMake
cmake_minimum_required(VERSION 3.9)

project(Vorlesung VERSION 1.0 LANGUAGES CXX)

set(CMAKE_CXX_STANDARD 17)

add_executable(erstes_programm
    main.cpp
)
```

## Buildschritte (macht VsCode automatisch)

1. ``cmake .``
2. ``cmake --build .``
3. ``erstes_programm`` kann nun ausgeführt werden

# Danke für Ihre Aufmerksamkeit! Fragen?
