---
title: "Prozedurale Programmierung -- Rechnerarchitektur"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: März 2022
---

## Von-Neumann-Architektur

:::::: {.columns}
::: {.column}

Nahezu alle Rechner heutzutage basieren auf der **von-Neumann-Architektur**

:::
::: {.column}

![Aufbau Von-Neumann-Architektur (Lukas Grossar, Public Domain)](Images/Von-Neumann-Architektur.png)

:::
::::::

## Von-Neumann-Architektur

- Programmcode und Daten belegen denselben Speicher
- Die Größe einer Speicherzelle ist die **Adressbreite** des Prozessors (heutzutage: 32/64-Bit)
- Die Bedeutung eines Bitmuster hängt vom **Kontext** ab
- 41h kann sein: (siehe auch **Vorlesung 3**)
	- Die Dezimalzahl 65d
	- Der Buchstabe A
	- Der x86 Op-Code "INC CX"
	- u.a.

## Von-Neumann-Architektur

Konsequenzen von geteiltem Daten- und Programmspeicher:

- Durch Programmfehler können Daten als Programmcode ausgeführt werden (Sicherheitslücke)
- Überschreiben von Programmcode möglich ("Self Modifying Code")
- Programmcode kann zur Laufzeit generiert werden (JIT Compiler: **J**ust **I**n **T**ime)

## Stack und Heap

- Daten werden in C++ auf dem Stack bzw. dem Heap gespeichert
- Vom Prinzip her sind dies einfach zwei Bereiche im Speicher, die unterschiedlich verwendet werden

## Stack

- Lokale Variablen (Variablen, die in Funktionen definiert sind), werden auf dem Stack gespeichert
- Jede Funktion hat einen _Stackframe_
- Der Stackframe ist der Bereich des Stacks, wo die Variablen der konkreten Funktion liegen
- Operationen auf dem Stack: Push (füge hinzu), Pop (nimm herunter)

## Push/Pop illustriert

![(Originalquelle und Lizenz unklar)](Images/Pusheen.jpg)

## Heap

- Für "langlebige Objekte" oder wo der Stack zu klein ist
- Für dynamische Speicheranforderungen (z.B. String vergrößern)
- Heapspeicher kann mit ``new`` angefordert werden (behandeln wir später noch konkret)
- C++-Objekte verwenden den Heap-Speicher, ohne das der Programmierer etwas davon mitbekommt

```cpp
int zahl = 1; // 1 liegt auf dem Stack
// Auf dem Stack liegt nur eine Adresse (Pointer)
// Der String selbst liegt auf dem Heap
std::string wort = "Hallo"
```

## Zahlensysteme am Computer

- Binär (Basis 2)
- Hexadezimal (Basis 16)
- Hex ist eine kompakte Darstellung von Binärzahlen
- Der Computer arbeitet immer mit Binärzahlen, weil die Schaltungen dafür einfacher sind

## Warum 4er-/8er-Blöcke

- Speicheradressierung seitens des Prozessors
- Durchgesetzt hat sich ein vielfaches von 8
- Z80: 8-bit Prozessor (GameBoy, manche Taschenrechner)
- Intel 8086: 16-bit Prozessor (zu MS-DOS-Zeiten)
- 32-Bit Prozessor (z.B. Pentium II, viele Android Smartphones)
- 64-bit Prozessor (fast alles ab Core2 Duo, 2006)

## Hochsprachen

Die Datentypen von Programmiersprachen entsprechen in der Regel Prozessortypen

- ``char`` (character): 8
- ``short``: 16
- ``int`` (Integer), ``long``: 32

## Überlauf

- In einer Speicherzelle mit Datenbreite 8 Bit liegt der Wert ``1111 1111b`` (255d bzw -1d)
- Der Wert wird um ``1`` erhöht:
- ``1111 1111b + 1d = 1 0000 0000b``
- Die ``1`` gehört NICHT zum neuen Wert, sondern ist ein "Overflow" (wird vom Prozessor signalisiert)
- Der neue Wert ist somit ``0000 0000b``

## Undefiniertes Verhalten

- In C++ ist vorzeichenbehafteter Überlauf "undefiniertes Verhalten" (undefined behaviour)
- Widerspruch zwischen "realer Welt" und "Sprachstandard"
- Mittlerweile verwendet jeder Computer das Zweierkomplement, daher ist der Überlauf wohldefiniert
- Compiler darf annehmen, dass so ein Überlauf nicht möglich ist

# Danke für Ihre Aufmerksamkeit! Fragen?
