---
title: "Prozedurale Programmierung -- Datentypen & Datenstrukturen"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: April 2022
---

## Vorwort

- C++ ist von 1979, hat daher viele störende Altlasten, aber wird mit der Zeit besser
- Es wirkt so, als wäre Python gegenüber C++ weit überlegen
- Kleine Projekte sind mit dynamisch typisierten Sprachen schneller entwickelt
- Statische Sprachen (wie C++) punkten bei größeren Projekten aufgrund der Typisierung (bessere Fehlererkennung)
- Einige Dinge sind in C++ besser als in Python umgesetzt (dazu kommen wir noch)
- Nur EA/LT:
  - C++ lässt sich viel leichter einbetten und portieren als Python
  - C++ läuft fast überall

# Umfragen

## Code-Einrückung

Die elementare Frage:

Wie rücke ich den Quelltext ein?

Optionen:

- Leerzeichen
- Tabulator

## Einrückungstiefe je Ebene

:::::: {.columns}
::: {.column}

2:

```cpp
int main() {
  if (erg > 0) {
    erg = erg + 1;
  }
  return 0;
}
```

4:

```cpp
int main() {
    if (erg > 0) {
        erg = erg + 1;
    }
    return 0;
}
```

:::
::: {.column}

6:

```cpp
int main() {
      if (erg > 0) {
            erg = erg + 1;
      }
      return 0;
}
```

8:

```cpp
int main() {
        if (erg > 0) {
                erg = erg + 1;
        }
        return 0;
}
```

:::
::::::

## Leerräume

Syntaktisch dienen Leerräume fast immer nur der Lesbarkeit:

- ``1+2*3/(1+4)`` ist äquivalent zu
- ``1 + 2 * 3 / (1 + 4)``

## Variablennamen

Variablen (und später auch Funktionen) können auf verschiedene Arten benannt werden

- Snake Case: ``das_ist_eine_variable``
- Camel Case: ``dasIstEineVariable``
- Pascal Case: ``DasIstEineVariable``

## Klammer-Positionierung

:::::: {.columns}
::: {.column}

K&R:

```cpp
if (erg > 0) {
    erg = erg + 1;
}
 
```

Allman:

```cpp
if (erg > 0)
{
    erg = erg + 1;
}
```

:::
::: {.column}

Whitesmith:

```cpp
if (erg > 0)
    {
    erg = erg + 1;
    }
```

GNU:

```cpp
if (erg > 0)
  {
    erg = erg + 1;
  }
```


:::
::::::

# Datentypen

## Datentypen

Welchen Datentyp sollte ich verwenden?

## Ganzzahlen

``int`` (4 Byte)

Ausnahmen:

- Berechnungen, bei denen 4 Byte zu wenig sind und überlaufen
- hardwarenahe Anwendungen, wo die konkrete Größe relevant ist

## Gleitkomma

``double`` (8 Byte)

Ausnahmen:

- Geschwindigkeit wichtiger als Genauigkeit (Bei vielen Systemen gibt es aber kaum messbare Unterschiede)
- Hardware hat keine Gleitkommaeinheit (FPU), dann möglichst komplett auf Gleitkomma verzichten, da sehr langsam (in Software umgesetzt)

## Zeichenketten

- ``std::string`` für Zeichenketten
- ``char`` für einzelne Zeichen (Achtung: Nur wenn das Zeichen im ASCII-Zeichensatz ist. Bei allen anderen Dingen wie Umlauten etc. auch hier ``std::string`` verwenden)
- C verwendet ``char*`` - Hier kann man sehr viel falsch machen bei der Verwendung. Wenn man keine Ahnung hat diesen Typ _nicht verwenden_.

## Arrays (Listen)

- ``std::vector<TYP>``
- Bei allen Dingen, wo eine Liste naheliegend ist: ``vector``.
- Die Antwort ist immer ``vector``.
- Ich möchte...? Nimm ``vector``.
- Aber für konstant große Listen geht doch auch ``std::array``? Ja, das geht aber auch mit ``vector``. 

# Arrays und Strings

## Datenstruktur (siehe Vorlesungsvideo)

Wenn ich hier von Array spreche schließe ich ``std::vector`` mit ein.

Array ist der Fachbegriff aus der Informatik für Daten, die sequentiell nebeneinander liegen und dadurch eine Liste bilden.

In fast allen Fällen sind Arrays eindimensional, d.h. 1D-Arrays.
 
## Komplexität

- Operationen auf Datenstrukturen werden in der theoretischen Informatik untersucht
- Dafür verwendet man Komplexitätsklassen
- Geben an, wie viele Operationen im Schnitt durchgeführt werden
- Landau-Notation oder Big-O-Notation
- Symbol Θ
- (Müssen Sie nicht wissen, ist nur eine Randbemerkung)

## Operationen auf Arrays (siehe Vorlesungsvideo)

Wir untersuchen beispielhaft die Komplexität der folgenden Operationen auf Arrays:

- Zugriff (indexbasiert)
- Einfügen
	- Vorne
	- In der Mitte
	- Hinten
- Löschen
	- Vorne
	- In der Mitte
	- Hinten
- konkretes Element suchen (unsortiertes Array)
- konkretes Element suchen (sortiertes Array)


# Danke für Ihre Aufmerksamkeit! Fragen?
