---
title: "Prozedurale Programmierung -- Algorithmen"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: April 2022
---

## Organisatorisches

- Verlegung Onlinepraktikum: Mo 8:00 Uhr (Hybrid)
- Vorlesung bleibt online: Do 15:45 Uhr

# Rekursion

## Was ist Rekursion

- "Um Rekursion zu verstehen, muss man zuerst Rekursion verstehen"
- Rekursion ist mehrmaliges Aufrufen der gleichen Funktion
- Abbruchbedingung benötigt, sonst endlose Rekursion (Programmabsturz wegen Stapelüberlauf/Stackoverflow)

## Beispiel: Fakultät

![](Images/faculty.png)

## Beispiel: Fibonacci

![](Images/fib.png)

# Header (``<algorithm>``)

## Header (``<algorithm>``)

- Dieser Header enthält viele praktische Algorithmen, welche auf Sequenzen angewendet werden können.
- Die Sequenzen werden durch _Iteratoren_ (Anfang und Ende) festgelegt.
- Algorithmen mit ``_if`` im Namen akzeptieren Lambda-Funktionen für erhöhte Flexibilität.
- Beispiel: ``std::count`` (Zählen) und ``std::count_if`` (Bedingtes zählen)
- Siehe Beispiel 01 und Storyteller

## Lambda-Funktionen/-Ausdrücke

Lambda-Funktionen sind anonyme Funktionen. Sind besonders nützlich in Kombination mit ``<algorithm>``.

``` cpp
auto name = [](Parameter1, Parameter2, ...) {
    /* Funktionsinhalt */
    return Rueckgabewert
}
```

## Beispiel (Lambda-Funktion inline)

``` cpp
std::vector<int> zahlen = /* Werte hier */;
std::count_if(zahlen.begin(), zahlen.end(), [](int zahl) {
    bool erg = zahl > 0;
    return erg;
});
```

## Beispiel (Lambda-Funktion als Variable)

``` cpp
// Typ muss bei Lambda-Funktionen immer auto sein
auto ist_positiv = [](int zahl) {
    bool erg = zahl > 0;
    return erg;
}

std::vector<int> zahlen = /* Werte hier */;
std::count_if(zahlen.begin(), zahlen.end(), ist_positiv);
```

## Lambda-Funktion: Bindung

Die Werte in `[]` geben an, wie externe Variablen gebunden werden:

-   Leer: Keine externe Variable im lambda-Ausdruck verfügbar
-   `&`: Externe Variablen sind als Referenz verfügbar
-   `=`: Externe Variablen sind als Kopie verfügbar

Das beachten wir nicht weiter, da die Bindungen oft nur in komplexen Beispielen notwendig sind. Sie sollten aber wissen, dass die Möglichkeit existiert.

## Iteratoren

- Bei allen Algorithmen wird nicht ein Vektor übergeben, sondern ein Bereich
- In der Regel ``begin()`` und ``end()``
- Iteratoren zeigen auf Listenelemente
- Iteratoren können vorwärts und rückwärts bewegt werden
- Bis jetzt bekannt:
	- ``.begin()``: Iterator an den Anfang
	- ``.end()``: Iterator an das Ende
- Iteratoren sind Pointer

## Pointer

- Pointer sind Variablen, welche die Adresse einer anderen Speicherzelle enthalten
- Wenn ein Pointer den Wert ``100h`` hat, dann ist dies also ein Verweis auf die Daten, die bei ``100h`` im Speicher liegen
- Das Lesen von einer Speicherzelle, die über einen Pointer referenziert wird, heißt _Derefenzierung_
- In modernem C++ können Pointer weitestgehend ignoriert werden

## Datenstruktur: Array vs. Verkettete Liste

Neben Arrays, gibt es noch andere iterierbare Datenstrukturen, beispielsweise verkettete Listen.

Das Iterator-Interface ist eine Abstraktion, da der Entwickler nicht wissen muss, wie die darunterliegende Datenstruktur aussieht.

(Siehe Vorlesungsvideo)

## Suchen in Listen

Mit ``std::find`` und ``std::find_if`` kann in Listen gesucht werden.

Diese geben einen Iterator zurück, der auf das Element zeigt, bzw. einen Iterator auf das Ende, wenn nichts gefunden wurde.

Siehe Beispiele 03 & 04

## Filtern mit ``std::remove_if``

``remove`` und ``remove_if`` entfernen die Elemente nicht, sondern verschieben diese an das Ende der Liste. Alle anderen Elemente bleiben in Ihrer relativen Position zueinander.

Der Rückgabewert ist ein Iterator auf das erste zu löschen Element.

**Die Werte der zu löschenden Elemente sind nicht definiert**.

## Löschen mit ``std::erase``

Richtig gelöscht wird mit ``std::erase``, indem man von _Rückgabe von ``remove``_ bis _Listenende_ löscht.

Siehe Beispiel 05 und Storyteller

# Danke für Ihre Aufmerksamkeit! Fragen?
