---
title: "Prozedurale Programmierung -- Dateiein- & Ausgabe"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Mai 2022
---

# Sortieren von Daten (Fortsetzung Vorlesung "Algorithmen")

## Terminologie

Beim Sortieren geht es darum, Werte in einer konkreten Reihenfolge anzuordnen, z. B. aufsteigende oder absteigende Zahlen.

Ein Sortieralgorithmus ist _stabil_, wenn die relative Anordnung der Elemente zueinander bestehen bleibt.

## ``std::sort`` und ``std::stable_sort``

- Best Case (sortierte Liste): O(n)
- Average & Worst Case: O(n\*log(n))
- ``std::sort`` kann stabil sein, muss aber nicht
- ``std::stable_sort`` ist stabil und etwas langsamer als ``std::sort``
- Wenn unsicher, ``std::stable_sort`` verwenden
- Siehe Beispiel 01

## Bubble Sort

- Die Liste wird von links nach rechts durchlaufen und wenn der rechte Nachbar kleiner als der linke Nachbar ist, wird getauscht.
- Dies wird so lange wiederholt, bis die Liste sortiert ist.
- In allen Fällen: O(n²)
- Stabil
- Siehe Beispiel 02

## Bogo Sort

:::::::::::::: {.columns}
::: {.column}

- Ein Beispiel für ein extrem schlechtes Sortierverfahren
- Mische solange zufällig, bis die Liste sortiert ist
- Nicht stabil
- Siehe Beispiel 03

:::
::: {.column}

- Best case: O(1)
- Average case: O((n + 1)!), oft schneller
- Worst case: O(∞) bei schlechtem Zufall

:::
::::::::::::::

# Dateiein- & Ausgabe

## Motivation

Bisher waren alle Zahlen, Namen usw. in unseren Programmen direkt festgelegt (hardcoding) oder vom Nutzer abgefragt.

So funktioniert es in der _Realität_ normalerweise nicht: Die Daten werden bereitgestellt und von Ihrem Programm eingelesen und verarbeitet.

Bei großen Datensätzen ist es quasi ausgeschlossen, dass der Nutzer jedes Mal diese von Hand eingibt.

## Streams in C++

- ``iostream`` mit ``std::cout`` und ``std::cin`` sind bekannt
- Siehe als Wiederholung Beispiel 01

## Dateistreams in C++

Dasselbe Streaminterface wird für Dateistreams verwendet.

Der Header dazu heißt ``<fstream>``.

## ``fstream``

- ``fstream`` ist das Interface für Dateiein-/Ausgabe in C++.
- ``ifstream``: Eingabestream (**i**nput **f**ilestream)
- ``ofstream``: Ausgabestream (**o**utput **f**ilestream)

## Verwendung

```cpp
// Datei öffnen
std::ifstream stream("datei.txt");

std::string zeile;
// String aus stream in Variable zeile lesen
stream >> zeile;
```

## Open Modes

Bei den Streams kann optional noch als zweiter Parameter der Modus übergeben werden:

- ``in``: Stream zum Lesen im Textmodus öffnen (Standard für ``ifstream``)
- ``out``: Stream zum Schreiben im Textmodus öffnen (Standard für ``ofstream``), Inhalt wird gelöscht
- ``binary``: Stream arbeitet im Binärmodus. Verwenden, wenn Datei kein reiner Text ist.
- ``app``: Alle Schreiboperationen sind am Ende der Datei, Inhalt bleibt erhalten
- ``trunc``: Dateiinhalt wird beim Öffnen gelöscht

## Beispiele

- 02: Eine Zeile lesen
- 03: Zahlen lesen
- 04: Zeichenweise lesen

## Kommandozeilenargumente

Die ``main``-Funktion akzeptiert zwei Parameter: ``argc`` und ``argv``.

- ``argc``: Anzahl an Parametern
- ``argv``: Array von Pointern auf den Start von Zeichenketten (C-Version von ``std::vector<std::string>``)
- Der erste Parameter ist immer der Programmname
- Kommandozeilenparameter können verarbeitet werden
- Siehe Beispiel 05 und 06

## Schreiben von Text

Siehe 07

# Flatfiles

## Was sind Flatfiles

- Semistrukturierte Textdateien
- Maschinenlesbar
- Als Semistrukturierte Daten bezeichnet man in der Datenbankforschung Informationen, die keiner allgemeinen Struktur unterliegen, sondern einen Teil der Strukturinformation mit sich tragen. (via Wikipedia)

## CSV

- Kommagetrennte Daten
- Alternative Trenner: z.B. TSV (Tabulator-getrennt)
- In Deutschland ist oft das ";" der Trenner, weil das Komma für die Kommastelle verwendet wird (Probleme beim Datenaustausch)
- Kann strukturiert oder semistrukturiert sein (ein Schema wird aber nicht erzwungen)

## CSV: Beispiel

![Beispiel für einen Stundenplan (via Wikipedia: CSV_(Dateiformat), CC-BY-SA)](Images/CSV.png)

## CSV: Beispiel: /etc/passwd

Menschen- und Maschinenlesbar

```
root:x:0:0:root:/root:/bin/bash
[...]
mail:x:8:12:mail:/var/spool/mail:/usr/bin/nologin
ftp:x:14:11:ftp:/srv/ftp:/usr/bin/nologin
http:x:33:33:http:/srv/http:/usr/bin/nologin
[...]
gabriel:x:1000:1000::/home/gabriel:/usr/bin/fish
```

## XML

- 1998 vom World Wide Web Consortium (W3C) standardisiert
- Hauptelemente sind Tags: ``<eintrag></eintrag>``
- Leere Tags können direkt geschlossen werden: ``<eintrag/>``
- Tags können Attribute haben: ``<eintrag attrib="wert"/>``
- Kann mit einem XML-Schema validiert werden

## XML: Beispiel

```
<Kreditkarte>
  <Nummer>1234-5678-9012-3456</Nummer>
  <Waehrung>EURO</Waehrung>
  <Inhaber Name="Mustermann" Vorname="Max">
    <Geschlecht>m</Geschlecht>
    <Hobbys>
      <Hobby>Reiten</Hobby>
      <Hobby>Badminton</Hobby>
    </Hobbys>
  </Inhaber>
</Kreditkarte>
```

## JSON

- 2017 standardisiert (existiert aber schon viel länger)
- Hat XML in vielen Bereichen verdrängt, weil es viel kompakter ist (weniger "Line Noise")
- Besteht aus Wertepaaren:
	- ``"eintrag": "wert"``
	- ``"eintrag": null`` (leeres Element)
	- ``"liste": [1, 2, 3]`` (Liste mit Zahlen)
	- ``{}`` (die geschweifte Klammer beginnt ein neues Objekt)

## JSON: Beispiel

```json
{
  "Nummer": "1234-5678-9012-3456",
  "Waehrung": "EURO",
  "Inhaber":
  {
    "Name": "Mustermann",
    "Vorname": "Max",
    "Geschlecht": "m",
    "Hobbys": ["Reiten", "Badminton"]
  }
}
```

## Beispiel: Iris-Datensatz

:::::::::::::: {.columns}
::: {.column}

- Datensatz mit 150 Beobachtungen von 4 Attributen von Schwertlilien
- Enthält Breite und Länge des Kelchblatts (Sepalum) und des Kronblatts (Petalum) in Zentimeter
- Zudem ist Art der Schwertlilie angegeben (siehe Abbildung)
- Stark vereinfachtes Beispiel: Siehe 08

:::
::: {.column}

![](Images/iris.png)

:::
::::::::::::::

# Danke für Ihre Aufmerksamkeit! Fragen?
