---
title: "Prozedurale Programmierung -- Objektorientierte Programmierung (Theorie)"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Mai 2022
---

# Modellierung

## Literaturempfehlung

![](Images/Balzert.jpg)

## Möglichkeiten der Modellierung

Es existieren unzählige Möglichkeiten zur Prozessmodellierung, auszugsweise sind hier einige genannt:

- UML-Diagramme
- Petrinetze (*)
- BPMN (*)

(*): Wird nicht behandelt

## UML (Unified Modelling Language)

- Für die Modellierung von Softwaresystemen
- Höhere Abstraktion als Programmiersprache: Auf verschiedene Programmiersprachen anwendbar

## UML-Sequenzdiagramm

:::::::::::::: {.columns}
::: {.column}

Beschreibt den zeitlichen Austausch von Nachrichten zwischen Objekten (Weiter unten = später)

:::
::: {.column}

![Beispiel für ein Sequenzdiagramm (Stkl, CC-BY-SA)](Images/UML_Sequenz.png)

:::
::::::::::::::

## UML Use Case / Anwendungsfall-Diagramm

:::::::::::::: {.columns}
::: {.column}

Stellt Anwendungsfälle und Akteure in ihrer jeweiligen Beziehung zueinander dar.

:::
::: {.column}

![Beispiel für ein Use Case Diagramm (Stkl, CC-BY-SA)](Images/UML_UseCase.png)

:::
::::::::::::::

## UML-Aktivitätsdiagramm

Entspricht einem Flowchart bzw. Ablaufplan

## Flowchart (Flussdiagramm) bzw. Programmablaufplan

![Symbole in einem Flowchart (aus: Wikipedia, Programmablaufplan, CC-BY-SA)](Images/Flowchart.png)

## Flowchart: Beispiel Flugreise

![(Balzert, Lehrbuch der Softwaretechnik, S.248)](Images/Balzert_S248.png)

## Flowchart: Komplexbeispiel Flugreise

![(Balzert, Lehrbuch der Softwaretechnik BPMN, S.249)](Images/Balzert_S249.png)

## Flowchart: Bubble Sort

![Bubble Sort als Flowchart (Eigene Darstellung, CC0)](Images/Bubble.png)

## UML-Klassendiagramm (!)

:::::::::::::: {.columns}
::: {.column}

- Verwendet in der objektorientierten Programmierung
- Zur grafischen Darstellung (Modellierung) von Klassen, Schnittstellen sowie deren Beziehungen

:::
::: {.column}

![Beispiel für ein Klassendiagramm (Stkl, CC-BY-SA)](Images/UML_Klassendiagramm.png)

:::
::::::::::::::

# Objektorientierte Programmierung

## Klassen & Objekte in C++

- Kapseln von "Dingen, die zusammengehören"
- Gemeinsame Eigenschaften
- Gemeinsame Funktionen
- usw.
- siehe z. B. ``std::string`` und ``std::vector``

## Begrifflichkeiten

``std::string`` ist der Name der Klasse

```cpp
std::string str;  // Instanz (Objekt) der Klasse
std::string str2; // Noch eine Instanz der Klasse
```

## Eigenschaften

- Jedes Objekt gehört einer Klasse an
- Die Klasse definiert das Verhalten ihrer Objekte
- Bietet Funktionen (Methoden) an, um das Objekt zu modifizieren oder abzufragen

## Vorteile

- Objekte stören sich nicht gegenseitig
    - z. B. zwei verschiedene Instanzen von ``std::vector``
- Implementierungsdetails können versteckt werden:
    - Speicherverwaltung von ``std::vector`` bei Aufruf von ``pop_back`` oder ``push_back``
    - Speicherlayout einer Datenstruktur
- Wiederverwendbarkeit in anderen Softwareprojekten
- Hilft Code-Duplizierung bzw. Redundanz vorzubeugen (dies ist ein Anti-Pattern)
- Komplexe Funktionalität möglich

## Begrifflichkeiten (2)

```cpp
int funktion(int a) {
    if (a > 0) {
        // Aufruf Konstruktur
        std::string str;

        // ... Mache etwas mit str
    }
    // Scope verlassen, str wird zerstört
    // Aufruf Destruktor

    return 0;
}
```

## Primitive/Einfache Datentypen in C++

- Primitive Datentypen sind _keine_ Klassen oder Objekte
- Dazu gehören:
  - Alle Zahlentypen (``int``, ``double`` usw.)
  - Zeichen (``char``)
  - Funktionen
  - Pointer und Referenzen

## Klassenhierarchie

- Mehrere Klassen können gemeinsame Eigenschaften haben. Dies nennt sich _Generalisierung_
- Klassen können von anderen Klassen abhängen, dies nennt sich _Aggregation_
- Dadurch entstehen _Klassenhierarchien_

## Klassenhierarchie am Beispiel Northwind

![](Images/NW.png)

## Zugriffsrechte

- Um die Kapselung zu verbessern, können für Felder und Methoden Zugriffsrechte festgelegt werden
- ``public``: Von jedem zugreifbar
- ``protected``: Von erbenden Klassen (Kind-Klassen) zugreifbar
- ``private``: Nur von der eigenen Klasse zugreifbar

## Beispielklasse ``Point``

:::::: {.columns}
::: {.column}

Ein Punkt im zweidimensionalen wird durch dessen X- und Y-Koordinate beschrieben.

:::
::: {.column}

![Das UML-Klassendiagramm der Klasse ``Point`` (eigene Darstellung, CC0)](Images/Point.png)

:::
::::::

## Beispielklasse ``Point``: Konstruktoren

- Mit ``Point()`` (gleicher Name wie Klassenname) wird ein Punkt im Koordinatenursprung erzeugt. Dies ist ein _Konstruktor_.
- Ein _Konstruktor_ ohne Parameterliste ist der _Standardkonstruktor_ (Default constructor).
- Mit ``Point(double x, double y)`` wird ein Punkt an den Koordinaten ``x`` und ``y`` erzeugt.

## Beispielklasse ``Point``: Objekterzeugung

Vom Punkt können nun mehrere individuelle Objekte erzeugt werden:

```cpp
// Standardkonstruktur (im Ursprung)
// Ohne Klammern (!)
Point u;

// Spezifische Koordinaten
Point p(0, 0);
Point q(1, 2);
Point r(0, -1);
```

## Klasse ``Point`` mit _Getter_ und _Setter_

:::::: {.columns}
::: {.column}

Ein Punkt im zweidimensionalen wird durch dessen X- und Y-Koordinate beschrieben.

Mit sogenannten _Gettern_ und _Settern_ können die Felder ``x`` und ``y`` gekapselt werden.

:::
::: {.column}

![Das UML-Klassendiagramm der Klasse ``Point`` (eigene Darstellung, CC0)](Images/Point2.png)

:::
::::::

## Klasse ``Point`` mit _Getter_ und _Setter_ (2)

:::::: {.columns}
::: {.column}

### Vorteile

- Die Implementierung der Klasse kann geändert werden, solange die Getter/Setter erhalten bleiben (Flexibilität)
- Fehlerprüfung/-Behandlung vor Zuweisung möglich

:::
::: {.column}

### Nachteile

- Zusätzlicher Quellcode durch Getter/Setter
- _Umständliches_ Interface
	- Vorher: ``p.x += 10``;
	- Nachher: ``p.setX(p.getX() + 10)``

:::
::::::

## Beispielklasse ``Point``: Weitere Methoden

Weitere Methoden wären denkbar, z.B. ``move_by``, ``move_to``, ``distance_to`` usw.

## Klassenhierarchie aus geometrischen Objekten

![](Images/Geom.png)

## Klassenhierarchie aus geometrischen Objekten (2)

![](Images/Geom2.png)

## Abstrakte Klassen

Bei abstrakten Klassen wird die Implementierung einiger Methoden ausgelassen.

Derartige Klassen können nicht instantiiert werden und die Kind-Klasse _muss_ die Methode implementieren.

## Zusammenfassung: Stichworte (Einprägen!)

- Instanzen von Klassen sind Objekte
- Konstruktoren erzeugen das Objekt
- Destruktoren zerstören das Objekt
- Objekte werden instantiiert
- Methoden sind Funktionen des Objekts
- Felder sind Variablen des Objekts

## Schreibweise von Klassen und Methoden

- Camel Case:
	- getPosition
	- moveObjectTo
- Upper Camel Case / Pascal Case:
	- GetPosition
	- MoveObjectTo
- Snake Case:
	- get_position
	- move_object_to

Geschmackssache, aber Konsistenz wichtig!

## Wort zum Schluss

**Modellierung und Programmierung sind kreative Prozesse**

Mehrere Wege führen zum Ziel

# Danke für Ihre Aufmerksamkeit! Fragen?
