---
title: "Prozedurale Programmierung -- Objektorientierte Programmierung (Praxis)"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Mai 2022
---

## Motivation

- Kapseln von "Dingen, die zusammengehören"
- Gemeinsame Eigenschaften
- Gemeinsame Funktionen
- usw.
- siehe z. B. ``std::string`` und ``std::vector``
- **Heute**: Entwerfen eigener Klassen

## Klasse ``Point``

:::::: {.columns}
::: {.column}

Ein Punkt im zweidimensionalen wird durch dessen X- und Y-Koordinate beschrieben.

:::
::: {.column}

![Das UML-Klassendiagramm der Klasse ``Point`` (eigene Darstellung, CC0)](Images/Point.png)

:::
::::::

## Klasse ``Circle``

:::::: {.columns}
::: {.column}

Ein Kreis wird durch seinen Mittelpunkt und dessen Radius beschrieben.

:::
::: {.column}

![Das UML-Klassendiagramm der Klasse ``Circle`` (eigene Darstellung, CC0)](Images/Circle.png)

:::
::::::

## UML-Klassendiagramm

![Klassendiagramm (eigene Darstellung, CC0)](Images/UML.png)

## Umsetzung

- Siehe Storyteller ``01`` und ``03``
- Siehe Quellcode von ``01_oop``

# Fehlerbehandlung

## Definition

Fehler sind unerwartete Probleme bei der Programmausführung.

Es gibt zwei Fehlerklassen:

- Programmfehler
- Benutzerfehler

## Programmfehler

Programmfehler resultieren oft aus Fehlern in der Programmierung und sollten nicht behandelt werden. Stattdessen sollte das Programm kontrolliert abstürzen und versuchen die aktuellen Nutzerdaten abzuspeichern.

Um _unmögliche_ Fälle im Programm zu erkennen, empfiehlt sich die Funktion ``assert``.

``assert`` haben wir nicht behandelt: Die Funktion lässt das Programm abstürzen, wenn die Bedingung ``false`` ist.

Die meisten Klassen von Hardwarefehlern sollten ebenfalls nicht behandelt werden und sind Sache des Betriebssystems.

## Benutzerfehler

Benutzerfehler sind z. B.:

- Falsche Eingabe in ein Eingabefeld
- Laden eines nicht unterstützten Dateiformates
- Speichern ist am gewählten Speicherort nicht möglich (Datenträger voll, keine Schreibrechte, …)

Benutzerfehler dürfen nicht zum Programmabsturz führen. Stattdessen sollte der Fehler behandelt werden und dem Benutzer eine Fehlermeldung angezeigt werden.

## Fehlermeldungen

Fehlermeldungen sollten aussagekräftig sein, damit der Nutzer den Fehler beheben kann oder zumindest den Fehler online recherchieren kann oder einen Techniker informieren kann.

## Fehlermeldungen: Positivbeispiel (1)

![via <https://docs.microsoft.com/en-us/windows/win32/uxguide/mess-error>. Die Microsoft Designrichtlinien sind lesenswert.](Images/fehler_rename.png)

## Fehlermeldungen: Positivbeispiel (2)

![via <https://www.computerbase.de/forum/threads/a.1725888/>](Images/fehler_pcie.jpg)

## Fehlermeldungen: Negativbeispiel (1)

![via <https://www.peteroberhuber.de/filezilla-execution-of-command-failed/>](Images/fehler_erfolg.jpg)

## Fehlermeldungen: Negativbeispiel (2)

![via <https://www.zwentner.com/keyboard-not-found-press-f1/>](Images/fehler_keyboard.jpg)

## Fehlermeldungen: Negativbeispiel (3)

![via <https://twitter.com/rkkski/status/288956938778189824>](Images/fehler_phone.jpg)

## Fehlermeldungen: Negativbeispiel (4)

![Nervige OPAL-Fehlermeldungen. Eigene Darstellung.](Images/fehler_opal.png)

## Typische Arten der Fehlerbehandlung

- Signalwert
- Exceptions (Ausnahmen)

## Signalwert

Ein Fehler könnte mit einem speziell kodierten Wert signalisiert werden.

z. B. Bei Erfolg wird eine positive Zahl zurückgeben, bei Fehlerfall eine -1.

## Signalwert: Probleme

- Konstruktoren:
	- Können keine Werte zurückgeben.
	- Signalwerte können also bei Konstruktion nicht verwendet werden
- Fehlerprüfung darf nicht vergessen werden
- Bei manchen Funktionen sind alle Rückgaben gültige Werte, Fehler also nicht erkennbar

## Signalwert: Probleme

Beispiel ``int atoi(const char*)``.

- Wandelt einen C-String (``char*``) in eine Zahl um
- Gibt bei Fehler (keine Zahl) ``0`` zurück

## Signalwert: Probleme (2)

Beispiel ``int atoi(const char*)``.

- Wandelt einen C-String (``char*``) in eine Zahl um
- Gibt bei Fehler (keine Zahl) ``0`` zurück
- _Aber ``0`` ist auch eine gültige Zahl!_
- Fehlererkennung nur umständlich möglich
- Bei Überlauf ist das Ergebnis undefiniert

## Signalwert: Probleme (3)

Moderne Lösung: ``std::stoi(std::string)``

Wirft bei Fehler _Exceptions_ (Ausnahmen)

## Exceptions (Ausnahmen)

- Werden von der Anwendung geworfen (``throw``)
- Können gefangen (``catch``) und weiter geworfen (``throw``) werden
- Wenn man eine Ausnahme ignoriert, wird das Programm beendet
- Die einzige sinnvolle Möglichkeit der Fehlerbehandlung in Konstruktoren

## Exceptions: Probleme

- Welche Exceptions auftreten können muss dokumentiert werden, eine Annotation im Code (checked Exception) ist nicht möglich
- Auf manchen Systemen sind Exceptions nicht verwendbar
  - Im aktuellen Projekt nicht gestattet (dies kann verschiedenste Gründe haben)
  - z. B. Mikrocontroller und andere eingebettete Systeme, da sie das Programm größer machen und recht langsam sind

## Beispiel: ``Circle``

:::::: {.columns}
::: {.column}

Die Klasse wird nun so abgeändert, dass der Radius größer als ``0`` sein muss.

:::
::: {.column}

![Das UML-Klassendiagramm der Klasse ``Circle`` (eigene Darstellung, CC0)](Images/Circle2.png)

:::
::::::

## UML-Klassendiagramm

![Klassendiagramm nach der Änderung (eigene Darstellung, CC0)](Images/UML2.png)

## Umsetzung

- Siehe Storyteller ``05``
- Siehe Quellcode von ``01_oop`` und ``02_error``

# Vererbung

## Klassenhierarchie

- Klassen können Eigenschaften _erben_
- Dies nennt sich _Klassenhierarchie_
- Kind-Klasse erbt Eigenschaften und Methoden von der Eltern-Klasse und kann diese erweitern oder überschreiben

## Beispiel: Tierlaute

:::::: {.columns}
::: {.column}

Die Klasse ``Animal`` enthält die abstrakte Methode ``Sound()`` und muss von den Kind-Klassen implementiert werden.

Zusätzlich kann mit ``get_name()`` die Art abgefragt werden. Die Kind-Klassen überschreiben dafür das Feld ``name``.

:::
::: {.column}

![Das UML-Klassendiagramm für Tierlaute (eigene Darstellung, CC0)](Images/Animal.png)

:::
::::::

## Umsetzung

- Siehe Storyteller ``06``
- Siehe Quellcode von ``03_animal``

# Danke für Ihre Aufmerksamkeit! Fragen?
