#include <iostream>
#include <memory>
#include <string>
#include <vector>

class Texture {
public:
    Texture(std::string name) {
        std::cout << "Texture erstellt: " << name << "\n";
    }
};

class PoolEntry {
public:
    std::string name;
    std::weak_ptr<Texture> texture;
};

std::vector<PoolEntry> pool;

std::shared_ptr<Texture> fetch_texture(std::string name) {
    for (auto& entry: pool) {
        if (entry.name == name) {
            auto shared_texture = entry.texture.lock();
            if (!shared_texture) {
                shared_texture = std::make_shared<Texture>(name);
                entry.texture = shared_texture;
                std::cout << "Bekannt und nicht mehr aktiv: " << name << "\n";
                return shared_texture;
            } else {
                std::cout << "Bekannt und aktiv: " << name << "\n";
                return shared_texture;
            }
        }
    }

    std::cout << "Neu: " << name << "\n";
    auto shared_texture = std::make_shared<Texture>(name);
    pool.push_back({name, shared_texture});
    return shared_texture;
}

int main() {
    {
        auto a = fetch_texture("A"); // Erstellt A
        auto a2 = fetch_texture("A"); // Nimmt erstelltes A
        // "A" wird, da kein shared_ptr auf A
    }

    std::shared_ptr<Texture> a4;

    {
        auto a3 = fetch_texture("A"); // Erstellt neues A
        a4 = a3; // Kopie des shared_ptr, hat nun zwei Referenzen
        // "A" wird nicht zerstört, da noch a4 ein shared_ptr auf "A" ist
    }

    {
        auto a5 = fetch_texture("A"); // Nimmt erstelltes A
        a4.reset(); // Eine Referenz weniger, "A" wird aber nicht zerstört

        // "A" wird zerstört, da a5 letzter shared_ptr war
    }

    auto a6 = fetch_texture("A"); // Erstellt A neu
    auto b1 = fetch_texture("B"); // Erstellt B
    auto b2 = fetch_texture("B"); // Gibt bereits erstelltes "B" zurück

    b1.reset(); // Eine Referenz weniger
    a6 = b1; // B hat nun zwei Referenzen, A hat 0, wird daher zerstört

    return 0;
}
