---
title: "Prozedurale Programmierung -- Einführung 2D-Grafik und SDL2"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Juni 2022

---

## Literaturempfehlung

- [Vornberger, Oliver (2006): "Computergrafik". Institut für Informatik, Universität Osnabrück](http://www-lehre.inf.uos.de/~cg/2006/PDF/skript.pdf)
- [Haenselmann, Thomas (2022): "Bildverarbeitung Sommer 2022". Hochschule Mittweida](https://www.cb.hs-mittweida.de/webs/haenselm/bildverarbeitung-sommer-2022/)

# 2D-Computergrafik

## Rastergrafik

:::::: {.columns}
::: {.column}

Bei Rastergrafiken bestehen die Grafiken aus **Pixeln**

Jeder Pixel ist ein **Bildpunkt**

Die **Farbtiefe** ist die Anzahl der Bits pro Pixel.

- Schwarz-Weiß: 1 Bit
- Graustufen: 8 Bit
- RGB (Rot, Grün, Blau): 24 Bit

:::
::: {.column}

:::
::::::

## Farbtiefe

![(Übernommen von Prof. Ittner, Hochschule Mittweida, Einführung in die Informatik)](Images/farbtiefe.png)

## Rastergrafik

:::::: {.columns}
::: {.column}

Bei Rastergrafiken bestehen die Grafiken aus **Pixeln**

Jeder Pixel ist ein **Bildpunkt**

Die **Farbtiefe** ist die Anzahl der Bits pro Pixel.

- Schwarz-Weiß: 1 Bit
- Graustufen: 8 Bit
- RGB (Rot, Grün, Blau): 24 Bit

:::
::: {.column}

![Foto von Nala (Eigene Darstellung)](Images/Nala.jpg)

:::
::::::

## Rastergrafik (2)

:::::: {.columns}
::: {.column}

Wenn das Bild vergrößert wird, werden die Pixel sichtbar.

:::
::: {.column}

![Zoom auf das Auge von Nala (Eigene Darstellung)](Images/Nala_gross.png)

:::
::::::

## Rastergrafik (3)

:::::: {.columns}
::: {.column}

![Foto von Nala (Eigene Darstellung)](Images/Nala.jpg)

:::
::: {.column}

![Zoom auf das Auge von Nala (Eigene Darstellung)](Images/Nala_gross.png)

:::
::::::

## Vektorgrafik (verwenden wir nicht)

:::::: {.columns}
::: {.column}

- Beschreibung durch Vektoren
- Grafische Darstellung durch einfache, mathematische Formen (Linie, Rechteck, Kreis, Polygon, usw.)
- Vorteile:
	- Kann beliebig groß sein, ohne Qualitätsverlust
	- Platzsparend
- Nachteile:
	- Nicht für detailreiche Grafiken oder Fotos geeignet
	- Langsameres rendern als bei Rastergrafik

:::
::: {.column}

![Vektorgrafik eines Netzwerks (Eigene Darstellung)](Images/Netzwerk.png)

:::
::::::

## Vektorgrafik (2) (verwenden wir nicht)

:::::: {.columns}
::: {.column}

Wenn das Bild vergrößert wird, sieht man die geringen Details einer üblichen Vektorgrafik, aber es wird nicht unscharf.

:::
::: {.column}

![Vergrößerte Vektorgrafik eines Netzwerks (Eigene Darstellung)](Images/Netzwerk_gross.png)

:::
::::::

## Vektorgrafik (3) (verwenden wir nicht)

:::::: {.columns}
::: {.column}

![Vektorgrafik eines Netzwerks (Eigene Darstellung)](Images/Netzwerk.png)

:::
::: {.column}

![Vergrößerte Vektorgrafik eines Netzwerks (Eigene Darstellung)](Images/Netzwerk_gross.png)

:::
::::::

# Einführung in SDL2

## Was ist SDL2

- SDL = Simple Direct Media Layer
- "2", weil es die zweite Version der Bibliothek ist
- Hat viele technische Probleme von SDL1 gelöst
- Cross-platform (Windows, Linux, macOS, Android, iOS und mehr)
- Ermöglicht Zugriff auf Grafikhardware, Audio, Tastatur und Maus
- Wird in vielen Produkten verwendet, besonders in Spielen

## Anmerkung für E-Technik

:::::: {.columns}
::: {.column}

- Auf dem Arduino gibt es kein SDL2
- Der Arduino für sich hat keinerlei Grafikhardware, Audio etc.
- Wenn es solche gibt, gibt es diese in unzähligen Varianten und kann an beliebige Pins angeschlossen werden
- Nicht für die sehr langsame Hardware optimiert
- Macht generell einfach wenig Sinn
- Für die Display-Steuerung verwenden wir stattdessen Adafruit

:::
::: {.column}

![Wir steuern stattdessen dieses Display an (SSD1306)](Images/SSD1306.jpg)

:::
::::::

## Programmiersprache von SDL2

- SDL2 ist in C geschrieben
- Keine Speicherverwaltung mit ``new``/``delete``, aber mit ``std::unique_ptr`` lösbar
- Keine Objekte im C++-Sinne
- Alle SDL-Funktionen sind global und akzeptieren die Strukturen als ersten Parameter
- Benötigt daher eine kleine Eingewöhnungsphase

## Zu SDL2 gehörende Bibliotheken

- Eine Bibliothek ist eine Ansammlung von Funktionen
- SDL2 ist eine Bibliothek und es gibt Erweiterungsbibliotheken, um die Funktionen von SDL2 zu erweitern
	- SDL2_image: Unterstützung von mehr Bildformaten, SDL2 kann nur Bitmap (unkomprimierte Bilder)
	- SDL2_mixer: Verbesserte Audiowiedergabe, SDL2 kann nur WAV Audio (unkomprimierten Ton)
	- SDL2_ttf: Textausgabe

## Aufbau SDL2-Programm

1. Fenster erstellen (``SDL_Window``)
2. Grafikausgabe erstellen (``SDL_Renderer``)
3. Endlosschleife (bis Benutzer Programm beendet):
	- Benutzereingaben ermitteln
	- Logik aktualisieren
	- Zeichnen

## Aufbau Arduino-Programm mit OLED-Display

1. Display initialisieren (``display.begin()``)
2. Endlosschleife (bis Benutzer Stecker zieht):
   - Taster / Sensoren lesen
   - Logik aktualisieren
   - Zeichnen

## Fazit

Die Funktionen bei SDL2 vs. Adafruit heißen anders, sind aber prinzipiell gleichwertig (sofern die Funktionalität beim Arduino sinnvoll ist)

# Zeichnen einfacher, geometrischer Objekte

## Koordinatensysteme

![Kartesisches Koordinatensystem. Zu beachten ist bei der 2D-Grafik die y-Achse: Diese verläuft nach unten.](Images/Grafik.png)

## Zeichnen von Rechtecken

:::::: {.columns}
::: {.column}

- Das Rechteck wird zeilenweise dargestellt.
- Dafür benötigen wir eine verschachtelte ``for``-Schleife.
- Die äußere Schleife erhöht ``y`` (also die Zeile)
- Für jede Zeile wird ``x`` durchlaufen (also die Spalte)

:::
::: {.column}

```cpp
for (int y = 0; y < 20; ++y) {
    for (int x = 0; x < 10; ++x) {
        setze_pixel(x, y);
    }
}
```

:::
::::::

## Zeichnen von Linien

- Problem:
  - Schräge Linien lassen sich am Computer nicht exakt darstellen
  - Müssen in das Pixelraster passen
  - Geradengleichung ist zu ineffizient
- Lösung:
  - Bresenham-Algorithmus
  - 1965 von Jack Bresenham
  - Sehr effizientes Verfahren (benötigt in der optimierten Variante nur Ganzzahlen und Addition)

## Zeichnen von Linien (2)

- Einschränkungen (damit Code kürzer): ``x2 > x1``, ``y2 > y1`` und ``m <= 1``
- Gezeichnet wird von ``x1`` nach ``x2``, beginnend bei ``(x1, y1)``
- Ein _Fehlerwert_ ermittelt, wie weit ``y`` von der realen Linie abweicht
- Der Fehler wird für jeden Pixel um den Anstieg (``dy / dx``) erhöht
- Wenn der Fehler ``> 0.5`` ist, wird ``y`` um ``1`` erhöht und der Fehler um ``1`` verringert
- Dies wird solange wiederholt, bis ``x`` den Wert von ``x2`` erreicht

## Zeichnen von Linien (3)

- Mit den Einschränkungen können wir nur in einem Bereich von 45° Linien zeichnen
- Um alle anderen sieben Bereiche abzudecken, muss die Laufrichtung von ``x`` und die Fehlerberechnung entsprechend angepasst werden

## Zeichnen von Kreisen

- Problem:
  - Wie bei Linien
  - Trigonometrische Funktionen (``sin`` und ``cos`` sind extrem langsam)
  - Werte von ``sin`` und ``cos`` können vorberechnet werden (Lookup-Table), aber dennoch ineffizient
- Lösung:
  - Bresenham-Algorithmus

## Zeichnen von Kreisen (2)

- Betrachten wieder nur 45°, der Rest ergibt sich aus der Symmetrie des Kreises
- Für jeden Punkt ``P(x, y)`` sei ``f(x, y) = x² + y² - r²``
- Weiterhin gilt:
  - ``f(x, y) = 0`` wenn ``(x, y)`` auf dem Kreis liegt
  - ``f(x, y) < 0`` wenn ``(x, y)`` innerhalb des Kreises liegt
  - ``f(x, y) > 0`` sonst (außerhalb)
- ``f`` entscheidet, ob ``y`` verringert werden muss
- ``d = f(x + 1, y - 1/2)`` (Mitte von ``y`` und ``y - 1``)

## Zeichnen von Kreisen (3)

![Vornberger, Oliver (2006): "Computergrafik". Institut für Informatik, Universität Osnabrück, S. 39](Images/Kreis.png)

## Zeichnen von Kreisen (4)

- Startwert für ``d``: ``f(1, r - 1/2) = 5/4 - r``
- Wenn ``d < 0``: ``f(x + 2, y - 1/2) = d + 2x + 3``
- Sonst: ``f(x + 2, y - 3/2) = d + 2x - 2y + 5``

## Zeichnen mit SDL2

SDL2 kann einige primitive direkt über die Grafikkarte zeichnen (schnell)

- ``SDL_RenderDrawLine`` für Linien
- ``SDL_RenderDrawRect`` für Rechtecke (nicht ausgefüllt)
- ``SDL_RenderFillRect`` für Rechtecke (ausgefüllt)
- Kreise lassen sich nicht direkt zeichnen

## Terminplanung

Ab nächster Woche:

Aufteilung in zwei Vorlesungen, wegen SDL2-Programmierung vs. Arduino:

- E-Technik: 14:15 - 15:45
- Alle anderen: 15:45 - 17:15 (wie bisher)

# Danke für Ihre Aufmerksamkeit! Fragen?
