#include "sdl_helper.h"

#include <chrono>
#include <thread>

using namespace std::chrono_literals;

void bresenham_line(SDL_Renderer* renderer, int x1, int y1, int x2, int y2) {
    int dx = x2 - x1;
    int dy = y2 - y1;

    double fehler = 0.0;
    double anstieg = static_cast<double>(dy) / static_cast<double>(dx);

    int y = y1;
    for (int x = x1; x <= x2; ++x) {
        SDL_RenderDrawPoint(renderer, x, y);
        SDL_RenderPresent(renderer);

        fehler += anstieg;

        if (fehler > 0.5) {
            ++y;
            fehler -= 1.0;
        }

        std::this_thread::sleep_for(20ms);
    }
}

int main(int, char**) {
    SDL_Helper sdl_helper;

    SDL_Renderer* renderer = sdl_helper.get_renderer();

    // Zeichenfarbe auf weiß setzen
    SDL_SetRenderDrawColor(renderer, 0xFF, 0xFF, 0xFF, 0xFF);

    // Gesamtes Fenster weiß einfärben
    SDL_RenderClear(renderer);

    // Änderungen sichtbar machen
    SDL_RenderPresent(renderer);

    // Zeichenfarbe auf rot setzen
    SDL_SetRenderDrawColor(renderer, 0xFF, 0, 0, 0xFF);

    bresenham_line(renderer, 10, 10, 300, 200);

    // Zeichenfarbe auf blau setzen
    SDL_SetRenderDrawColor(renderer, 0, 0, 0xFF, 0xFF);

    bresenham_line(renderer, 10, 30, 200, 100);


    for (;;) {
        SDL_Event evt;
        while (SDL_PollEvent(&evt)) {
            if (evt.type == SDL_QUIT) {
                // Benutzer hat Programmende angefordert (Fenster geschlossen)
                return 0;
            }
        }

        SDL_RenderPresent(renderer);
    }

    return 0;
}
