#include "sdl_helper.h"

#include <string>
#include <iostream>
#include <memory>
#include <stdexcept>

#include <SDL.h>

SDL_Helper::SDL_Helper() {
    /* SDL initialisieren */
    const int width = 320;
    const int height = 240;

    // Alle Komponenten von SDL initialisieren
    // siehe https://wiki.libsdl.org/SDL_Init
    if (SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        throw std::runtime_error(std::string("SDL_Init fehlgeschlagen ") + SDL_GetError());
    }

    // Fenster erstellen, siehe https://wiki.libsdl.org/SDL_CreateWindow
    // Parameter:
    // - Text in Titelleiste
    // - X & Y Position (Zahl in Pixeln oder Konstante)
    // - Fensterbreite & -hoehe in Pixeln
    // - Flags für den Fenstermanager
    window.reset(SDL_CreateWindow(
        "SDL Demo",
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        width, height,
        SDL_WINDOW_RESIZABLE));

    if (!window) {
        throw std::runtime_error(std::string("SDL_CreateWindow fehlgeschlagen ") + SDL_GetError());
    }

    // Renderer erstellen, siehe https://wiki.libsdl.org/SDL_CreateRenderer
    // Ein Renderer wird zum Zeichnen in ein Fenster verwendet
    // Parameter:
    // - SDL_Window handle von oben
    // - Zu verwendenter Treiber (-1 = Den ersten funktionierenden verwenden)
    // - Flags: SDL_RENDERER_SOFTWARE, da es sonst bei miesen Treibern flimmert
    renderer.reset(SDL_CreateRenderer(window.get(), -1, SDL_RENDERER_SOFTWARE));

    if (!renderer) {
        throw std::runtime_error(std::string("SDL_CreateRenderer fehlgeschlagen ") + SDL_GetError());
    }

    // Schwarzes Fenster zeichnen
	SDL_RenderClear(renderer.get());
	SDL_RenderPresent(renderer.get());

    // Automatisch skalieren, wenn sich Fenstergroesse aendert
	SDL_RenderSetLogicalSize(renderer.get(), width, height);
}

SDL_Helper::~SDL_Helper() {
    // Beim beenden SDL_Quit aufrufen
    SDL_Quit();
}

SDL_Renderer* SDL_Helper::get_renderer() {
    return renderer.get();
}
