---
title: "Prozedurale Programmierung -- Spiele auf dem Arduino"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Juni 2022
---

## Definition: Spiele-Engine (laut Wikipedia)

- Laufzeitumgebung für Computerspiele
- Steuert den Spielablauf und die visuelle Darstellung
- Besteht aus mehreren Komponenten
- Bekannte Engines:
    - Unreal Engine
    - Unity Engine
    - Godot (Open Source)
    - Blender (3D-Grafikprogramm mit integrierter Engine, Open Source)

## Spiele auf dem Arduino

- Game-Engines können natürlich nicht verwendet werden
- CPU: 16 Mhz (Computer: 2 Ghz)
- Arbeitsspeicher: 8 KB (Computer: 8 GB)
- I/O: 54 Digitalpins und 16 Analogpins (Computer: USB u. a.)
- Code muss speziell für den Arduino geschrieben sein

## Bildschirm

:::::: {.columns}
::: {.column}

- SSD1306
- 1-farbiges (an und aus) OLED-Display mit 128x64 Pixeln
- Jeder Pixel direkt ansteuerbar → Spiele umsetzbar
- Besser als LCD, da diese nur Text ausgeben können (langweilig)
- Für die Display-Steuerung verwenden wir die Bibliothek Adafruit

:::
::: {.column}

![Wir steuern stattdessen dieses Display an (SSD1306)](Images/SSD1306.jpg)

:::
::::::

## Wie erstelle ich ein Computerspiel

Ein Computerspiel ist quasi "nur" eine Endlosschleife und Prozessierung von Benutzereingaben und die Reaktion auf diese.

Empfohlenes Video: [Become A Game Developer In 5 Minutes Or Less - Mathieu Ropert - CppCon 2020](https://www.youtube.com/watch?v=H4N8xYue_3Q)

## Der Game-Loop

1. Nutzerinteraktionen verarbeiten
2. Logik / Physik aktualisieren
3. Rendern
4. Goto 1, bis Programm geschlossen wird

## Logik / Physik aktualisieren

Logikberechnungen finden in der Regel mit einer konstanten Updaterate statt, z. B. 60 Updates je Sekunde (UPS).

Die konstante Updaterate stellt sicher, dass das Spiel unabhängig von der Prozessorgeschwindigkeit läuft.

UPS ist keine offizielle Abkürzung.

## Logik / Physik aktualisieren (2)

Hier werden alle Objekte der Spielszene aktualisiert

- Spielfigur bewegen
- Objekte anhand ihrer Beschleunigung weiterbewegen
- Partikel aktualisieren
- Siegbedingungen prüfen
- …

## Rendern

Beim Rendern wird die Geometrie, Sprites usw. berechnet.

Das Rendern ist normalerweise an die Bildwiederholrate des Monitors gekoppelt (Vertikale Synchronisation / V-Sync).

Das OLED wird von Adafruit mit ca. 25 Hz angesteuert.

## Definition: Sprite

- Sprites sind Grafikobjekte, die auf dem Bildschirm dargestellt werden
- Operationen auf Sprites ("Affine Transformation"):
    - Bewegung
    - Rotation
    - Zoom
    - Kippen, Strecken, Stauchen usw.

## Definition: Lag / Frameskip

- Tritt auf, wenn das System nicht schnell genug den Frame darstellen kann (16 ms bei 60 FPS).
- Update und Rendering laufen unabhängig voneinander
- Frame wird ausgelassen (Frameskip) und nicht gerendert, Updates laufen mit 60 UPS weiter
- Wenn keine 60 UPS möglich: Spiel wird langsamer
- Frameskip macht auf dem Arduino natürlich wenig Sinn, stattdessen muss das Spiel so designt sein, dass es ruckelfrei läuft

## Timing auf dem Arduino

```cpp
// Globale Variable (16666 Mikrosekunden pro Frame)
long time_per_frame = 1000000 / 60;

void loop() {
    // Startzeit vom loop-Aufruf ermitteln
    long time_begin = micros();

    // Event Handling/Logik/Rendering/etc...

    // Ermitteln, wie lange die Schleife ausgeführt wurde
    long time_end = micros();
    long time_delta = time_end - time_begin;

    if (time_delta < time_per_frame && time_end > time_begin) {
        // Wenn in diesem Frame noch Zeit übrig ist: Warten
        delay((time_per_frame - time_delta) / 1000);
    }
}
```

## Timing auf dem Arduino (Anmerkung)

```cpp
if (time_delta < time_per_frame && time_end > time_begin)
```

Laut der Dokumentation läuft der Mikrosekunden-Zähler nach 70 Minuten über.

Daher wird mit ``time_end > time_begin`` sichergestellt, das der Zähler nicht gerade übergelaufen ist und daher das Ergebnis falsch ist.

## Timing auf dem Arduino (2. Anmerkung)

Wenn das Display angeschlossen ist, wird ``loop()`` ungefähr 25x pro Sekunde aufgerufen.

Ursache mit unklar, vermutlich eine Synchronisation mit dem Display.

Wir müssen also nicht manuell für das richtige Timing sorgen.

## Das heutige Ziel

![](Images/UML_Arduino.png)

## Nutzerinteraktionen verarbeiten

- Tastatur und Maus existieren beim Arduino nicht (aber lassen sich bestimmt auch anschließen)
- Stattdessen Taster und Sensoren
- Reaktion mit LEDs und Darstellung auf dem Display

## Eingabe

Bei Tastern gibt es drei wichtige Zustände:

- Key Down / Triggered: Die Taste wurde in diesem Frame gedrückt
- Pressed: Die Taste wird in diesem Frame weiterhin gedrückt
- Key Up / Released: Die Taste wurde in diesem Frame losgelassen

Die Klasse ``Button`` aus dem Praktikum implementiert ``triggered`` und ``pressed`` bereits.

# Danke für Ihre Aufmerksamkeit! Fragen?
