---
title: "Prozedurale Programmierung -- Eine Spiele-Engine mit SDL2"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Juni 2022
---

## Definition: Spiele-Engine (laut Wikipedia)

- Laufzeitumgebung für Computerspiele
- Steuert den Spielablauf und die visuelle Darstellung
- Besteht aus mehreren Komponenten
- Bekannte Engines:
    - Unreal Engine
    - Unity Engine
    - Godot (Open Source)
    - Blender (3D-Grafikprogramm mit integrierter Engine, Open Source)

## SDL2 als Spiele-Engine

- SDL2 ist keine Spiele-Engine
- SDL2 ist eine Abstraktion über Betriebssystem-Funktionen:
    - Video (Grafikausgabe)
    - Audio
    - Eingabe (Maus/Tastatur/Gamepad)
- Der Rest muss selbst programmiert werden
- Geringe Systemanforderungen
- Perfekt für die Lehre: Man wird nicht von Funktionalität "erschlagen"

## Wie erstelle ich ein Computerspiel

Ein Computerspiel ist quasi "nur" eine Endlosschleife und Prozessierung von Benutzereingaben und die Reaktion auf diese.

Empfohlenes Video: [Become A Game Developer In 5 Minutes Or Less - Mathieu Ropert - CppCon 2020](https://www.youtube.com/watch?v=H4N8xYue_3Q)

## Der Game-Loop

1. Nutzerinteraktionen verarbeiten
2. Logik / Physik aktualisieren
3. Rendern
4. Goto 1, bis Programm geschlossen wird

## Logik / Physik aktualisieren

Logikberechnungen finden in der Regel mit einer konstanten Updaterate statt, z. B. 60 Updates je Sekunde (UPS).

Die konstante Updaterate stellt sicher, dass das Spiel unabhängig von der Prozessorgeschwindigkeit läuft.

UPS ist keine offizielle Abkürzung.

## Logik / Physik aktualisieren (2)

Hier werden alle Objekte der Spielszene aktualisiert

- Spielfigur bewegen
- Objekte anhand ihrer Beschleunigung weiterbewegen
- Partikel aktualisieren
- Siegbedingungen prüfen
- …

## Rendern

Beim Rendern wird die Geometrie, Sprites usw. berechnet.

Das Rendern ist normalerweise an die Bildwiederholrate des Monitors gekoppelt (Vertikale Synchronisation / V-Sync).

Normalerweise 60 Hz, Gaming-Monitore mittlerweile 144 Hz.

Der Einfachheit fassen halber gehen wir von 60 Bilder pro Sekunde aus und ignorieren Lag.

## Definition: Sprite

- Sprites sind Grafikobjekte, die auf dem Bildschirm dargestellt werden
- Operationen auf Sprites ("Affine Transformation"):
    - Bewegung
    - Rotation
    - Zoom
    - Kippen, Strecken, Stauchen usw.

## Definition: Lag / Frameskip

- Tritt auf, wenn das System nicht schnell genug den Frame darstellen kann (16 ms bei 60 FPS).
- Update und Rendering laufen unabhängig voneinander
- Frame wird ausgelassen (Frameskip) und nicht gerendert, Updates laufen mit 60 UPS weiter
- Wenn keine 60 UPS möglich: Spiel wird langsamer

## Timing in C++: Header ``<chrono>``

```cpp
// 16666 Mikrosekunden pro Frame
const auto time_per_frame = std::chrono::microseconds(1000000 / 60);
for (;;) {
    // Startzeit der Schleife ermitteln
    auto time_begin = std::chrono::steady_clock::now();

    // Event Handling/Logik/Rendering/etc...

    // Ermitteln, wie lange die Schleife ausgeführt wurde
    auto time_end = std::chrono::steady_clock::now();
    auto time_delta = time_end - time_begin;

    if (time_delta < time_per_frame) {
        // Wenn in diesem Frame noch Zeit übrig ist: Warten
        std::this_thread::sleep_for(time_per_frame - time_delta);
    }
}
```

## Das heutige Ziel

![](Images/UML.png)

## Nutzerinteraktionen verarbeiten

Nutzerinteraktionen sind die essenzielle Komponente für jede Art von Software.

Die Eingabe über die Tastatur haben wir bereits auf der Konsole verwendet.

Bei grafischen Benutzeroberflächen reagiert man auf _Events_.

## Events

Grafische Benutzeroberflächen sind eventbasiert: Events werden an den Prozess gesendet und dieser verarbeitet diese in seinem _Event-Loop_

Die Eventverarbeitung in SDL2 erfolgt mit ``SDL_PollEvent``.

- Keyboard Event: Tastendruck
- Mouse Motion/Button/Wheel Event: Mausaktivitäten
- Touch Event: Bildschirmberührung
- Window Event: Änderungen des Fensters (Größe, Position)
- Quit Event: Programmende angefordert

## Eingabe

Bei der Eingabe gibt es drei wichtige Zustände je Taste:

- Key Down / Triggered: Die Taste wurde in diesem Frame gedrückt
- Pressed (kein Event): Die Taste wird in diesem Frame weiterhin gedrückt
- Key Up / Released: Die Taste wurde in diesem Frame losgelassen

Pressed wird nicht von SDL2 als Event gemeldet, ist aber sehr nützlich um z. B. Spielerbewegungen zu implementieren.

# Danke für Ihre Aufmerksamkeit! Fragen?
