#include "sdl_helper.h"

#include <string>
#include <iostream>
#include <stdexcept>

SDL_Helper::SDL_Helper() {
    /* SDL initialisieren */
    width = 320;
    height = 240;

    // Schaltet den Compositor unter Linux nicht aus (verhindert auf manchen Systemen flimmern beim Start)
    SDL_SetHint(SDL_HINT_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR, "0");

    // Alle Komponenten von SDL initialisieren
    // siehe https://wiki.libsdl.org/SDL_Init
    if (SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        throw std::runtime_error(std::string("SDL_Init fehlgeschlagen ") + SDL_GetError());
    }

    // Fenster erstellen, siehe https://wiki.libsdl.org/SDL_CreateWindow
    // Parameter:
    // - Text in Titelleiste
    // - X & Y Position (Zahl in Pixeln oder Konstante)
    // - Fensterbreite & -hoehe in Pixeln
    // - Flags für den Fenstermanager
    window.reset(SDL_CreateWindow(
        "SDL Demo",
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        width, height,
        SDL_WINDOW_RESIZABLE));

    if (!window) {
        throw std::runtime_error(std::string("SDL_CreateWindow fehlgeschlagen ") + SDL_GetError());
    }

    // Renderer erstellen, siehe https://wiki.libsdl.org/SDL_CreateRenderer
    // Ein Renderer wird zum Zeichnen in ein Fenster verwendet
    // Parameter:
    // - SDL_Window handle von oben
    // - Zu verwendenter Treiber (-1 = Den ersten funktionierenden verwenden)
    // - Flags: Zusaetzliche Optionen fuer den Treiber. Normalerweise 0
    renderer.reset(SDL_CreateRenderer(window.get(), -1, SDL_RENDERER_PRESENTVSYNC));

    if (!renderer) {
        throw std::runtime_error(std::string("SDL_CreateRenderer fehlgeschlagen ") + SDL_GetError());
    }

    // Schwarzes Fenster zeichnen
	SDL_RenderClear(renderer.get());
	SDL_RenderPresent(renderer.get());

    // Automatisch skalieren, wenn sich Fenstergroesse aendert
	SDL_RenderSetLogicalSize(renderer.get(), width, height);
}

SDL_Helper::~SDL_Helper() {
    // Beim beenden SDL_Quit aufrufen
    SDL_Quit();
}

SDL_Renderer* SDL_Helper::get_renderer() const {
    return renderer.get();
}

int SDL_Helper::get_width() const {
    return width;
}

int SDL_Helper::get_height() const {
    return height;
}
