#include "engine.h"
#include "rectangle.h"

#include <cmath>
#include <iostream>

Engine::Engine(SDL_Renderer* renderer, int window_width, int window_height) {
    this->renderer = renderer;
    this->window_width = window_width;
    this->window_height = window_height;

    rect = Rectangle(Point(100, 100), 20, 20);
    rect.set_color(255, 0, 0);
    rect.velocity = Point(0, 0);
}

bool Engine::process_events() {
    // Auf Nutzereingaben reagieren
    // siehe https://wiki.libsdl.org/SDL_Event
    SDL_Event evt;

    while (SDL_PollEvent(&evt)) {
        // Jeweils ein Event verarbeiten
        if (evt.type == SDL_QUIT) {
        // Benutzer hat Programmende angefordert (Fenster geschlossen)
            return 0;
        } else if (evt.type == SDL_KEYDOWN) {
            // Tastatur-Taste gedrückt
            if (evt.key.keysym.scancode == SDL_SCANCODE_A) {
                rect.velocity.x--;
            } else if (evt.key.keysym.scancode == SDL_SCANCODE_S) {
                rect.velocity.y++;
            } else if (evt.key.keysym.scancode == SDL_SCANCODE_W) {
                rect.velocity.y--;
            } else if (evt.key.keysym.scancode == SDL_SCANCODE_D) {
                rect.velocity.x++;
            }
        } else if (evt.type == SDL_KEYUP) {
            // Tastatur-Taste losgelassen
        } else if (evt.type == SDL_MOUSEBUTTONDOWN) {
            // Maustaste gedrückt
            if (evt.button.button == 1) {
                int x = evt.button.x - (rect.position.x + rect.width / 2);
                int y = evt.button.y - (rect.position.y + rect.height / 2);
                auto angle = atan2(y, x);
                rect.velocity.x = cos(angle) * 2;
                rect.velocity.y = sin(angle) * 2;
            }
        } else if (evt.type == SDL_MOUSEBUTTONUP) {
            // Maustaste losgelassen
        } else if (evt.type == SDL_MOUSEMOTION) {
            // Maus im Fenster bewegt
        }
    }

    return true;
}

void Engine::update_and_render() {
    // Fenster schwarz ausmalen
    SDL_SetRenderDrawColor(renderer, 0x0, 0x0, 0x0, 0xFF);
    SDL_RenderClear(renderer);

    /* Eigener Code hier */
    /*SDL_Rect rect = { 0, 0, 100, 50 };
    SDL_SetRenderDrawColor(renderer, 0xFF, 0x0, 0x0, 0xFF);
    SDL_RenderFillRect(renderer, &rect);*/

    rect.update();
    rect.draw(renderer);

    if (rect.position.x + rect.width > window_width || rect.position.x < 0) {
        rect.velocity.x = -rect.velocity.x;
    }

    if (rect.position.y + rect.height > window_height || rect.position.y < 0) {
        rect.velocity.y = -rect.velocity.y;
    }

    // Aenderungen sichtbar machen
    SDL_RenderPresent(renderer);
}
