---
title: "Prozedurale Programmierung -- Arduino: Rastern, Kommunikation und Interrupts"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Juni 2022
---

## Das heutige Ziel

![](Images/UML_Arduino.png)

## Nutzerinteraktionen verarbeiten

- Tastatur und Maus existieren beim Arduino nicht (aber lassen sich bestimmt auch anschließen)
- Stattdessen Taster und Sensoren
- Reaktion mit LEDs und Darstellung auf dem Display

## Eingabe

Bei Tastern gibt es drei wichtige Zustände:

- Key Down / Triggered: Die Taste wurde in diesem Frame gedrückt
- Pressed: Die Taste wird in diesem Frame weiterhin gedrückt
- Key Up / Released: Die Taste wurde in diesem Frame losgelassen

Die Klasse ``Button`` aus dem Praktikum implementiert ``triggered`` und ``pressed`` bereits.

## Pixelart

:::::: {.columns}
::: {.column}

- Rastergrafik
- Verwendet beschränkte Anzahl von Pixeln als Stilmittel
- Wird manuell gepixelt
- Meist für niedrige Auflösungen, bei hohen verwendet man 3D-Grafik-Programme oder Zeichenstift

:::
::: {.column}

![Ein Alien aus Spaceinvaders](Images/Alien.png)

:::
::::::

## Rastern von Pixelart

- Ermitteln aller Koordinaten, wo gezeichnet werden muss
- Abspeichern als zwei Arrays (x und y)

## Herz

:::::: {.columns}
::: {.column}

![Das stark vergrößerte Pixelraster](Images/Arduino_Gitter.png)

:::
::: {.column}

![Das fertig gerasterte Herz](Images/Arduino_Herz.png)

:::
::::::

## Serielle Eingabe von Daten

Der Arduino kann über die Serielle Schnittstelle des Computer mit Daten versorgt werden.

Senden von Daten (Linux): ``echo -n "Hallo" > /dev/ttcACM0``

Lesen von Daten: Mit ``Serial.available()`` prüfen und mit ``Serial.read()`` oder ``std::cin`` lesen.

## Interrupts

- Mit Interrupts kann der Programmfluss unterbrochen werden
- Diese werden in der Regel hardwareseitig ausgelöst (Hardware-Interrupt)
- Interrupts ermöglichen das schnelle reagieren auf Ereignisse.
- Interrupts müssen so kurz wie möglich sein
- Während eines Interrupts sind Interrupts gesperrt
- Zu viele Interrupts verlangsamen den Prozessor erheblich (Modern Prozessoren haben eine lange _Pipeline_, kodieren also nachfolgende Befehle und führen andere derweil aus, bei einem Interrupt muss die Pipeline verworfen werden)

## Interrupts auf dem Arduino

``attachInterrupt(digitalPinToInterrupt(pin), ISR, mode)``

``ISR``: Name einer Funktion mit Signatur ``void name()``

Funktioniert nicht für alle Pins (siehe Dokumentation)

Mögliche Optionen für ``mode``:

- ``LOW``: Pin ist ``LOW``
- ``CHANGE``: Pin ändert sich
- ``RISING``: Von ``LOW`` auf ``HIGH``
- ``FALLING``: Von ``HIGH`` auf ``LOW``

## Interrupts auf dem Arduino (2)

Zusätzlich gibt es noch Timer-Interrupts.

Diese lösen in festen Intervallen aus.

Sind deutlich hässlicher zu programmieren.

Machen wir nächste Woche.

# Danke für Ihre Aufmerksamkeit! Fragen?
