#include <Arduino.h>
#include <ArduinoSTL.h>

#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include "button.h"
#include "circle.h"
#include "led.h"
#include "heart.h"
#include "pins_arduino.h"

/** Hier können globale Variablen definiert werden. */
const int width = 128;
const int height = 64;
Adafruit_SSD1306 display(width, height);

Led rot;
Button button;
std::vector<Circle> kreise;
Heart heart;

int zaehler = 0;
int read = 0;

/** Wird nach der Hardware-Initialisierung einmalig aufgerufen */
void setup() {
    // Konsolenausgabe aktivieren
    Serial.begin(9600);

    std::cout << "Hallo Welt!\n";

    rot = Led(2);
    button = Button(8);

    rot.on();

    // Heapspeicher für 10 Kreise vorreservieren
    kreise.reserve(10);

    kreise.push_back(Circle(Point(10, 10), 5));
    kreise.push_back(Circle(Point(20, 5), 3));
    kreise.push_back(Circle(Point(30, 40), 10));

    kreise[0].velocity = Point(1, 1); // Diagonal unten rechts
    kreise[1].velocity = Point(3, 0); // Nach rechts
    kreise[2].velocity = Point(0, 1); // Nach unten

    heart = Heart(Point(50, 40));
    heart.velocity = Point(2, 3);

    // Display initialisieren
    if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
        std::cout << "SSD1306 fehler!\n";
    }

    delay(1000);
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 0);
    display.print("Hallo Welt!");
    display.display();

    rot.off();
}

/** Wird in einer Endlosschleife aufgerufen */
void loop() {
    ++zaehler;

    display.clearDisplay();
    display.setCursor(0, 0);
    display.print("Hallo Welt!");

    display.setCursor(0, 16);
    display.print(read);

    display.setCursor(0, 32);
    display.print(zaehler);

    button.update();

    if (button.is_pressed()) {
        rot.on();
        display.setCursor(0, 32);
        display.print("Taste gedrueckt!");
    } else {
        rot.off();
    }

    if (button.is_triggered()) {
        for (auto& kreis: kreise) {
            kreis.position.move_to(10, 10);
        }
    }

    for (auto& kreis: kreise) {
        /*display.drawCircle(
            kreis.top_left.x + kreis.get_radius() + zaehler,
            kreis.top_left.y + kreis.get_radius() + zaehler,
            kreis.get_radius(),
            WHITE);*/
        kreis.draw(display);
        kreis.update();

        // Wenn die Kreise den Rand berühren: Richtung umkehren
        if (kreis.position.x + kreis.get_diameter() >= width) {
            kreis.velocity.x = -kreis.velocity.x;
        } else if (kreis.position.x < 0) {
            kreis.velocity.x = -kreis.velocity.x;
        }

        if (kreis.position.y + kreis.get_diameter() >= height) {
            kreis.velocity.y = -kreis.velocity.y;
        } else if (kreis.position.y < 0) {
            kreis.velocity.y = -kreis.velocity.y;
        }
    }

    heart.draw(display);
    heart.update();

    if (heart.position.x + heart.width >= width) {
        heart.velocity.x = -heart.velocity.x;
    } else if (heart.position.x < 0) {
        heart.velocity.x = -heart.velocity.x;
    }

    if (heart.position.y + heart.width >= height) {
        heart.velocity.y = -heart.velocity.y;
    } else if (heart.position.y < 0) {
        heart.velocity.y = -heart.velocity.y;
    }

    display.display();

    if (Serial.available()) {
        read = Serial.read();
    }

    // Pin 3: Interrupt ausloesen, wenn 3 gedrueckt
    // std::algorithm (lambda-Funktionen []() {})
    attachInterrupt(digitalPinToInterrupt(3), []() {
        zaehler = 0;
    }, RISING);

    // Andere Art von Interrupts: Timer-Interrupts (nächste Woche)
}
