#include <Arduino.h>
#include <ArduinoSTL.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include "button.h"
#include "circle.h"
#include "led.h"
#include "heart.h"

/** Hier können globale Variablen definiert werden. */
const int width = 128;
const int height = 64;
Adafruit_SSD1306 display(width, height, &Wire, -1);

int counter = 0;
long time_per_frame = 1000000 / 60;
int last_fps = 0;
int fps = 0;
long time_last_fps = 0;

std::vector<Circle> circles;
std::vector<Heart> hearts;

Led red;
Button more;
Button less;
Button more_heart;
Button less_heart;

/** Wird nach der Hardware-Initialisierung einmalig aufgerufen */
void setup() {
    // Konsolenausgabe aktivieren
    Serial.begin(9600);

    std::cout << "Hallo Welt!\n";

    red = Led(2);
    red.on();

    more = Button(8);
    less = Button(9);
    more_heart = Button(10);
    less_heart = Button(11);

    // Display initialisieren
    if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
        Serial.println("SSD1306 allocation failed");
        for(;;);
    }

    delay(1000);
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);

    time_last_fps = micros();

    circles.reserve(20);
    circles.push_back(Circle({0, 0}, 5));
    circles.back().velocity = {2, 2};

    hearts.reserve(20);

    red.off();
}

/** Wird in einer Endlosschleife aufgerufen */
void loop() {
    // Startzeit vom loop-Aufruf ermitteln
    long time_begin = micros();

    display.clearDisplay();

    //red.on();

    /* Eigener Code hier */
    display.setCursor(0, 0);
    display.print(last_fps);

    if (time_begin - time_last_fps > 1000000) {
        time_last_fps = time_begin;
        last_fps = fps;
        fps = 0;
    }

    for (auto& circle: circles) {
        circle.update();
        circle.draw(display);

        if (circle.position.x + circle.get_diameter() > width || circle.position.x < 0) {
            circle.velocity.x = -circle.velocity.x;
        }

        if (circle.position.y + circle.get_diameter() > height || circle.position.y < 0) {
            circle.velocity.y = -circle.velocity.y;
        }
    }

    for (auto& heart: hearts) {
        heart.update();
        heart.draw(display);

        if (heart.position.x + heart.width > width || heart.position.x < 0) {
            heart.velocity.x = -heart.velocity.x;
        }

        if (heart.position.y + heart.width > height || heart.position.y < 0) {
            heart.velocity.y = -heart.velocity.y;
        }
    }

    more.update();
    less.update();
    more_heart.update();
    less_heart.update();
    if (more.is_triggered()) {
        circles.push_back(Circle({(double)random(10, width - 20), (double)random(10, height - 20)}, (double)random(3, 10)));
        circles.back().velocity = {(double)random(1, 6), (double)random(1, 6)};
    }

    if (less.is_triggered() && !circles.empty()) {
        circles.pop_back();
    }

    if (more_heart.is_triggered()) {
        hearts.push_back(Heart({(double)random(10, width - 20), (double)random(10, height - 20)}, random(9, 11)));
        hearts.back().velocity = {(double)random(1, 6), (double)random(1, 6)};
    }

    if (less_heart.is_triggered() && !hearts.empty()) {
        hearts.pop_back();
    }

    if (more.is_pressed() || less.is_pressed() || more_heart.is_pressed() || less_heart.is_pressed()) {
        red.on();
    } else {
        red.off();
    }

    // Ermitteln, wie lange die Schleife ausgeführt wurde
    ++fps;

    //red.off();

    display.display();
/*
    long time_end = micros();
    long time_delta = time_end - time_begin;

    if (time_delta < time_per_frame && time_end > time_begin) {
        // Wenn in diesem Frame noch Zeit übrig ist: Warten
        delayMicroseconds(time_delta);
    }
*/
}
