---
title: "Prozedurale Programmierung -- Eventbehandlung mit SDL2"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Juni 2022
---

## Nutzerinteraktionen verarbeiten

Nutzerinteraktionen sind die essenzielle Komponente für jede Art von Software.

Die Eingabe über die Tastatur haben wir bereits auf der Konsole verwendet.

Bei grafischen Benutzeroberflächen reagiert man auf _Events_.

## Events

Grafische Benutzeroberflächen sind eventbasiert: Events werden an den Prozess gesendet und dieser verarbeitet diese in seinem _Event-Loop_

Die Eventverarbeitung in SDL2 erfolgt mit ``SDL_PollEvent``.

```cpp
SDL_Event evt;

while (SDL_PollEvent(&evt)) {
    // Jeweils ein Event verarbeiten
    if (evt.type == SDL_QUIT) {
        // Benutzer hat Programmende angefordert (Fenster geschlossen)
        return false;
    }
}

return true;
```

## Arten von Events

- Keyboard Event: Tastendruck
- Mouse Motion/Button/Wheel Event: Mausaktivitäten
- Touch Event: Bildschirmberührung
- Controller Added/Removed/Motion/Button Event: Gamecontroller
- Window Event: Änderungen des Fensters (Größe, Position, Fokus)
- Quit Event: Programmende angefordert
- Und noch viele mehr (siehe <https://wiki.libsdl.org/SDL_Event>)

## Eingabe

Bei der Eingabe gibt es drei wichtige Zustände je Taste:

- Key Down / Triggered: Die Taste wurde in diesem Frame gedrückt
- Pressed (kein Event): Die Taste wird in diesem Frame weiterhin gedrückt
- Key Up / Released: Die Taste wurde in diesem Frame losgelassen

Pressed wird nicht von SDL2 als Event gemeldet, ist aber sehr nützlich um z. B. Spielerbewegungen zu implementieren.

# Danke für Ihre Aufmerksamkeit! Fragen?
