#include "circle.h"

#include <stdexcept>

Circle::Circle() {
    radius = 1.0;
    position = Point(-1, -1);
}

Circle::Circle(Point top_left, double radius) {
    this->position = top_left;
    set_radius(radius);
}

double Circle::get_radius() const {
    return radius;
}

void Circle::set_radius(double radius) {
    if (radius <= 0) {
        throw std::runtime_error("radius <= 0 ist unzulässig");
    }

    this->radius = radius;
}

double Circle::get_diameter() const {
    return 2 * radius;
}

Point Circle::get_center() const {
    Point mittelpunkt(
        position.x + radius,
        position.y + radius
    );

    return mittelpunkt;
}

bool Circle::is_inside(const Point& p) const {
    Point mittelpunkt = get_center();

    return (mittelpunkt.distance_to(p) < radius);
}


void Circle::draw(SDL_Renderer* renderer) {
    int x = position.x + radius;
    int y = position.y + radius;

    int offsetx, offsety, d;
    offsetx = 0;
    offsety = radius;
    d = radius -1;

    // Zeichenfarbe setzen
    SDL_SetRenderDrawColor(renderer, red, green, blue, 0x0);

    while (offsety >= offsetx) {
        SDL_RenderDrawLine(renderer, x - offsety, y + offsetx,
                                     x + offsety, y + offsetx);
        SDL_RenderDrawLine(renderer, x - offsetx, y + offsety,
                                     x + offsetx, y + offsety);
        SDL_RenderDrawLine(renderer, x - offsetx, y - offsety,
                                     x + offsetx, y - offsety);
        SDL_RenderDrawLine(renderer, x - offsety, y - offsetx,
                                     x + offsety, y - offsetx);

        if (d >= 2*offsetx) {
            d -= 2*offsetx + 1;
            offsetx +=1;
        }
        else if (d < 2 * (radius - offsety)) {
            d += 2 * offsety - 1;
            offsety -= 1;
        }
        else {
            d += 2 * (offsety - offsetx - 1);
            offsety -= 1;
            offsetx += 1;
        }
    }
}

