#include "board.h"
#include "rectangle.h"

#include <ArduinoSTL.h>

Board::Board() {
    snake.push_back(Point(1, 1));
    snake.push_back(Point(2, 1));
    snake.push_back(Point(3, 1));
    snake.push_back(Point(4, 1));

    // Links, Oben, Unten, Rechts
    button_left = Button(8);
    button_up = Button(3);
    button_down = Button(10);
    button_right = Button(11);

    // Zufallszahlengenerator seeden
    // (funktioniert nicht richtig)
    // Idee: Ports lesen, wo nichts dransteckt (Rauschen)
    // 16 Bit Zufall, analogRead liefert 8 Bit Zahl
    // 0000 0000 0000 0000
    // 3333 3333 2222 2222
    int a3 = analogRead(A3);
    int a2 = analogRead(A2);

    Serial.println(a2);
    Serial.println(a3);

    randomSeed((a3 << 8) + a2);

    // Futter platzieren
    place_food();
}

void Board::draw(Adafruit_SSD1306& display) {
    for (Point& p: snake) {
        // Idee erstmal alle Schlangenkomponenten nicht ausmalen
        Rectangle rect(
            Point(p.x * rect_dim, p.y * rect_dim),
            rect_dim, rect_dim);
        rect.fill = false;
        rect.draw(display);
    }

    // Idee: Der Kopf ist immer das letzte Element im Vector
    Point& kopf = snake.back();
    Rectangle rkopf(
        Point(kopf.x * rect_dim, kopf.y * rect_dim),
        rect_dim, rect_dim);
    rkopf.draw(display);

    Circle food_circle(Point(food.x * rect_dim, food.y * rect_dim), rect_dim / 2.0);
    food_circle.draw(display);

    if (game_over) {
        // Annahme (6px pro Buchstaben inkl. Leerraum)
        int text_width = 9 * 6;
        int text_height = 6;

        display.setCursor(
            128 / 2 - text_width / 2,
            64 / 2 - text_height / 2);
        display.print("Game Over");

        // Tickzähler erhöht sich etwa 25x pro Sekunde
        if (tick % 25 == 0) {
            display.invertDisplay(display_invert);
            display_invert = !display_invert;
        }
    }
}

void Board::update() {
    ++tick;

    if (game_over) {
        return;
    }

    button_left.update();
    button_up.update();
    button_down.update();
    button_right.update();

    if (button_left.is_triggered()) {
        direction = left;
    } else if (button_up.is_triggered()) {
        direction = up;
    } else if (button_down.is_triggered()) {
        direction = down;
    } else if (button_right.is_triggered()) {
        direction = right;
    }

    // Serielle Eingabe lesen
    if (Serial.available()) {
        char ch;
        std::cin >> ch;

        if (ch == 'u') {
            direction = up;
        } else if (ch == 'd') {
            direction = down;
        } else if (ch == 'l') {
            direction = left;
        } else if (ch == 'r') {
            direction = right;
        } else if (ch == 'f') {
            place_food();
        }
    }

    // Soll die Schlange wieder bewegt werden?
    if (tick < ticks_per_move) {
        return;
    }

    // Schlange bewegen
    tick = 0;

    Point& old_head = snake.back();

    // Neues Kopfsegment anfügen
    if (direction == left) {
        snake.push_back(Point(old_head.x - 1, old_head.y));
    } else if (direction == up) {
        snake.push_back(Point(old_head.x, old_head.y - 1));
    } else if (direction == down) {
        snake.push_back(Point(old_head.x, old_head.y + 1));
    } else if (direction == right) {
        snake.push_back(Point(old_head.x + 1, old_head.y));
    }

    Point& head = snake.back();

    // Spielfeldrand prüfen
    if (head.x < 0 || head.x >= field_width) {
        game_over = true;
    }

    if (head.y < 0 || head.y >= field_height) {
        game_over = true;
    }

    // Schwanz gefressen
    for (int i = 0; i < snake.size() - 1; ++i) {
        // Schlangensegmente außer Kopf durchlaufen
        if (head.x == snake[i].x && head.y == snake[i].y) {
            game_over = true;
        }
    }

    // Auf Futter prüfen
    if (head.x == food.x && head.y == food.y) {
        place_food();
        ticks_per_move -= 0.25;
    } else {
        // Letzte Schwanzsegment entfernen
        snake.erase(snake.begin());
    }
}

void Board::place_food() {
    int x;
    int y;

    bool again = true;
    while (again) {
        again = false;

        x = random(0, field_width);
        y = random(0, field_height);

        // Verhindern, dass Essen auf der Schlange erzeugt wird
        for (Point& p: snake) {
            if (x == p.x && y == p.y) {
                again = true;
            }
        }
    }

    food = Point(x, y);
}
