#include <Arduino.h>
#include <ArduinoSTL.h>

#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include "button.h"
#include "circle.h"
#include "led.h"
#include "board.h"

/** Hier können globale Variablen definiert werden. */
const int width = 128;
const int height = 64;
Adafruit_SSD1306 display(width, height);

Led rot;
Board board;

bool rot_status = false;

ISR(TIMER1_OVF_vect) {
    if (rot_status) {
        rot.off();
    } else {
        rot.on();
    }
    rot_status = !rot_status;
    TCNT1 = 0xFFFF / 2;
}

/** Wird nach der Hardware-Initialisierung einmalig aufgerufen */
void setup() {
    // Konsolenausgabe aktivieren
    Serial.begin(9600);

    TCCR1A = 0x00; // OC2A and OC2B disconnected; Wave Form Generator: Normal Mode
    TCCR1B = (1<<CS12) | (1<<CS10); // prescaler = 256; alternative: 1024 (set CS12 and CS10)
    TIMSK1 = (1<<TOIE1); // interrupt when TCNT1 is overflowed
    TCNT1 = 0xFFFF / 2;

    rot = Led(2);
    rot.on();

    // Display initialisieren
    if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
        std::cout << "SSD1306 fehler!\n";
    }

    delay(1000);
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 0);
    display.print("Hallo Welt!");
    display.display();

    rot.off();
}

/** Wird in einer Endlosschleife aufgerufen */
void loop() {
    //rot.pulse();

    display.clearDisplay();

    board.update();
    board.draw(display);

    display.display();
}
