#include "board.h"
#include "rectangle.h"

#include <random>

Board::Board() {
    // Festlegung:
    // Das letzte Element im Vektor ist der Kopf (Rot)
    // Alle anderen Elemente sind der Schwanz
    snake.push_back(Point(1, 1));
    snake.push_back(Point(2, 1));
    snake.push_back(Point(3, 1));
    snake.push_back(Point(4, 1));

    place_food();

    game_over = false;
}

void Board::draw(SDL_Renderer* renderer) {
    // Eigener draw-Code hier
    if (game_over) {
        SDL_SetRenderDrawColor(renderer, 250, 170, 170, 0xFF);
        SDL_RenderClear(renderer);
    }

    // Durchlaufen der Schlange und zeichnen
    for (Point& p: snake) {
        Rectangle rect(
            Point(p.x * rect_dim, p.y * rect_dim),
            rect_dim, rect_dim);

        rect.draw(renderer);
    }

    // Kopf rot zeichnen
    Point& kopf = snake.back();
    Rectangle rect(
        Point(kopf.x * rect_dim, kopf.y * rect_dim),
        rect_dim, rect_dim);
    rect.set_color(255, 0, 0);
    rect.draw(renderer);

    // Blaues Futter zeichnen
    Rectangle food_rect(
        Point(food.x * rect_dim, food.y * rect_dim),
        rect_dim, rect_dim);
    food_rect.set_color(0, 0, 255);
    food_rect.draw(renderer);

    // Gitter zeichnen
    for (int i = 0; i <= field_dim; ++i) {
        Rectangle rect(Point(i * rect_dim, 0), 1, 720);
        Rectangle rect2(Point(0, i * rect_dim), 720, 1);
        rect.set_color(128, 128, 128);
        rect2.set_color(128, 128, 128);
        rect.draw(renderer);
        rect2.draw(renderer);
    }
}

void Board::update_field() {
    tick++;

    // Ist genug Zeit vergangen?
    if (tick < ticks_per_move) {
        return;
    }

    if (game_over) {
        return;
    }

    tick = 0;

    Point& kopf = snake.back();

    if (direction == right) {
        snake.push_back(Point(kopf.x + 1, kopf.y));
    } else if (direction == left) {
        snake.push_back(Point(kopf.x - 1, kopf.y));
    } else if (direction == up) {
        snake.push_back(Point(kopf.x, kopf.y - 1));
    } else if (direction == down) {
        snake.push_back(Point(kopf.x, kopf.y + 1));
    }

    // Prüfen ob Futter
    Point& neuer_kopf = snake.back();
    if (neuer_kopf.x == food.x && neuer_kopf.y == food.y) {
        // Essen :)
        place_food();
        ticks_per_move -= 2;
    } else {
        // Letztes Schwanzelement entfernen
        snake.erase(snake.begin());
    }

    // Prüfen, ob Spielfeldrand berührt
    if (neuer_kopf.x >= field_dim || neuer_kopf.x < 0) {
        // Feld links oder rechts verlassen
        game_over = true;
    } else if (neuer_kopf.y >= field_dim || neuer_kopf.y < 0) {
        // Feld oben oder unten verlassen
        game_over = true;
    }

    // Prüfen, ob sich Schlange gebissen hat
    /*for (Point& p: snake) {
        // Bug: Hier wird auch der Kopf mit dem Kopf verglichen
        if (p.x == neuer_kopf.x && p.y == neuer_kopf.y) {
            game_over = true;
        }
    }*/
    // Schlange ohne Kopf
    for (int i = 0; i < snake.size() - 1; ++i) {
        Point p = snake[i];
        if (p.x == neuer_kopf.x && p.y == neuer_kopf.y) {
            game_over = true;
        }
    }
}

void Board::place_food() {
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<int> dist(0, field_dim - 1);

    for (;;) {
        int x = dist(gen);
        int y = dist(gen);
        food = Point(x, y);

        // Prüfen, ob Futter auf Schlange
        bool auf_schlange = false;
        for (Point& p: snake) {
            if (p.x == food.x && p.y == food.y) {
                auf_schlange = true;
            }
        }

        // Wenn auf schlange -> Schleife nochmal ausführen, sonst
        // abbrechen
        if (!auf_schlange) {
            break;
        }
    }
}

void Board::set_direction(int new_direction) {
    direction = new_direction;
}
