#include "engine.h"

#include <SDL_gamecontroller.h>
#include <iostream>

Engine::Engine(SDL_Renderer* renderer, int window_width, int window_height) {
    this->renderer = renderer;
    this->window_width = window_width;
    this->window_height = window_height;
}

bool Engine::process_events() {
    SDL_Event evt;

    while (SDL_PollEvent(&evt)) {
        // Jeweils ein Event verarbeiten
        if (evt.type == SDL_QUIT) {
            // Benutzer hat Programmende angefordert (Fenster geschlossen)
            return false;
        } else if (evt.type == SDL_KEYDOWN) {
            if (evt.key.keysym.scancode == SDL_SCANCODE_W) {
                board.set_direction(board.up);
            } else if (evt.key.keysym.scancode == SDL_SCANCODE_S) {
                board.set_direction(board.down);
            } else if (evt.key.keysym.scancode == SDL_SCANCODE_A) {
                board.set_direction(board.left);
            } else if (evt.key.keysym.scancode == SDL_SCANCODE_D) {
                board.set_direction(board.right);
            }
        } else if (evt.type == SDL_CONTROLLERDEVICEADDED) {
            // Controller wurde angesteckt
            int id = evt.cdevice.which;
            if (SDL_IsGameController(id)) {
                SDL_GameController* controller = SDL_GameControllerOpen(id);
                if (controller) {
                    std::cout << "Controller added: " << SDL_GameControllerName(controller) << "\n";
                }
            }
        } else if (evt.type == SDL_CONTROLLERDEVICEREMOVED) {
            int id = evt.cdevice.which;
            SDL_GameController* controller = SDL_GameControllerFromInstanceID(id);
            if (controller) {
                std::cout << "Controller removed: " << SDL_GameControllerName(controller) << "\n";
                SDL_GameControllerClose(controller);
            }
        } else if (evt.type == SDL_CONTROLLERBUTTONDOWN) {
            // Gegenteil: SDL_CONTROLLERBUTTONUP
            if (evt.cbutton.button == SDL_CONTROLLER_BUTTON_Y) {
                board.set_direction(board.up);
            } else if (evt.cbutton.button == SDL_CONTROLLER_BUTTON_A) {
                board.set_direction(board.down);
            } else if (evt.cbutton.button == SDL_CONTROLLER_BUTTON_X) {
                board.set_direction(board.left);
            } else if (evt.cbutton.button == SDL_CONTROLLER_BUTTON_B) {
                board.set_direction(board.right);
            }
        }
    }

    return true;
}

void Engine::update_and_render() {
    // Fenster weiß ausmalen
    SDL_SetRenderDrawColor(renderer, 0xFF, 0xFF, 0xFF, 0xFF);
    SDL_RenderClear(renderer);

    /* Eigener Code hier */
    board.update_field();
    board.draw(renderer);

    // Aenderungen sichtbar machen
    SDL_RenderPresent(renderer);
}
