# Praktikum 12: Snake

Zum krönenden Abschluss werden wir nun noch [Snake](https://snake.googlemaps.com/) programmieren.

Bei Snake bewegt sich eine Schlange über ein Spielfeld auf der Suche nach ihrem Futter.

Jedes Mal, wenn die Schlange ihr Futter verspeist, wird diese um ein Feld länger und etwas schneller.

Das Spiel ist vorbei, wenn sich die Schlange selbst in den Schwanz beißt oder den Rand berührt.

## Erläuterung des gegebenen Codes

Diesmal ist bereits etwas Quelltext vorgegeben. Siehe dazu _Quellcode (Snake)_.

Im Gegensatz zum Game of Life, welches mit 10 Updates pro Sekunde lief, wird Snake mit 60 Updates pro Sekunde ausgeführt.

Dies liegt daran, dass die Geschwindigkeit dynamisch an den Schwierigkeitsgrad (wie viel wurde schon gefressen) angepasst werden soll.

Der Ansatz ist bereits in ``Board`` gegeben.

In ``draw`` wird schon das Gitter gezeichnet. Der Code entspricht dem aus Game of Life.

In ``update`` ist die Schlangenbewegung angedeutet: Dafür gibt es einen ``tick``-Zähler. Dieser wird 60x pro Sekunde erhöht (weil die Schleife in ``main`` mit 60 Updates läuft und daher ``board.update`` 60x aufgerufen wird). Sobald der ``tick``-Zähler den Schwellwert von ``ticks_per_move`` erreicht hat, wird er auf 0 zurückgesetzt und die Schlange soll sich bewegen. Am Anfang ist ``ticks_per_move`` 10, d. h. die Schlange bewegt sich 10x pro Sekunde. Um den Schwierigkeitsgrad zu erhöhen, soll nach jedem verzerrten Futter die Rate leicht reduziert werden.

## Die Schlange (``snake``)

Diesmal verwenden wir eine andere Datenstruktur als beim Game of Life. Das Feld selbst ist uns diesmal egal, da auf diesem nicht viel passiert. Stattdessen konzentrieren wir uns auf die Protagonisten: die Schlange und das Futter.

Die Schlange (``snake``) ist ein ``std::vector<Point>``.

Jeder ``Point`` im Vektor beschreibt ein Segment der Schlange. Der letzte ``Point`` ist der Schlangenkopf, alle anderen Elemente sind der Schwanz.

Verwenden Sie als Initialisierung im Konstruktor von ``Board`` folgenden Code. Dieser erzeugt eine 4-elementige Schlange mit Kopf bei ``(4, 1)``.

```cpp
snake.push_back(Point(1, 1));
snake.push_back(Point(2, 1));
snake.push_back(Point(3, 1));
snake.push_back(Point(4, 1));
```

## Zeichnen der Schlange in ``draw()``

Die Schlange soll wie abgebildet gezeichnet werden.

![](Images/Snake_Koordinaten.png)

Dazu wird der Vektor der Schlange durchlaufen und für jedes Segment der Schlange, soll ein Rechteck mit Höhe und Breite eines Feldes (also ``rect_dim``) gezeichnet werden.

Der Schwanz soll _schwarz_ sein und der Kopf _rot_.

Starten Sie das Programm. Wenn alles funktioniert, werden Sie eine unbewegliche Schlange entsprechend der vorherigen Abbildung sehen.

## Bewegen der Schlange

Im Header von ``Board`` befinden sich ebenfalls bereits vier Konstanten für die Bewegungsrichtung (``up``, ``down``, ``left`` und ``right``). Zudem ist mit ``direction = right`` auch schon die anfängliche Bewegungsrichtung vorgegeben.

Aber es fehlt natürlich noch der Code, um die Schlange wirklich zu bewegen.

Die Bewegung machen wir in ``update`` nach folgendem Ansatz:

Ermitteln Sie die Koordinaten des Kopfes (also des letzten Elements im Vektor). Fügen Sie an das Ende des Vektors ein neues Element ein. Dieses muss um ``x + 1`` relativ zum ursprünglichen Kopf verschoben sein. Da dieses neue Element das letzte Element im Vektor ist, wird es automatisch zum Kopf und der ursprüngliche Kopf wird zum Schwanz degradiert. Da der neue Kopf weiter rechts im Feld ist, wirkt es wie eine Bewegung. _Probieren Sie es aus_.

![Das bisherige Zwischenergebnis. Die Schlange ist ein Feld länger geworden.](Images/Snake_Rechts.png)

Die Schlange hat aber momentan unendliches Wachstum. Um dies zu lösen, entfernen wir einfach das hinterste Schwanzstück (also das erste Element im Vektor). Nun funktioniert die Bewegung auch schon korrekt. _Probieren Sie es aus_

![Die Schlange wurde wieder auf die Ursprungslänge gestutzt und hat sich ein Feld nach rechts bewegt.](Images/Snake_Rechts2.png)

## Die verbleibenden Bewegungsrichtungen

![Hier das UML-Diagramm mit allen notwendigen Änderungen](Images/UML.png)

Implementieren Sie zuerst in der Klasse ``Board`` eine neue Methode ``set_direction``. Diese ist ein Setter für die Richtung (``direction``).

Reagieren Sie nun im Event-Handler von ``Engine`` auf die Pfeiltasten und rufen Sie entsprechend ``set_direction`` mit der korrekten Bewegungsrichtung auf. Verwenden Sie dafür auch die Konstanten in ``board.h``.

Zudem muss nun noch in ``Board.update`` die Schlangenbewegung für die anderen Bewegungsrichtungen implementiert werden.

## Platzieren von Futter

Futter (``food``) wird ebenfalls wieder über seine Koordinaten (``Point``) beschrieben.

Die Platzierung von Futter soll in der Methode ``place_food`` erfolgen. In dieser Methode wird eine zufällige Koordinate solange ermittelt, bis das Feld frei ist (also keine Schlange auf dem Feld ist). Anschließend wird die Koordinate dem Futter zugewiesen.

Das Futter kann nun in ``draw`` auf die übliche Art gezeichnet werden. Die Farbe soll _blau_ sein.

Um das Futter bei Spielstart zu platzieren, rufen Sie im Konstruktor ``place_food`` auf.

![Schlange und Futter](Images/Snake_Futter.png)

## Verzehren von Futter

Prüfen Sie zum Verzehren in ``update_field``, ob die Koordinaten des Kopfes der Schlange den Koordinaten des Futters entsprechen.

Wenn dies der Fall ist, passieren die folgenden Dinge:

1. Das Futter wird mit ``place_food`` neu verortet
2. Die Schlange wird ein wenig schneller. Verringern Sie dazu ``ticks_per_move`` um ``0.5``. Dadurch wird die Schlange nach jedem zweiten Futter etwas schneller.
3. Verlängern Sie die Schlange um eins. Überspringen Sie dazu einmalig einfach den Code, der das letzte Schwanzsegment entfernt.

## Game Over

Bis jetzt läuft das Spiel unendlich, selbst wenn ein Fehler passiert.

Dies ist natürlich nicht Sinn der Sache.

In folgenden Situationen soll die neue Variable ``bool game_over`` auf ``true`` gesetzt werden:

- Der Kopf der Schlange berührt den Spielfeldrand
- Die Schlange beißt sich in den Schwanz

Wenn das Spiel vorbei ist, soll die Hintergrundfarbe des Feldes geändert werden (freie Farbwahl) und ``update_field`` darf nicht mehr ausgeführt werden, beispielsweise mit einem Abbruch direkt am Anfang, wenn ``game_over true`` ist.

![Da beißt sich die ~~Katze~~ Schlange in den Schwanz](Images/Snake_GameOver.png)
