---
title: "Prozedurale Programmierung -- Recap & Prüfung"
author: "Gabriel Kind"
institute: "Hochschule Mittweida"
date: Juli 2022
---

# Evaluierungsbögen

## Ablauf

- Fakultäten legen selber fest, welche Seminargruppen zu welchen Modulen befragt werden
- Dozent kann theoretisch auch Evaluation beantragen, war aber dieses Semester zu beschäftigt um mich darum zu kümmern

## Befragte Gruppe

- 47 Prüfungseinschreibungen
- Befragt wurden:
  - E-Technik (5 Antworten)
  - Lasertechnik (3 Antworten)
- Antwort also von 17%

## Umfrageergebnisse (E-Technik)

![](Images/EA.png)

## Umfrageergebnisse (E-Technik, 2)

![](Images/EA_2.png)

## Umfrageergebnisse (Lasertechnik)

![](Images/LT.png)

## Umfrageergebnisse (Lasertechnik, 2)

![](Images/LT_2.png)

## Lob (E-Technik)

- Arduinoeinbindung
- Besonders toll finde ich, dass, mit Einbeziehung der Arduinos bei den Elektrotechnikern, auch spezifische Fähigkeiten gelehrt werden, die für die jeweilige Fachrichtung auch später noch sehr relevant sind.
- Die Motivation und Umsetzung durch unseren Dozenten. Ihre Vorlesungen sind immer praxisbezogen, interessant und witzig. Sie sind immer freundlich, zuvorkommend, hilfsbereit und haben die nötige Geduld, eine Aufgabe auch zum 10. Mal zu erklären. Und vor allem haben Sie die wichtige Fähigkeit, auch trockene und uninteressante Themen spannend und lustig zu gestalten, was für die Motivation und den Lernerfolg der Studenten nicht unerheblich ist.

## Tadel (E-Technik)

- Mir war am Anfang der Vorlesung nicht ganz klar, wie weit mein Kenntnisstand am Ende des Moduls reichen soll. Ich würde es gut finden, wenn Sie am Anfang des Semesters eine Beispielaufgabe vorstellen könnten, die man am Ende in der Lage sein sollte zu lösen
- Persönlich bin ich nicht so angetan vom Flipped Classroom Konzept. Ich denke eine zum Teil interaktive Vorlesung könnte genauso effektiv sein. Dabei würde ich auf die Storyteller nicht verzichten, da diese sehr hilfreich sind. Stattdessen könnten man diese als Zusatz beibehalten.

## Lob (Lasertechnik)

- Storyteller haben geholfen
- Moodlekurs übersichtlich und informativ
- Videoaufnahmen der Vorlesungen und Quellcodes waren hilfreich
- Wir haben in den Praktika immer Bezug auf die Vorlesungen genommen und konnten damit eigene funktionierende Programme / Funktionen programmieren, die dann auch funktionieren und anwendbar sind

## Tadel (Lasertechnik)

- Am Ende wurde es komplizierter und damit schwieriger nachzuvollziehen. Im Praktikum ein wenig mehr begleitendes Programmieren statt nur auf sich allein gestellt zu sein zumindest das Grundgerüst beim Thema "Objektorientierte Prog." oder "SDL2"

# Rückblick

## Literatur

- <https://en.cppreference.com>: DIE C++ Referenz
- <https://stackoverflow.com>: Antworten auf fast alle Programmierprobleme
- Falls Sie Bücher mögen: Gottschling, P. (2019): "FORSCHUNG mit
modernem C++: C++17-INTENSIVKURS für Wissenschaftler,
Ingenieure und Programmierer". Hanser Verlag

## Disclaimer

Keine Garantie auf Vollständigkeit (!)

## Was ist Prüfungsrelevant?

Eigentlich fast alles.

Was auf keinen Fall prüfungsrelevant ist, hat ein (*)

## V01 & P01: Einführung

- Geschichte von C++ (*)
- Einrichtung Visual Studio Code
- Grundlegende Syntax und ein erstes Beispielprogramm

## V02 & P02: Grundlagen

- Grundlegende Syntax
- ``main``-Funktion
- Textein- und Ausgabe
- Zahlen und deren Typen
- Verzweigungen mit ``if``
- Theorie: Bedeutung Stack und Heap (*)
- Theorie: Bedienung des Debuggers (*, aber hilfreich bei der Fehlersuche)

## V03 & P03: Datentypen und Datenstrukturen

- Mehrere Bedingungen gleichzeitig in ``if`` (``&&``/``||``)
- Theorie: Codeformatierung (Einrückung etc.)
- Theorie: Komplexitätsklassen (*)
- Strings und deren Funktionen
- Vektoren und deren Funktionen
- ``foreach``-Schleifen

## V04 & P04: Funktionen und Projektstruktur

- Funktionen (Syntax und Implementierung)
- ``for``- & ``while``-Schleife
- Projektstruktur (``.cpp``/``.h``, Include Guards)
- Quelldateien zu ``CMakeLists.txt`` hinzufügen
- ``const``
- Referenzen (``&``)
- Zufallszahlen erzeugen

## V05 & P05: Algorithmen

- Rekursive Funktionen
- Funktionen aus ``<algorithm>``
- Lambda-Funktionen
- Template-Funktionen (*)
- Theorie & Praxis: Iteratoren (Pointer)
- Theorie: Interne Datenstruktur von ``std::vector`` u. a. (*)
- Theorie: Sortierverfahren (*, außer ``std::(stable)_sort``)

## V06 & P06: Dateiverarbeitung

- Lesen und schreiben von einfachen Textdateien
- Verarbeiten von Kommandozeilenparametern
- Lesen von komplexeren Textformaten wie CSV (*)
- Lesen und schreiben von Binärdateien (*)

## V07, V08, P07, P08, P09: Objektorientierte Programmierung

- Hier wurden die theoretischen Grundlagen der Modellierung eingeführt
- UML: Nur Klassendiagramme; Rest (*)
- Objektorientierte Programmierung
    - Klassen
    - Konstruktoren
    - Felder
    - Methoden
    - Getter und Setter
    - Zugriffsrechte (``public``, ``private``, ``protected``)
    - Vererbung
    - Abstrakte Klassen (*)

## V07, V08, P07, P08, P09: Objektorientierte Programmierung (2)

- Anhand einer Beschreibung und eines UML-Diagramms Software entwickeln
- Objekte können auf anderen Objekten aufbauen (Komposition)
- Erstellen eigener UML-Diagramme
    - Siehe dazu auch die Videos bei P07
    - Modellierung (Erstellen von UML aus Text)
    - Erstellen von UML-Diagrammen mit yEd

## V08 & P09: Ausnahmebehandlung

- Behandeln von Fehlern mit ``try`` & ``catch``
- Werfen von Ausnahmen mit ``throw`` (nur ``std::runtime_error``)

## V09: Speicherverwaltung (*)

- Arten von Speicherverwaltung
- Stackbasiert
- Heapbasiert (manuell mit ``new``)
- Heapbasiert (automatisch mit Smart Pointern)

Muss nicht selbst programmiert werden, kann aber in gegebenem Quelltext vorkommen.

## SDL2 bzw. Arduino

Hier wurde dann thematisch gesplittet.

- Elektrotechnik: Arduino
- Rest: SDL2

## V10: SDL2 & Computergrafik

- Theorie: Raster- und Vektorgrafik (*)
- SDL2
    - Theorie: Was ist SDL2 (*)
    - Andere Komponenten von SDL2 (Image, Mixer, TTF) (*)
    - Fenster erstellen (Code wird vorgegeben)
    - Rendern im Fenster
- Bresenham-Algorithmen (*)

## P10: SDL2 kennenlernen

- Zeichnen von einfachen, geometrischen Objekten
- Setzen von Farben
- Einfache Animationen (DVD-Logo)

## V11: Spiele-Engine mit SDL2

- Hier wurde hergeleitet, wie mit OOP und SDL2 eine einfache Spiele-Engine erstellt werden kann
- Basierend auf dieser Engine können nun weitere Funktionalitäten hinzugefügt werden
- Die Engine ist die Basis für alle weiteren Projekte gewesen

## P11: Game of Life

- Implementierung von Game of Life
- Programmieren eines komplexeren Spiels in SDL2
- Umwandlung UML-Klassendiagramm und Beschreibung in Quelltext
- Umgang mit zweidimensionalen Feldern

## V12 & P12: SDL2 (Eventbehandlung)

- Behandeln von verschiedenen Events
- Tastatureingabe
- Maussteuerung
- Game Controller u. a. (*)

## V13: Snake

- Erstellen eines Spiels am Beispiel von Snake.
- Snake muss nicht explizit drankommen.
- Es kann ein beliebiges, anderes in SDL2 erstelltes Spiel oder Programm sein.

## P09: Einrichtung Arduino

- Anschließen von Komponenten an den Arduino
- Programmieren des Arduino

## P10: Objektorientierung auf dem Arduino

- Hier wurde hergeleitet, wie mit OOP die Programmierung auf dem Arduino erleichtert werden kann
- Entworfen haben wir ``LED`` und ``Button``
- Effekte (Lichterkette, Pulsieren)
- Reaktion auf Tastendrücke

## V11 & P11: Display-Ansteuerung und 2D-Grafik

- Rendern auf dem ``SSD1306`` mit Adafruit
- Rastern von Grafiken
- Initialisierungscode wird vorgegeben

## V12 & P12: Rastern und Interrupts

- Hier wurde hergeleitet, wie mit OOP auf dem Arduino eine einfache Spiele-Engine erstellt werden kann
- Basierend auf dieser Engine können nun weitere Funktionalitäten hinzugefügt werden
- Die Engine ist die Basis für alle weiteren Projekte gewesen
- Interrupt-Behandlung (Aufruf von Funktionen bei steigender/fallender Spannung)

## V13 & P13: Timer-Interrupts, Serielle Kommunikation & Snake

- Timer-Interrupts (*)
- Erstellen eines Spiels auf dem Arduino am Beispiel von Snake.
- Reaktion auf Tastendrücke
- Textdarstellung auf dem Display
- Steuerung der Schlange über die Kommandozeile (Serielle Kommunikation, *)
- Snake muss nicht explizit drankommen
- Es kann ein beliebiges, anderes auf dem Arduino lauffähiges Spiel oder Programm sein

## Fehlende Inhalte

- Zu SDL2 und Arduino fehlen noch Storyteller
- Bei der Einrichtung fehlen noch einige Erweiterungen und Einstellungen, die sich während des Semesters ergeben haben
- Reiche ich bis zum Anfang der zweiten Prüfungswoche nach

# Prüfung

## Gibt es eine Probeklausur?

Nein.

In der Prüfung werden keine neuen Konzepte eingeführt.

Mit dem Wissen aus Vorlesungen und Praktika sind alle Aufgaben lösbar (Transferwissen).

## Beispiel für Transferwissen

:::::: {.columns}
::: {.column}

Angenommen, wir haben folgende Aufgabe besprochen:

Eine Person ist zwischen 30 und 50 Jahren alt:

```cpp
if (person.getAge() >= 30 &&
    person.getAge() <= 50)
```

:::
::: {.column}

:::
::::::

## Beispiel für Transferwissen (2)

:::::: {.columns}
::: {.column}

Angenommen, wir haben folgende Aufgabe besprochen:

Eine Person ist zwischen 30 und 50 Jahren alt:

```cpp
if (person.getAge() >= 30 &&
    person.getAge() <= 50)
```

:::
::: {.column}

Dann können Sie auch nachfolgende Aufgabe lösen:

Ein Auto hat 3 oder 5 Türen:

```
if (auto.getDoors() == 3 ||
    auto.getDoors() == 5)
```

Beide Aufgaben sind prinzipiell das Gleiche Konzept

:::
::::::

## Vortag der Prüfung (Checkliste)

- C++ Compiler installiert
- Visual Studio Code installiert und konfiguriert
- C++-Projekte (CMake) lassen sich kompilieren und ausführen
- Programm für UML-Diagramme installieren (z. B. yEd)
- SDL2:
  - ``vcpkg`` installiert
  - SDL2-Projekte lassen sich kompilieren und ausführen
- Arduino:
  - PlatformIO installiert
  - Arduino-Projekte können kompiliert und geflasht werden
  - Alle Komponenten verkabelt und getestet (welche konkret teile ich noch mit)

## Prüfungsformat

- _Eigentlich_ ist die Prüfung 90 Min. schriftlich
- Programmieren unter Stress in 90 Min. ist problematisch und schränkt die Aufgaben stark ein
- Daher: Take-Home Exam

## Prüfungsformat

- Take-Home Exam
- Sie bekommen, inklusive Abgabezeit und Mittagspause, 7 Stunden (!)
- Übliche Bearbeitungszeit wird 2 - 3 Stunden sein, Rest ist Puffer
- Unüberwacht
- Alle Hilfsmittel erlaubt, außer _aktive_ Kommunikation (Reden mit anderen, Fragen in Foren, Chats, usw. stellen, Aufgaben von anderen Personen bearbeiten lassen)

## Quellenangaben

Wenn Sie größere Mengen Quelltext aus dem Internet übernehmen (ab 4 Zeilen), die Quelle in einem Kommentar angeben

Ausnahme: Jeglicher Code von mir

## Raumbuchungen

Falls Sie lieber in der Hochschule sein möchten, sind folgende Räume reserviert:

- 6-024
- 6-101
- 8-102: Exklusiv für E-Technik (wegen Arduino)

## Prüfungsablauf

- 8:00: Identifikation mit Ausweis oder HSMW-Card (**Kamera notwendig**) ODER
- 8:20: Identifikation vor Ort, Treffpunkt wird noch mitgeteilt
- 8:35: Belehrung (Hybrid)
- 8:50: Zurückkehren an den Arbeitsplatz
- 9:00: Prüfungsbeginn
- 12:00: Mittagspause (1h)
- Ab 16:00: Abgabefunktion wird geschlossen

## Prüfungsablauf (2)

Als Prüfungsportal wird wieder OPAL (HSMW Exam) verwendet. Link findet sich dann auf Moodle.

Dort bekommen Sie eine Prüfung zugewürfelt (bekannt von der Grundlagen IT-Prüfung) und Sie laden dann an gleicher Stelle Ihre Antworten hoch.

## Bei Fragen

- Den Großteil der Prüfung bin ich über Zoom-Chat und Telefon (03727 58 1013) erreichbar
- Ich helfe nur bei Problemen mit HSMW-Exam und beantworte Verständnisfragen zu den Prüfungsaufgaben
- Keine Hilfestellungen und kein IT-Support ("mein C++ Compiler geht nicht")
- Ausnahme: Home-Bereich voll (2 GB)
- Für kaputte Arduino habe ich Ersatzteile, aber nur, wenn nicht selbst verschuldet (z.B. OLED-Display gegrillt, weil 5V und Minuspol vertauscht)

## Prüfungsstruktur

:::::: {.columns}
::: {.column}

- Jeder bekommt zwei Teilprüfungen
- 1. Teilprüfung: C++ allgemein
- 2. Teilprüfung: Wird basierend auf Seminargruppe zugeordnet
    - Arduino (EA21 und auf Anfrage)
    - SDL2 (alle Anderen)

:::
::: {.column}

![Jeder sieht alle drei Prüfungen, aber Leserechte gibt es nur für zwei](Images/Exam.png)

:::
::::::

## Prüfungsstruktur (2)

- Prüfung ist noch nicht fertig
- Voraussichtlicher Aufbau:
    - 4 bis 5 Aufgaben
    - ca. 25% OOP
    - ca. 25% SDL2/Arduino (Enthält prinzipbedingt auch OOP)
    - Wenige Theorieaufgaben
- 40% ist bestanden

## Stilpunkte

- Da Sie viel Bearbeitungszeit haben, gibt es Punktabzug für:
    - Schlampige Codeformatierung
    - Compilerwarnungen
    - Schwerwiegend: Compilerfehler
- Kommentare sind nicht notwendig (außer wenn in der Aufgabe gefordert)

## Formatierungsbeispiel

:::::: {.columns}
::: {.column}

OKAY:

```cpp
int main(int argc, char** argv) {
    if (argc >= 2) {
        std::cout << "Hallo\n";
    }

    return 0;
}
```

:::
::: {.column}

OKAY:

```cpp
int main(int argc, char** argv)
{
    if (argc >= 2)
    {
        std::cout << "Hallo\n";
    }

    return 0;
}
```

:::
::::::

## Negativbeispiel

![Beispiel für schlecht formatierten Code ist alles auf funduino.de](Images/Funduino.png)

## Compilerwarnungen (Sehen Sie die Probleme?)

:::::: {.columns}
::: {.column}

```cpp
int funktion(int a, int b) {
    return a + 3;
}
```

:::
::: {.column}

```cpp
int funktion(int a, int b) {
    int b = 10;

    return a + 3;
}
```

:::
::::::

## Compilerwarnungen (2)

:::::: {.columns}
::: {.column}

Warnung: ``b`` wird nicht verwendet

```cpp
int funktion(int a, int b) {
    return a + 3;
}
```

:::
::: {.column}

Warnung: ``b`` wird geschrieben, aber nicht gelesen

```cpp
int funktion(int a, int b) {
    int b = 10;

    return a + 3;
}
```

:::
::::::

## Compilerfehler (Sehen Sie die Fehler?)

```cpp
class Point {
public:
    Point();
    Point();

    get_x() const;
    double get_y();

    void move_by;
    void move_to(a: int, b: int);
}
```

## Compilerfehler (2)

```cpp
class Point {
public:
    Point();
    Point(); // Fehler: Konstruktor doppelt

    get_x() const; // Fehler: Rückgabetyp fehlt
    double get_y(); // Unschön, kein Fehler: const fehlt

    void move_by; // Fehler: Parameterliste fehlt
    void move_to(a: int, b: int); // Fehler: int a, nicht a: int
} // Fehler: Semikolon fehlt
```

## Abschließende Worte

Schauen Sie sich einfach nochmal alles an.

Bitte nicht schummeln.

Ich programmiere seit 15 Jahren C++.

Ich sehe, wenn es nicht von Ihnen ist.

Viel Erfolg, so schwer ist es nicht. :)

# Danke für Ihre Aufmerksamkeit! Fragen?
